@@ -129,9 +129,8 @@ void Bind::setDeclSpecifiers(Symbol *symbol, const FullySpecifiedType &declSpeci
     symbol->setStorage(storage);
 
     if (Function *funTy = symbol->asFunction()) {
-        if (declSpecifiers.isVirtual()) {
+        if (declSpecifiers.isVirtual())
             funTy->setVirtual(true);
-        }
     }
 
     if (declSpecifiers.isDeprecated())
@@ -182,6 +181,13 @@ void Bind::operator()(DeclarationAST *ast, Scope *scope)
     (void) switchScope(previousScope);
 }
 
+void Bind::operator()(StatementAST *ast, Scope *scope)
+{
+    Scope *previousScope = switchScope(scope);
+    statement(ast);
+    (void) switchScope(previousScope);
+}
+
 FullySpecifiedType Bind::operator()(ExpressionAST *ast, Scope *scope)
 {
     Scope *previousScope = switchScope(scope);
@@ -466,9 +472,8 @@ void Bind::enumerator(EnumeratorAST *ast, Enum *symbol)
         EnumeratorDeclaration *e = control()->newEnumeratorDeclaration(ast->identifier_token, name);
         e->setType(control()->integerType(IntegerType::Int)); // ### introduce IntegerType::Enumerator
 
-        if (ExpressionAST *expr = ast->expression) {
+        if (ExpressionAST *expr = ast->expression)
             e->setConstantValue(asStringLiteral(expr->firstToken(), expr->lastToken()));
-        }
 
         symbol->addMember(e);
     }
@@ -517,9 +522,7 @@ void Bind::memInitializer(MemInitializerAST *ast, Function *fun)
     /*const Name *name =*/ this->name(ast->name);
 
     Scope *previousScope = switchScope(fun);
-    for (ExpressionListAST *it = ast->expression_list; it; it = it->next) {
-        /*ExpressionTy value =*/ this->expression(it->value);
-    }
+    this->expression(ast->expression);
     (void) switchScope(previousScope);
 }
 
@@ -539,14 +542,18 @@ const Name *Bind::nestedNameSpecifier(NestedNameSpecifierAST *ast)
     return class_or_namespace_name;
 }
 
-bool Bind::visit(NewPlacementAST *ast)
+bool Bind::visit(ExpressionListParenAST *ast)
 {
-    (void) ast;
-    assert(!"unreachable");
+    // unsigned lparen_token = ast->lparen_token;
+    for (ExpressionListAST *it = ast->expression_list; it; it = it->next) {
+        /*ExpressionTy value =*/ this->expression(it->value);
+    }
+    // unsigned rparen_token = ast->rparen_token;
+
     return false;
 }
 
-void Bind::newPlacement(NewPlacementAST *ast)
+void Bind::newPlacement(ExpressionListParenAST *ast)
 {
     if (! ast)
         return;
@@ -578,23 +585,6 @@ FullySpecifiedType Bind::newArrayDeclarator(NewArrayDeclaratorAST *ast, const Fu
     return type;
 }
 
-bool Bind::visit(NewInitializerAST *ast)
-{
-    (void) ast;
-    assert(!"unreachable");
-    return false;
-}
-
-void Bind::newInitializer(NewInitializerAST *ast)
-{
-    if (! ast)
-        return;
-
-    // unsigned lparen_token = ast->lparen_token;
-    ExpressionTy expression = this->expression(ast->expression);
-    // unsigned rparen_token = ast->rparen_token;
-}
-
 bool Bind::visit(NewTypeIdAST *ast)
 {
     (void) ast;
@@ -1084,6 +1074,7 @@ void Bind::capture(CaptureAST *ast)
     if (! ast)
         return;
 
+    name(ast->identifier);
 }
 
 bool Bind::visit(LambdaDeclaratorAST *ast)
@@ -1330,7 +1321,6 @@ bool Bind::visit(RangeBasedForStatementAST *ast)
         block->addMember(decl);
     }
 
-    /*ExpressionTy initializer =*/ this->expression(ast->initializer);
     /*ExpressionTy expression =*/ this->expression(ast->expression);
     this->statement(ast->statement);
     (void) switchScope(previousScope);
@@ -1636,7 +1626,7 @@ bool Bind::visit(NewExpressionAST *ast)
     ExpressionTy type_id = this->expression(ast->type_id);
     // unsigned rparen_token = ast->rparen_token;
     this->newTypeId(ast->new_type_id);
-    this->newInitializer(ast->new_initializer);
+    this->expression(ast->new_initializer);
     return false;
 }
 
@@ -1653,11 +1643,7 @@ bool Bind::visit(TypenameCallExpressionAST *ast)
 {
     // unsigned typename_token = ast->typename_token;
     /*const Name *name =*/ this->name(ast->name);
-    // unsigned lparen_token = ast->lparen_token;
-    for (ExpressionListAST *it = ast->expression_list; it; it = it->next) {
-        ExpressionTy value = this->expression(it->value);
-    }
-    // unsigned rparen_token = ast->rparen_token;
+    this->expression(ast->expression);
     return false;
 }
 
@@ -1667,11 +1653,7 @@ bool Bind::visit(TypeConstructorCallAST *ast)
     for (SpecifierListAST *it = ast->type_specifier_list; it; it = it->next) {
         type = this->specifier(it->value, type);
     }
-    // unsigned lparen_token = ast->lparen_token;
-    for (ExpressionListAST *it = ast->expression_list; it; it = it->next) {
-        ExpressionTy value = this->expression(it->value);
-    }
-    // unsigned rparen_token = ast->rparen_token;
+    this->expression(ast->expression);
     return false;
 }
 
@@ -1804,7 +1786,7 @@ bool Bind::visit(BracedInitializerAST *ast)
 {
     // unsigned lbrace_token = ast->lbrace_token;
     for (ExpressionListAST *it = ast->expression_list; it; it = it->next) {
-        ExpressionTy value = this->expression(it->value);
+        /*ExpressionTy value =*/ this->expression(it->value);
     }
     // unsigned comma_token = ast->comma_token;
     // unsigned rbrace_token = ast->rbrace_token;
@@ -1868,9 +1850,8 @@ bool Bind::visit(SimpleDeclarationAST *ast)
 
         const Name *declName = 0;
         unsigned sourceLocation = location(it->value, ast->firstToken());
-        if (declaratorId && declaratorId->name) {
+        if (declaratorId && declaratorId->name)
             declName = declaratorId->name->name;
-        }
 
         Declaration *decl = control()->newDeclaration(sourceLocation, declName);
         decl->setType(declTy);
@@ -1901,7 +1882,10 @@ bool Bind::visit(SimpleDeclarationAST *ast)
             if (Function *funTy = decl->type()->asFunctionType()) {
                 funTy->setMethodKey(methodKey);
 
-                if (funTy->isVirtual() && it->value->equal_token)
+                bool pureVirtualInit = it->value->equal_token
+                        && it->value->initializer
+                        && it->value->initializer->asNumericLiteral();
+                if (funTy->isVirtual() && pureVirtualInit)
                     funTy->setPureVirtual(true);
             }
         }
@@ -2655,10 +2639,14 @@ bool Bind::visit(TemplateIdAST *ast)
     }
 
     const Identifier *id = identifier(ast->identifier_token);
+    const int tokenKindBeforeIdentifier(translationUnit()->tokenKind(ast->identifier_token - 1));
+    const bool isSpecialization = (tokenKindBeforeIdentifier == T_CLASS ||
+                                   tokenKindBeforeIdentifier == T_STRUCT);
     if (templateArguments.empty())
-        _name = control()->templateNameId(id);
+        _name = control()->templateNameId(id, isSpecialization);
     else
-        _name = control()->templateNameId(id, &templateArguments[0], templateArguments.size());
+        _name = control()->templateNameId(id, isSpecialization, &templateArguments[0],
+                templateArguments.size());
 
     ast->name = _name;
     return false;
@@ -2985,6 +2973,8 @@ bool Bind::visit(EnumSpecifierAST *ast)
     Enum *e = control()->newEnum(sourceLocation, enumName);
     e->setStartOffset(tokenAt(sourceLocation).end()); // at the end of the enum or identifier token.
     e->setEndOffset(tokenAt(ast->lastToken() - 1).end());
+    if (ast->key_token)
+        e->setScoped(true);
     ast->symbol = e;
     _scope->addMember(e);
 
