@@ -1,20 +1,19 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** This file is part of Qt Creator.
 **
-** Contact:  Qt Software Information (qt-info@nokia.com)
-**
-** Commercial Usage
-**
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
-**
 ** Alternatively, this file may be used under the terms of the GNU Lesser
 ** General Public License version 2.1 as published by the Free Software
 ** Foundation and appearing in the file LICENSE.LGPL included in the
@@ -22,24 +21,26 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** If you are unsure which license is appropriate for your use, please
-** contact the sales department at qt-sales@nokia.com.
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
+** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-**************************************************************************/
+****************************************************************************/
 
 #include "pluginspec.h"
-#include "pluginspec.h"
+
 #include "pluginspec_p.h"
 #include "iplugin.h"
 #include "iplugin_p.h"
 #include "pluginmanager.h"
 
-#include <QtCore/QFile>
-#include <QtCore/QFileInfo>
-#include <QtCore/QXmlStreamReader>
-#include <QtCore/QRegExp>
-#include <QtCore/QCoreApplication>
-#include <QtDebug>
+#include <QDir>
+#include <QFile>
+#include <QFileInfo>
+#include <QXmlStreamReader>
+#include <QRegExp>
+#include <QCoreApplication>
+#include <QDebug>
 
 #ifdef Q_OS_LINUX
 // Using the patched version breaks on Fedora 10, KDE4.2.2/Qt4.5.
@@ -50,7 +51,7 @@
 
 #if USE_UNPATCHED_QPLUGINLOADER
 
-#   include <QtCore/QPluginLoader>
+#   include <QPluginLoader>
     typedef QT_PREPEND_NAMESPACE(QPluginLoader) PluginLoader;
 
 #else
@@ -82,13 +83,30 @@
     Version string that a plugin must match to fill this dependency.
 */
 
+/*!
+    \variable ExtensionSystem::PluginDependency::type
+    Defines whether the dependency is required or optional.
+    \sa ExtensionSystem::PluginDependency::Type
+*/
+
+/*!
+    \enum ExtensionSystem::PluginDependency::Type
+    Whether the dependency is required or optional.
+    \value Required
+           Dependency needs to be there.
+    \value Optional
+           Dependency is not necessarily needed. You need to make sure that
+           the plugin is able to load without this dependency installed, so
+           for example you may not link to the dependency's library.
+*/
+
 /*!
     \class ExtensionSystem::PluginSpec
     \brief Contains the information of the plugins xml description file and
     information about the plugin's current state.
 
     The plugin spec is also filled with more information as the plugin
-    goes through it's loading process (see PluginSpec::State).
+    goes through its loading process (see PluginSpec::State).
     If an error occurs, the plugin spec is the place to look for the
     error details.
 */
@@ -102,7 +120,7 @@
     \value  Invalid
             Starting point: Even the xml description file was not read.
     \value  Read
-            The xml description file has been successfully read, and it's
+            The xml description file has been successfully read, and its
             information is available via the PluginSpec.
     \value  Resolved
             The dependencies given in the description file have been
@@ -118,20 +136,30 @@
             extensionsInitialized has been called. The loading process is
             complete.
     \value Stopped
-            The plugin has been shut down, i.e. the plugin's IPlugin::shutdown() method has been called.
+            The plugin has been shut down, i.e. the plugin's IPlugin::aboutToShutdown() method has been called.
     \value Deleted
             The plugin instance has been deleted.
 */
+
 using namespace ExtensionSystem;
 using namespace ExtensionSystem::Internal;
 
 /*!
-    \fn bool PluginDependency::operator==(const PluginDependency &other)
+    \fn uint qHash(const ExtensionSystem::PluginDependency &value)
     \internal
 */
-bool PluginDependency::operator==(const PluginDependency &other)
+uint ExtensionSystem::qHash(const ExtensionSystem::PluginDependency &value)
 {
-    return name == other.name && version == other.version;
+    return qHash(value.name);
+}
+
+/*!
+    \fn bool PluginDependency::operator==(const PluginDependency &other) const
+    \internal
+*/
+bool PluginDependency::operator==(const PluginDependency &other) const
+{
+    return name == other.name && version == other.version && type == other.type;
 }
 
 /*!
@@ -225,6 +253,86 @@ QString PluginSpec::url() const
     return d->url;
 }
 
+/*!
+    \fn QString PluginSpec::category() const
+    The category that the plugin belongs to. Categories are groups of plugins which allow for keeping them together in the UI.
+    Returns an empty string if the plugin does not belong to a category.
+*/
+QString PluginSpec::category() const
+{
+    return d->category;
+}
+
+/*!
+    \fn bool PluginSpec::isExperimental() const
+    Returns if the plugin has its experimental flag set.
+*/
+bool PluginSpec::isExperimental() const
+{
+    return d->experimental;
+}
+
+/*!
+    Returns if the plugin is disabled by default.
+    This might be because the plugin is experimental, or because
+    the plugin manager's settings define it as disabled by default.
+*/
+bool PluginSpec::isDisabledByDefault() const
+{
+    return d->disabledByDefault;
+}
+
+/*!
+    \fn bool PluginSpec::isEnabledInSettings() const
+    Returns if the plugin should be loaded at startup. True by default
+    The user can change it from the Plugin settings.
+    Note: That this function returns true even if a plugin is disabled because
+    of a not loaded dependencies, or a error in loading.
+*/
+bool PluginSpec::isEnabledInSettings() const
+{
+    return d->enabledInSettings;
+}
+
+/*!
+    \fn bool PluginSpec::isEffectivelyEnabled() const
+    Returns if the plugin is loaded at startup.
+    \see PluginSpec::isEnabled
+*/
+bool PluginSpec::isEffectivelyEnabled() const
+{
+    return !d->disabledIndirectly
+            && (d->enabledInSettings || d->forceEnabled)
+            && !d->forceDisabled;
+}
+
+/*!
+    \fn bool PluginSpec::isDisabledIndirectly() const
+    Returns true if loading was not done due to user unselecting this plugin or its dependencies.
+*/
+bool PluginSpec::isDisabledIndirectly() const
+{
+    return d->disabledIndirectly;
+}
+
+/*!
+    \fn bool PluginSpec::isForceEnabled() const
+    Returns if the plugin is enabled via the -load option on the command line.
+*/
+bool PluginSpec::isForceEnabled() const
+{
+    return d->forceEnabled;
+}
+
+/*!
+    \fn bool PluginSpec::isForceDisabled() const
+    Returns if the plugin is disabled via the -noload option on the command line.
+*/
+bool PluginSpec::isForceDisabled() const
+{
+    return d->forceDisabled;
+}
+
 /*!
     \fn QList<PluginDependency> PluginSpec::dependencies() const
     The plugin dependencies. This is valid after the PluginSpec::Read state is reached.
@@ -352,7 +460,7 @@ IPlugin *PluginSpec::plugin() const
 
     \sa PluginSpec::dependencies()
 */
-QList<PluginSpec *> PluginSpec::dependencySpecs() const
+QHash<PluginDependency, PluginSpec *> PluginSpec::dependencySpecs() const
 {
     return d->dependencySpecs;
 }
@@ -360,30 +468,43 @@ QList<PluginSpec *> PluginSpec::dependencySpecs() const
 //==========PluginSpecPrivate==================
 
 namespace {
-    const char * const PLUGIN = "plugin";
-    const char * const PLUGIN_NAME = "name";
-    const char * const PLUGIN_VERSION = "version";
-    const char * const PLUGIN_COMPATVERSION = "compatVersion";
-    const char * const VENDOR = "vendor";
-    const char * const COPYRIGHT = "copyright";
-    const char * const LICENSE = "license";
-    const char * const DESCRIPTION = "description";
-    const char * const URL = "url";
-    const char * const DEPENDENCYLIST = "dependencyList";
-    const char * const DEPENDENCY = "dependency";
-    const char * const DEPENDENCY_NAME = "name";
-    const char * const DEPENDENCY_VERSION = "version";
-    const char * const ARGUMENTLIST = "argumentList";
-    const char * const ARGUMENT = "argument";
-    const char * const ARGUMENT_NAME = "name";
-    const char * const ARGUMENT_PARAMETER = "parameter";
+    const char PLUGIN[] = "plugin";
+    const char PLUGIN_NAME[] = "name";
+    const char PLUGIN_VERSION[] = "version";
+    const char PLUGIN_COMPATVERSION[] = "compatVersion";
+    const char PLUGIN_EXPERIMENTAL[] = "experimental";
+    const char PLUGIN_DISABLED_BY_DEFAULT[] = "disabledByDefault";
+    const char VENDOR[] = "vendor";
+    const char COPYRIGHT[] = "copyright";
+    const char LICENSE[] = "license";
+    const char DESCRIPTION[] = "description";
+    const char URL[] = "url";
+    const char CATEGORY[] = "category";
+    const char DEPENDENCYLIST[] = "dependencyList";
+    const char DEPENDENCY[] = "dependency";
+    const char DEPENDENCY_NAME[] = "name";
+    const char DEPENDENCY_VERSION[] = "version";
+    const char DEPENDENCY_TYPE[] = "type";
+    const char DEPENDENCY_TYPE_SOFT[] = "optional";
+    const char DEPENDENCY_TYPE_HARD[] = "required";
+    const char ARGUMENTLIST[] = "argumentList";
+    const char ARGUMENT[] = "argument";
+    const char ARGUMENT_NAME[] = "name";
+    const char ARGUMENT_PARAMETER[] = "parameter";
 }
 /*!
     \fn PluginSpecPrivate::PluginSpecPrivate(PluginSpec *spec)
     \internal
 */
 PluginSpecPrivate::PluginSpecPrivate(PluginSpec *spec)
-    : plugin(0),
+    :
+    experimental(false),
+    disabledByDefault(false),
+    enabledInSettings(true),
+    disabledIndirectly(false),
+    forceEnabled(false),
+    forceDisabled(false),
+    plugin(0),
     state(PluginSpec::Invalid),
     hasError(false),
     q(spec)
@@ -404,17 +525,17 @@ bool PluginSpecPrivate::read(const QString &fileName)
         = license
         = description
         = url
+        = category
         = location
-        = "";
+        = QString();
     state = PluginSpec::Invalid;
     hasError = false;
-    errorString = "";
+    errorString.clear();
     dependencies.clear();
     QFile file(fileName);
-    if (!file.exists())
-        return reportError(tr("File does not exist: %1").arg(file.fileName()));
     if (!file.open(QIODevice::ReadOnly))
-        return reportError(tr("Could not open file for read: %1").arg(file.fileName()));
+        return reportError(tr("Cannot open file %1 for reading: %2")
+                           .arg(QDir::toNativeSeparators(file.fileName()), file.errorString()));
     QFileInfo fileInfo(file);
     location = fileInfo.absolutePath();
     filePath = fileInfo.absoluteFilePath();
@@ -431,7 +552,7 @@ bool PluginSpecPrivate::read(const QString &fileName)
     }
     if (reader.hasError())
         return reportError(tr("Error parsing file %1: %2, at line %3, column %4")
-                .arg(file.fileName())
+                .arg(QDir::toNativeSeparators(file.fileName()))
                 .arg(reader.errorString())
                 .arg(reader.lineNumber())
                 .arg(reader.columnNumber()));
@@ -439,6 +560,35 @@ bool PluginSpecPrivate::read(const QString &fileName)
     return true;
 }
 
+void PluginSpec::setEnabled(bool value)
+{
+    d->enabledInSettings = value;
+}
+
+void PluginSpec::setDisabledByDefault(bool value)
+{
+    d->disabledByDefault = value;
+}
+
+void PluginSpec::setDisabledIndirectly(bool value)
+{
+    d->disabledIndirectly = value;
+}
+
+void PluginSpec::setForceEnabled(bool value)
+{
+    d->forceEnabled = value;
+    if (value)
+        d->forceDisabled = false;
+}
+
+void PluginSpec::setForceDisabled(bool value)
+{
+    if (value)
+        d->forceEnabled = false;
+    d->forceDisabled = value;
+}
+
 /*!
     \fn bool PluginSpecPrivate::reportError(const QString &err)
     \internal
@@ -457,7 +607,7 @@ static inline QString msgAttributeMissing(const char *elt, const char *attribute
 
 static inline QString msgInvalidFormat(const char *content)
 {
-    return QCoreApplication::translate("PluginSpec", "'%1' has invalid format").arg(content);
+    return QCoreApplication::translate("PluginSpec", "'%1' has invalid format").arg(QLatin1String(content));
 }
 
 static inline QString msgInvalidElement(const QString &name)
@@ -482,16 +632,17 @@ static inline QString msgUnexpectedToken()
 void PluginSpecPrivate::readPluginSpec(QXmlStreamReader &reader)
 {
     QString element = reader.name().toString();
-    if (element != QString(PLUGIN)) {
-        reader.raiseError(QCoreApplication::translate("PluginSpec", "Expected element '%1' as top level element").arg(PLUGIN));
+    if (element != QLatin1String(PLUGIN)) {
+        reader.raiseError(QCoreApplication::translate("PluginSpec", "Expected element '%1' as top level element")
+                          .arg(QLatin1String(PLUGIN)));
         return;
     }
-    name = reader.attributes().value(PLUGIN_NAME).toString();
+    name = reader.attributes().value(QLatin1String(PLUGIN_NAME)).toString();
     if (name.isEmpty()) {
         reader.raiseError(msgAttributeMissing(PLUGIN, PLUGIN_NAME));
         return;
     }
-    version = reader.attributes().value(PLUGIN_VERSION).toString();
+    version = reader.attributes().value(QLatin1String(PLUGIN_VERSION)).toString();
     if (version.isEmpty()) {
         reader.raiseError(msgAttributeMissing(PLUGIN, PLUGIN_VERSION));
         return;
@@ -500,31 +651,40 @@ void PluginSpecPrivate::readPluginSpec(QXmlStreamReader &reader)
         reader.raiseError(msgInvalidFormat(PLUGIN_VERSION));
         return;
     }
-    compatVersion = reader.attributes().value(PLUGIN_COMPATVERSION).toString();
+    compatVersion = reader.attributes().value(QLatin1String(PLUGIN_COMPATVERSION)).toString();
     if (!compatVersion.isEmpty() && !isValidVersion(compatVersion)) {
         reader.raiseError(msgInvalidFormat(PLUGIN_COMPATVERSION));
         return;
     } else if (compatVersion.isEmpty()) {
         compatVersion = version;
     }
+    disabledByDefault = readBooleanValue(reader, PLUGIN_DISABLED_BY_DEFAULT);
+    experimental = readBooleanValue(reader, PLUGIN_EXPERIMENTAL);
+    if (reader.hasError())
+        return;
+    if (experimental)
+        disabledByDefault = true;
+    enabledInSettings = !disabledByDefault;
     while (!reader.atEnd()) {
         reader.readNext();
         switch (reader.tokenType()) {
         case QXmlStreamReader::StartElement:
             element = reader.name().toString();
-            if (element == VENDOR)
+            if (element == QLatin1String(VENDOR))
                 vendor = reader.readElementText().trimmed();
-            else if (element == COPYRIGHT)
+            else if (element == QLatin1String(COPYRIGHT))
                 copyright = reader.readElementText().trimmed();
-            else if (element == LICENSE)
+            else if (element == QLatin1String(LICENSE))
                 license = reader.readElementText().trimmed();
-            else if (element == DESCRIPTION)
+            else if (element == QLatin1String(DESCRIPTION))
                 description = reader.readElementText().trimmed();
-            else if (element == URL)
+            else if (element == QLatin1String(URL))
                 url = reader.readElementText().trimmed();
-            else if (element == DEPENDENCYLIST)
+            else if (element == QLatin1String(CATEGORY))
+                category = reader.readElementText().trimmed();
+            else if (element == QLatin1String(DEPENDENCYLIST))
                 readDependencies(reader);
-            else if (element == ARGUMENTLIST)
+            else if (element == QLatin1String(ARGUMENTLIST))
                 readArgumentDescriptions(reader);
             else
                 reader.raiseError(msgInvalidElement(name));
@@ -554,18 +714,17 @@ void PluginSpecPrivate::readArgumentDescriptions(QXmlStreamReader &reader)
         switch (reader.tokenType()) {
         case QXmlStreamReader::StartElement:
             element = reader.name().toString();
-            if (element == ARGUMENT) {
+            if (element == QLatin1String(ARGUMENT))
                 readArgumentDescription(reader);
-            } else {
+            else
                 reader.raiseError(msgInvalidElement(name));
-            }
             break;
         case QXmlStreamReader::Comment:
         case QXmlStreamReader::Characters:
             break;
         case QXmlStreamReader::EndElement:
             element = reader.name().toString();
-            if (element == ARGUMENTLIST)
+            if (element == QLatin1String(ARGUMENTLIST))
                 return;
             reader.raiseError(msgUnexpectedClosing(element));
             break;
@@ -583,18 +742,29 @@ void PluginSpecPrivate::readArgumentDescriptions(QXmlStreamReader &reader)
 void PluginSpecPrivate::readArgumentDescription(QXmlStreamReader &reader)
 {
     PluginArgumentDescription arg;
-    arg.name = reader.attributes().value(ARGUMENT_NAME).toString();
+    arg.name = reader.attributes().value(QLatin1String(ARGUMENT_NAME)).toString();
     if (arg.name.isEmpty()) {
         reader.raiseError(msgAttributeMissing(ARGUMENT, ARGUMENT_NAME));
         return;
     }
-    arg.parameter = reader.attributes().value(ARGUMENT_PARAMETER).toString();
+    arg.parameter = reader.attributes().value(QLatin1String(ARGUMENT_PARAMETER)).toString();
     arg.description = reader.readElementText();
     if (reader.tokenType() != QXmlStreamReader::EndElement)
         reader.raiseError(msgUnexpectedToken());
     argumentDescriptions.push_back(arg);
 }
 
+bool PluginSpecPrivate::readBooleanValue(QXmlStreamReader &reader, const char *key)
+{
+    const QString valueString = reader.attributes().value(QLatin1String(key)).toString();
+    const bool isOn = valueString.compare(QLatin1String("true"), Qt::CaseInsensitive) == 0;
+    if (!valueString.isEmpty() && !isOn
+            && valueString.compare(QLatin1String("false"), Qt::CaseInsensitive) != 0) {
+        reader.raiseError(msgInvalidFormat(key));
+    }
+    return isOn;
+}
+
 /*!
     \fn void PluginSpecPrivate::readDependencies(QXmlStreamReader &reader)
     \internal
@@ -607,18 +777,17 @@ void PluginSpecPrivate::readDependencies(QXmlStreamReader &reader)
         switch (reader.tokenType()) {
         case QXmlStreamReader::StartElement:
             element = reader.name().toString();
-            if (element == DEPENDENCY) {
+            if (element == QLatin1String(DEPENDENCY))
                 readDependencyEntry(reader);
-            } else {
+            else
                 reader.raiseError(msgInvalidElement(name));
-            }
             break;
         case QXmlStreamReader::Comment:
         case QXmlStreamReader::Characters:
             break;
         case QXmlStreamReader::EndElement:
             element = reader.name().toString();
-            if (element == DEPENDENCYLIST)
+            if (element == QLatin1String(DEPENDENCYLIST))
                 return;
             reader.raiseError(msgUnexpectedClosing(element));
             break;
@@ -636,16 +805,28 @@ void PluginSpecPrivate::readDependencies(QXmlStreamReader &reader)
 void PluginSpecPrivate::readDependencyEntry(QXmlStreamReader &reader)
 {
     PluginDependency dep;
-    dep.name = reader.attributes().value(DEPENDENCY_NAME).toString();
+    dep.name = reader.attributes().value(QLatin1String(DEPENDENCY_NAME)).toString();
     if (dep.name.isEmpty()) {
         reader.raiseError(msgAttributeMissing(DEPENDENCY, DEPENDENCY_NAME));
         return;
     }
-    dep.version = reader.attributes().value(DEPENDENCY_VERSION).toString();
+    dep.version = reader.attributes().value(QLatin1String(DEPENDENCY_VERSION)).toString();
     if (!dep.version.isEmpty() && !isValidVersion(dep.version)) {
         reader.raiseError(msgInvalidFormat(DEPENDENCY_VERSION));
         return;
     }
+    dep.type = PluginDependency::Required;
+    if (reader.attributes().hasAttribute(QLatin1String(DEPENDENCY_TYPE))) {
+        QString typeValue = reader.attributes().value(QLatin1String(DEPENDENCY_TYPE)).toString();
+        if (typeValue == QLatin1String(DEPENDENCY_TYPE_HARD)) {
+            dep.type = PluginDependency::Required;
+        } else if (typeValue == QLatin1String(DEPENDENCY_TYPE_SOFT)) {
+            dep.type = PluginDependency::Optional;
+        } else {
+            reader.raiseError(msgInvalidFormat(DEPENDENCY_TYPE));
+            return;
+        }
+    }
     dependencies.append(dep);
     reader.readNext();
     if (reader.tokenType() != QXmlStreamReader::EndElement)
@@ -669,7 +850,7 @@ bool PluginSpecPrivate::provides(const QString &pluginName, const QString &plugi
 */
 QRegExp &PluginSpecPrivate::versionRegExp()
 {
-    static QRegExp reg("([0-9]+)(?:[.]([0-9]+))?(?:[.]([0-9]+))?(?:_([0-9]+))?");
+    static QRegExp reg(QLatin1String("([0-9]+)(?:[.]([0-9]+))?(?:[.]([0-9]+))?(?:_([0-9]+))?"));
     return reg;
 }
 /*!
@@ -721,9 +902,10 @@ bool PluginSpecPrivate::resolveDependencies(const QList<PluginSpec *> &specs)
         hasError = true;
         return false;
     }
-    QList<PluginSpec *> resolvedDependencies;
+    QHash<PluginDependency, PluginSpec *> resolvedDependencies;
     foreach (const PluginDependency &dependency, dependencies) {
         PluginSpec *found = 0;
+
         foreach (PluginSpec *spec, specs) {
             if (spec->provides(dependency.name, dependency.version)) {
                 found = spec;
@@ -731,22 +913,48 @@ bool PluginSpecPrivate::resolveDependencies(const QList<PluginSpec *> &specs)
             }
         }
         if (!found) {
-            hasError = true;
-            if (!errorString.isEmpty())
-                errorString.append("\n");
-            errorString.append(QCoreApplication::translate("PluginSpec", "Could not resolve dependency '%1(%2)'")
-                .arg(dependency.name).arg(dependency.version));
+            if (dependency.type == PluginDependency::Required) {
+                hasError = true;
+                if (!errorString.isEmpty())
+                    errorString.append(QLatin1Char('\n'));
+                errorString.append(QCoreApplication::translate("PluginSpec", "Could not resolve dependency '%1(%2)'")
+                    .arg(dependency.name).arg(dependency.version));
+            }
             continue;
         }
-        resolvedDependencies.append(found);
+        resolvedDependencies.insert(dependency, found);
     }
     if (hasError)
         return false;
+
     dependencySpecs = resolvedDependencies;
+
     state = PluginSpec::Resolved;
+
     return true;
 }
 
+void PluginSpecPrivate::disableIndirectlyIfDependencyDisabled()
+{
+    if (!enabledInSettings)
+        return;
+
+    if (disabledIndirectly)
+        return;
+
+    QHashIterator<PluginDependency, PluginSpec *> it(dependencySpecs);
+    while (it.hasNext()) {
+        it.next();
+        if (it.key().type == PluginDependency::Optional)
+            continue;
+        PluginSpec *dependencySpec = it.value();
+        if (!dependencySpec->isEffectivelyEnabled()) {
+            disabledIndirectly = true;
+            break;
+        }
+    }
+}
+
 /*!
     \fn bool PluginSpecPrivate::loadLibrary()
     \internal
@@ -765,21 +973,21 @@ bool PluginSpecPrivate::loadLibrary()
 #ifdef QT_NO_DEBUG
 
 #ifdef Q_OS_WIN
-    QString libName = QString("%1/%2.dll").arg(location).arg(name);
+    QString libName = QString::fromLatin1("%1/%2.dll").arg(location).arg(name);
 #elif defined(Q_OS_MAC)
-    QString libName = QString("%1/lib%2.dylib").arg(location).arg(name);
+    QString libName = QString::fromLatin1("%1/lib%2.dylib").arg(location).arg(name);
 #else
-    QString libName = QString("%1/lib%2.so").arg(location).arg(name);
+    QString libName = QString::fromLatin1("%1/lib%2.so").arg(location).arg(name);
 #endif
 
 #else //Q_NO_DEBUG
 
 #ifdef Q_OS_WIN
-    QString libName = QString("%1/%2d.dll").arg(location).arg(name);
+    QString libName = QString::fromLatin1("%1/%2d.dll").arg(location).arg(name);
 #elif defined(Q_OS_MAC)
-    QString libName = QString("%1/lib%2_debug.dylib").arg(location).arg(name);
+    QString libName = QString::fromLatin1("%1/lib%2_debug.dylib").arg(location).arg(name);
 #else
-    QString libName = QString("%1/lib%2.so").arg(location).arg(name);
+    QString libName = QString::fromLatin1("%1/lib%2.so").arg(location).arg(name);
 #endif
 
 #endif
@@ -787,14 +995,14 @@ bool PluginSpecPrivate::loadLibrary()
     PluginLoader loader(libName);
     if (!loader.load()) {
         hasError = true;
-        errorString = loader.errorString();
-        errorString.append(QCoreApplication::translate("PluginSpec", "\nLibrary base name: %1").arg(libName));
+        errorString = QDir::toNativeSeparators(libName)
+            + QString::fromLatin1(": ") + loader.errorString();
         return false;
     }
     IPlugin *pluginObject = qobject_cast<IPlugin*>(loader.instance());
     if (!pluginObject) {
         hasError = true;
-        errorString = QCoreApplication::translate("PluginSpec", "Plugin is not valid (doesn't derive from IPlugin)");
+        errorString = QCoreApplication::translate("PluginSpec", "Plugin is not valid (does not derive from IPlugin)");
         loader.unload();
         return false;
     }
@@ -859,16 +1067,34 @@ bool PluginSpecPrivate::initializeExtensions()
     return true;
 }
 
+/*!
+    \fn bool PluginSpecPrivate::delayedInitialize()
+    \internal
+*/
+bool PluginSpecPrivate::delayedInitialize()
+{
+    if (hasError)
+        return false;
+    if (state != PluginSpec::Running)
+        return false;
+    if (!plugin) {
+        errorString = QCoreApplication::translate("PluginSpec", "Internal error: have no plugin instance to perform delayedInitialize");
+        hasError = true;
+        return false;
+    }
+    return plugin->delayedInitialize();
+}
+
 /*!
     \fn bool PluginSpecPrivate::stop()
     \internal
 */
-void PluginSpecPrivate::stop()
+IPlugin::ShutdownFlag PluginSpecPrivate::stop()
 {
     if (!plugin)
-        return;
-    plugin->shutdown();
+        return IPlugin::SynchronousShutdown;
     state = PluginSpec::Stopped;
+    return plugin->aboutToShutdown();
 }
 
 /*!
@@ -883,4 +1109,3 @@ void PluginSpecPrivate::kill()
     plugin = 0;
     state = PluginSpec::Deleted;
 }
-
