@@ -1,6 +1,6 @@
 /**************************************************************************
 **
-** Copyright (c) 2012 AudioCodes Ltd.
+** Copyright (c) 2013 AudioCodes Ltd.
 ** Author: Orgad Shaneh <orgad.shaneh@audiocodes.com>
 ** Contact: http://www.qt-project.org/legal
 **
@@ -58,6 +58,7 @@
 #include <utils/synchronousprocess.h>
 #include <utils/parameteraction.h>
 #include <utils/fileutils.h>
+#include <utils/hostosinfo.h>
 #include <utils/qtcassert.h>
 #include <utils/runextensions.h>
 #include <vcsbase/basevcseditorfactory.h>
@@ -95,7 +96,9 @@
 #include <QVariant>
 #include <QVBoxLayout>
 #include <QXmlStreamReader>
-
+#ifdef WITH_TESTS
+#include <QTest>
+#endif
 
 namespace ClearCase {
 namespace Internal {
@@ -115,6 +118,7 @@ static const char CMD_ID_UPDATEINDEX[]        = "ClearCase.UpdateIndex";
 static const char CMD_ID_UPDATE_VIEW[]        = "ClearCase.UpdateView";
 static const char CMD_ID_CHECKIN_ALL[]        = "ClearCase.CheckInAll";
 static const char CMD_ID_STATUS[]             = "ClearCase.Status";
+static const char *CLEARCASE_ROOT_FILES[] = { "view.dat", ".view.dat" };
 
 static const VcsBase::VcsBaseEditorParameters editorParameters[] = {
 {
@@ -152,16 +156,20 @@ static inline const VcsBase::VcsBaseEditorParameters *findType(int ie)
 
 static inline QString debugCodec(const QTextCodec *c)
 {
-    return c ? QString::fromAscii(c->name()) : QString::fromAscii("Null codec");
+    return c ? QString::fromLatin1(c->name()) : QString::fromLatin1("Null codec");
 }
 
 // ------------- ClearCasePlugin
 ClearCasePlugin *ClearCasePlugin::m_clearcasePluginInstance = 0;
 
+ViewData::ViewData() :
+    isDynamic(false),
+    isUcm(false)
+{
+}
+
 ClearCasePlugin::ClearCasePlugin() :
     VcsBase::VcsBasePlugin(QLatin1String(ClearCase::Constants::CLEARCASECHECKINEDITOR_ID)),
-    m_isDynamic(false),
-    m_isUcm(false),
     m_commandLocator(0),
     m_checkOutAction(0),
     m_checkInCurrentAction(0),
@@ -212,10 +220,27 @@ bool ClearCasePlugin::isCheckInEditorOpen() const
     return !m_checkInMessageFileName.isEmpty();
 }
 
+static QString ccFindRepositoryForDirectory(const QString &dirS)
+{
+    const QString home = QDir::homePath();
+
+    QDir directory(dirS);
+    do {
+        const QString absDirPath = directory.absolutePath();
+        if (directory.isRoot() || absDirPath == home)
+            break;
+
+        for (uint i = 0; i < sizeof(CLEARCASE_ROOT_FILES) / sizeof(*CLEARCASE_ROOT_FILES); ++i)
+            if (QFileInfo(directory, QLatin1String(CLEARCASE_ROOT_FILES[i])).isFile())
+                return absDirPath;
+    } while (directory.cdUp());
+    return QString();
+}
+
 /*! Find top level for view that contains \a directory
  *
- * - Snapshot Views will have the CLEARCASE_ROOT_FILE (view.dat) in its top dir
- * - Dynamic views can either be
+ * - Snapshot view has one of CLEARCASE_ROOT_FILES (view.dat or .view.dat) in its top dir
+ * - Dynamic view can either be
  *      - M:/view_name,
  *      - or mapped to a drive letter, like Z:/
  *    (drive letters are just examples)
@@ -228,14 +253,12 @@ QString ClearCasePlugin::findTopLevel(const QString &directory) const
 
     // Snapshot view
     QString topLevel =
-            findRepositoryForDirectory(directory, QLatin1String(ClearCase::Constants::CLEARCASE_ROOT_FILE));
+            ccFindRepositoryForDirectory(directory);
     if (!topLevel.isEmpty() || !clearCaseControl()->isConfigured())
         return topLevel;
 
     // Dynamic view
-    bool isDynamic;
-    ccGetView(directory, &isDynamic);
-    if (isDynamic) {
+    if (ccGetView(directory).isDynamic) {
         QDir dir(directory);
         // Go up to one level before root
         QDir outer = dir;
@@ -244,7 +267,7 @@ QString ClearCasePlugin::findTopLevel(const QString &directory) const
             dir.cdUp();
         topLevel = dir.path(); // M:/View_Name
         dir.cdUp(); // Z:/ (dynamic view with assigned letter)
-        if (!ccGetView(dir.path()).isEmpty())
+        if (!ccGetView(dir.path()).name.isEmpty())
             topLevel = dir.path();
     }
 
@@ -255,7 +278,8 @@ static const VcsBase::VcsBaseSubmitEditorParameters submitParameters = {
     ClearCase::Constants::CLEARCASE_SUBMIT_MIMETYPE,
     ClearCase::Constants::CLEARCASECHECKINEDITOR_ID,
     ClearCase::Constants::CLEARCASECHECKINEDITOR_DISPLAY_NAME,
-    ClearCase::Constants::CLEARCASECHECKINEDITOR
+    ClearCase::Constants::CLEARCASECHECKINEDITOR,
+    VcsBase::VcsBaseSubmitEditorParameters::DiffFiles
 };
 
 bool ClearCasePlugin::initialize(const QStringList & /*arguments */, QString *errorMessage)
@@ -297,7 +321,7 @@ bool ClearCasePlugin::initialize(const QStringList & /*arguments */, QString *er
     const QString description = QLatin1String("ClearCase");
     const QString prefix = QLatin1String("cc");
     // register cc prefix in Locator
-    m_commandLocator = new Locator::CommandLocator(prefix, description, prefix);
+    m_commandLocator = new Locator::CommandLocator("cc", description, prefix);
     addAutoReleasedObject(m_commandLocator);
 
     //register actions
@@ -583,22 +607,24 @@ ClearCaseSubmitEditor *ClearCasePlugin::openClearCaseSubmitEditor(const QString
 
 void ClearCasePlugin::updateStatusActions()
 {
+    FileStatus fileStatus = FileStatus::Unknown;
     bool hasFile = currentState().hasFile();
-    QString fileName = currentState().relativeCurrentFile();
+    if (hasFile) {
+        QString fileName = currentState().relativeCurrentFile();
+        fileStatus = m_statusMap->value(fileName, FileStatus(FileStatus::Unknown));
 
-    FileStatus fileStatus = m_statusMap->value(fileName, FileStatus(FileStatus::Unknown));
-
-    if (ClearCase::Constants::debug)
-        qDebug() << Q_FUNC_INFO << fileName << ", status = " << fileStatus.status;
+        if (ClearCase::Constants::debug)
+            qDebug() << Q_FUNC_INFO << fileName << ", status = " << fileStatus.status;
+    }
 
     m_checkOutAction->setEnabled(hasFile && (fileStatus.status & (FileStatus::CheckedIn | FileStatus::Hijacked)));
     m_undoCheckOutAction->setEnabled(hasFile && (fileStatus.status & FileStatus::CheckedOut));
-    m_undoHijackAction->setEnabled(!m_isDynamic && hasFile && (fileStatus.status & FileStatus::Hijacked));
+    m_undoHijackAction->setEnabled(!m_viewData.isDynamic && hasFile && (fileStatus.status & FileStatus::Hijacked));
     m_checkInCurrentAction->setEnabled(hasFile && (fileStatus.status & FileStatus::CheckedOut));
     m_addFileAction->setEnabled(hasFile && (fileStatus.status & FileStatus::NotManaged));
 
-    m_checkInActivityAction->setEnabled(m_isUcm);
-    m_diffActivityAction->setEnabled(m_isUcm);
+    m_checkInActivityAction->setEnabled(m_viewData.isUcm);
+    m_diffActivityAction->setEnabled(m_viewData.isUcm);
 }
 
 void ClearCasePlugin::updateActions(VcsBase::VcsBasePlugin::ActionState as)
@@ -613,7 +639,7 @@ void ClearCasePlugin::updateActions(VcsBase::VcsBasePlugin::ActionState as)
     if (hasTopLevel)
         m_topLevel = state.topLevel();
 
-    m_updateViewAction->setParameter(m_isDynamic ? QString() : m_view);
+    m_updateViewAction->setParameter(m_viewData.isDynamic ? QString() : m_viewData.name);
 
     const QString fileName = state.currentFileName();
     m_checkOutAction->setParameter(fileName);
@@ -714,11 +740,10 @@ bool ClearCasePlugin::vcsUndoHijack(const QString &workingDir, const QString &fi
     QStringList args(QLatin1String("update"));
     args << QLatin1String(keep ? "-rename" : "-overwrite");
     args << QLatin1String("-log");
-#ifdef Q_OS_WIN32
-    args << QLatin1String("NUL");
-#else
+    if (Utils::HostOsInfo::isWindowsHost())
+        args << QLatin1String("NUL");
+    else
     args << QLatin1String("/dev/null");
-#endif
     args << QDir::toNativeSeparators(fileName);
 
     const ClearCaseResponse response =
@@ -937,6 +962,7 @@ void ClearCasePlugin::startCheckInAll()
         if (iterator.value().status == FileStatus::CheckedOut)
             files.append(QDir::toNativeSeparators(iterator.key()));
     }
+    files.sort();
     startCheckIn(topLevel, files);
 }
 
@@ -973,6 +999,7 @@ void ClearCasePlugin::startCheckInActivity()
             last = file;
         }
     }
+    files.sort();
     startCheckIn(topLevel, files);
 }
 
@@ -1009,10 +1036,10 @@ void ClearCasePlugin::startCheckIn(const QString &workingDir, const QStringList
     m_checkInMessageFileName = saver.fileName();
     m_checkInView = workingDir;
     // Create a submit editor and set file list
-    ClearCaseSubmitEditor *editor = openClearCaseSubmitEditor(m_checkInMessageFileName, m_isUcm);
+    ClearCaseSubmitEditor *editor = openClearCaseSubmitEditor(m_checkInMessageFileName, m_viewData.isUcm);
     editor->setStatusList(files);
 
-    if (m_isUcm && (files.size() == 1)) {
+    if (m_viewData.isUcm && (files.size() == 1)) {
         QString activity = ccGetFileActivity(workingDir, files.first());
         editor->submitEditorWidget()->setActivity(activity);
     }
@@ -1072,9 +1099,9 @@ void ClearCasePlugin::history(const QString &workingDir,
 
 void ClearCasePlugin::viewStatus()
 {
-    if (m_view.isEmpty())
-        m_view = ccGetView(m_topLevel);
-    QTC_ASSERT(!m_view.isEmpty() && !m_settings.disableIndexer, return);
+    if (m_viewData.name.isEmpty())
+        m_viewData = ccGetView(m_topLevel);
+    QTC_ASSERT(!m_viewData.name.isEmpty() && !m_settings.disableIndexer, return);
     VcsBase::VcsBaseOutputWindow *outputwindow = VcsBase::VcsBaseOutputWindow::instance();
     outputwindow->appendCommand(QLatin1String("Indexed files status (C=Checked Out, H=Hijacked, ?=Missing)"));
     bool anymod = false;
@@ -1261,7 +1288,7 @@ Core::IEditor *ClearCasePlugin::showOutputInEditor(const QString& title, const Q
 {
     const VcsBase::VcsBaseEditorParameters *params = findType(editorType);
     QTC_ASSERT(params, return 0);
-    const Core::Id id = params->id;
+    const Core::Id id = Core::Id(QByteArray(params->id));
     if (ClearCase::Constants::debug)
         qDebug() << "ClearCasePlugin::showOutputInEditor" << title << id.name()
                  <<  "Size= " << output.size() <<  " Type=" << editorType << debugCodec(codec);
@@ -1318,7 +1345,7 @@ bool ClearCasePlugin::vcsOpen(const QString &workingDir, const QString &fileName
 
     const QString file = QDir::toNativeSeparators(relFile);
     const QString title = QString::fromLatin1("Checkout %1").arg(file);
-    CheckOutDialog coDialog(title, m_isUcm);
+    CheckOutDialog coDialog(title, m_viewData.isUcm);
 
     if (!m_settings.disableIndexer &&
             (fi.isWritable() || m_statusMap->value(relFile).status == FileStatus::Unknown))
@@ -1328,11 +1355,11 @@ bool ClearCasePlugin::vcsOpen(const QString &workingDir, const QString &fileName
         return true;
     }
     // Only snapshot views can have hijacked files
-    bool isHijacked = (!m_isDynamic && (m_statusMap->value(relFile).status & FileStatus::Hijacked));
+    bool isHijacked = (!m_viewData.isDynamic && (m_statusMap->value(relFile).status & FileStatus::Hijacked));
     if (!isHijacked)
         coDialog.hideHijack();
     if (coDialog.exec() == QDialog::Accepted) {
-        if (m_isUcm && !vcsSetActivity(topLevel, title, coDialog.activity()))
+        if (m_viewData.isUcm && !vcsSetActivity(topLevel, title, coDialog.activity()))
             return false;
 
         Core::FileChangeBlocker fcb(absPath);
@@ -1475,7 +1502,7 @@ bool ClearCasePlugin::ccFileOp(const QString &workingDir, const QString &title,
     fileOpDlg.setWindowTitle(title);
 
     verticalLayout = new QVBoxLayout(&fileOpDlg);
-    if (m_isUcm) {
+    if (m_viewData.isUcm) {
         actSelector = new ActivitySelector;
         verticalLayout->addWidget(actSelector);
     }
@@ -1501,7 +1528,7 @@ bool ClearCasePlugin::ccFileOp(const QString &workingDir, const QString &title,
         return true;
 
     QString comment = commentEdit->toPlainText();
-    if (m_isUcm && actSelector->changed())
+    if (m_viewData.isUcm && actSelector->changed())
         vcsSetActivity(workingDir, fileOpDlg.windowTitle(), actSelector->activity());
 
     QString dirName = QDir::toNativeSeparators(QFileInfo(workingDir, fileName).absolutePath());
@@ -1708,25 +1735,22 @@ bool ClearCasePlugin::ccCheckUcm(const QString &viewname, const QString &working
     return QRegExp(QLatin1String("(^|\\n)ucm\\n")).indexIn(catcsData) != -1;
 }
 
-QString ClearCasePlugin::ccGetView(const QString &workingDir, bool *isDynamic, bool *isUcm) const
+ViewData ClearCasePlugin::ccGetView(const QString &workingDir) const
 {
-    QStringList args(QLatin1String("lsview"));
-    args << QLatin1String("-cview");
-    QString data = runCleartoolSync(workingDir, args);
-    if (data.isEmpty()) {
-        if (isDynamic)
-            *isDynamic = false;
-        return QString();
+    static QHash<QString, ViewData> viewCache;
+
+    bool inCache = viewCache.contains(workingDir);
+    ViewData &res = viewCache[workingDir];
+    if (!inCache) {
+        QStringList args(QLatin1String("lsview"));
+        args << QLatin1String("-cview");
+        QString data = runCleartoolSync(workingDir, args);
+        res.isDynamic = !data.isEmpty() && (data.at(0) == QLatin1Char('*'));
+        res.name = data.mid(2, data.indexOf(QLatin1Char(' '), 2) - 2);
+        res.isUcm = ccCheckUcm(res.name, workingDir);
     }
-    if (isDynamic)
-        *isDynamic = !data.isEmpty() && (data.at(0) == QLatin1Char('*'));
-
-    QString viewname = data.mid(2, data.indexOf(QLatin1Char(' '), 2) - 2);
-    if (isUcm)
-        *isUcm = ccCheckUcm(viewname, workingDir);
-
-    return viewname;
 
+    return res;
 }
 
 void ClearCasePlugin::updateStreamAndView()
@@ -1739,15 +1763,15 @@ void ClearCasePlugin::updateStreamAndView()
     QRegExp intStreamExp(QLatin1String("stream:([^@]*)"));
     if (intStreamExp.indexIn(sresponse.mid(tabPos + 1)) != -1)
         m_intStream = intStreamExp.cap(1);
-    m_view = ccGetView(m_topLevel, &m_isDynamic, &m_isUcm);
-    m_updateViewAction->setParameter(m_isDynamic ? QString() : m_view);
+    m_viewData = ccGetView(m_topLevel);
+    m_updateViewAction->setParameter(m_viewData.isDynamic ? QString() : m_viewData.name);
 }
 
 void ClearCasePlugin::projectChanged(ProjectExplorer::Project *project)
 {
-    if (m_view == ccGetView(m_topLevel)) // New project on same view as old project
+    if (m_viewData.name == ccGetView(m_topLevel).name) // New project on same view as old project
         return;
-    m_view.clear();
+    m_viewData = ViewData();
     m_stream.clear();
     m_intStream.clear();
     disconnect(Core::ICore::mainWindow(), SIGNAL(windowActivated()), this, SLOT(syncSlot()));
@@ -1760,14 +1784,12 @@ void ClearCasePlugin::projectChanged(ProjectExplorer::Project *project)
             return;
         connect(Core::ICore::mainWindow(), SIGNAL(windowActivated()), this, SLOT(syncSlot()));
         updateStreamAndView();
-        if (m_view.isEmpty())
+        if (m_viewData.name.isEmpty())
             return;
-        Core::EditorManager *editorManager = Core::ICore::editorManager();
-        editorManager->setWindowTitleAddition(editorManager->windowTitleAddition() + QLatin1String(" - ") + m_view);
         updateIndex();
     }
     if ( ClearCase::Constants::debug )
-        qDebug() << "stream: " << m_stream << "; intStream: " << m_intStream << "view: " << m_view;
+        qDebug() << "stream: " << m_stream << "; intStream: " << m_intStream << "view: " << m_viewData.name;
 }
 
 void ClearCasePlugin::tasksFinished(const QString &type)
@@ -1943,6 +1965,38 @@ void ClearCasePlugin::sync(QFutureInterface<void> &future, QString topLevel, QSt
     ccSync.run(future, topLevel, files);
 }
 
+#ifdef WITH_TESTS
+void ClearCasePlugin::testDiffFileResolving_data()
+{
+    QTest::addColumn<QByteArray>("header");
+    QTest::addColumn<QByteArray>("fileName");
+
+    QTest::newRow("Modified") << QByteArray(
+            "--- src/plugins/clearcase/clearcaseeditor.cpp@@/main/1\t2013-01-20 23:45:48.549615210 +0200\n"
+            "+++ src/plugins/clearcase/clearcaseeditor.cpp@@/main/2\t2013-01-20 23:45:53.217604679 +0200\n"
+            "@@ -58,6 +58,10 @@\n\n")
+        << QByteArray("src/plugins/clearcase/clearcaseeditor.cpp");
+}
+
+void ClearCasePlugin::testDiffFileResolving()
+{
+    ClearCaseEditor editor(editorParameters + 3, 0);
+    editor.testDiffFileResolving();
+}
+
+void ClearCasePlugin::testLogResolving()
+{
+    QByteArray data(
+                "13-Sep.17:41   user1      create version \"src/plugins/clearcase/clearcaseeditor.h@@/main/branch1/branch2/9\" (baseline1, baseline2, ...)\n"
+                "22-Aug.14:13   user2      create version \"src/plugins/clearcase/clearcaseeditor.h@@/main/branch1/branch2/8\" (baseline3, baseline4, ...)\n"
+                );
+    ClearCaseEditor editor(editorParameters + 1, 0);
+    editor.testLogResolving(data,
+                            "src/plugins/clearcase/clearcaseeditor.h@@/main/branch1/branch2/9",
+                            "src/plugins/clearcase/clearcaseeditor.h@@/main/branch1/branch2/8");
+}
+#endif
+
 } // namespace Internal
 } // namespace ClearCase
 
