@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -32,12 +32,15 @@
 #include "cmakebuildconfiguration.h"
 
 #include <coreplugin/icore.h>
+#include <utils/hostosinfo.h>
 #include <utils/pathchooser.h>
 #include <utils/fancylineedit.h>
+#include <utils/historycompleter.h>
 #include <projectexplorer/kitinformation.h>
 #include <projectexplorer/kitmanager.h>
 #include <projectexplorer/toolchain.h>
 #include <projectexplorer/abi.h>
+#include <projectexplorer/projectexplorerconstants.h>
 #include <texteditor/fontsettings.h>
 
 #include <QVBoxLayout>
@@ -69,9 +72,10 @@ namespace CMakeProjectManager {
 namespace Internal {
     class GeneratorInfo
     {
+        Q_DECLARE_TR_FUNCTIONS(CMakeProjectManager::Internal::GeneratorInfo)
     public:
         enum Ninja { NoNinja, OfferNinja, ForceNinja };
-        static QList<GeneratorInfo> generatorInfosFor(ProjectExplorer::Kit *k, Ninja n, bool hasCodeBlocks);
+        static QList<GeneratorInfo> generatorInfosFor(ProjectExplorer::Kit *k, Ninja n, bool preferNinja, bool hasCodeBlocks);
 
         GeneratorInfo();
         explicit GeneratorInfo(ProjectExplorer::Kit *kit, bool ninja = false);
@@ -80,14 +84,13 @@ namespace Internal {
         bool isNinja() const;
 
         QString displayName() const;
-        QString generatorArgument() const;
-        QString generator() const;
+        QByteArray generatorArgument() const;
+        QByteArray generator() const;
 
     private:
         ProjectExplorer::Kit *m_kit;
         bool m_isNinja;
     };
-
 }
 }
 
@@ -110,37 +113,36 @@ bool GeneratorInfo::isNinja() const {
     return m_isNinja;
 }
 
-QString GeneratorInfo::generator() const
+QByteArray GeneratorInfo::generator() const
 {
     if (!m_kit)
-        return QString();
+        return QByteArray();
     ProjectExplorer::ToolChain *tc = ProjectExplorer::ToolChainKitInformation::toolChain(m_kit);
     ProjectExplorer::Abi targetAbi = tc->targetAbi();
     if (m_isNinja) {
-        return QLatin1String("Ninja");
+        return "Ninja";
     } else if (targetAbi.os() == ProjectExplorer::Abi::WindowsOS) {
         if (targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2005Flavor
                 || targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2008Flavor
                 || targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2010Flavor
                 || targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2012Flavor) {
-            return QLatin1String("NMake Makefiles");
+            return "NMake Makefiles";
         } else if (targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMSysFlavor) {
-#ifdef Q_OS_WIN
-                return QLatin1String("MinGW Makefiles");
-#else
-                return QLatin1String("Unix Makefiles");
-#endif
+            if (Utils::HostOsInfo::isWindowsHost())
+                return "MinGW Makefiles";
+            else
+                return "Unix Makefiles";
         }
     }
-    return QLatin1String("Unix Makefiles");
+    return "Unix Makefiles";
 }
 
-QString GeneratorInfo::generatorArgument() const
+QByteArray GeneratorInfo::generatorArgument() const
 {
-    QString tmp = generator();
+    QByteArray tmp = generator();
     if (tmp.isEmpty())
         return tmp;
-    return QLatin1String("-GCodeBlocks - ") + tmp;
+    return QByteArray("-GCodeBlocks - ") + tmp;
 }
 
 QString GeneratorInfo::displayName() const
@@ -148,7 +150,7 @@ QString GeneratorInfo::displayName() const
     if (!m_kit)
         return QString();
     if (m_isNinja)
-        return QApplication::tr("Ninja (%1)").arg(m_kit->displayName());
+        return tr("Ninja (%1)").arg(m_kit->displayName());
     ProjectExplorer::ToolChain *tc = ProjectExplorer::ToolChainKitInformation::toolChain(m_kit);
     ProjectExplorer::Abi targetAbi = tc->targetAbi();
     if (targetAbi.os() == ProjectExplorer::Abi::WindowsOS) {
@@ -156,22 +158,21 @@ QString GeneratorInfo::displayName() const
                 || targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2008Flavor
                 || targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2010Flavor
                 || targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2012Flavor) {
-            return QApplication::tr("NMake Generator (%1)").arg(m_kit->displayName());
+            return tr("NMake Generator (%1)").arg(m_kit->displayName());
         } else if (targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMSysFlavor) {
-#ifdef Q_OS_WIN
-                return QApplication::tr("MinGW Generator (%1)").arg(m_kit->displayName());
-#else
-                return QApplication::tr("Unix Generator (%1)").arg(m_kit->displayName());
-#endif
+            if (Utils::HostOsInfo::isWindowsHost())
+                return tr("MinGW Generator (%1)").arg(m_kit->displayName());
+            else
+                return tr("Unix Generator (%1)").arg(m_kit->displayName());
         }
     } else {
         // Non windows
-        return QApplication::tr("Unix Generator (%1)").arg(m_kit->displayName());
+        return tr("Unix Generator (%1)").arg(m_kit->displayName());
     }
     return QString();
 }
 
-QList<GeneratorInfo> GeneratorInfo::generatorInfosFor(ProjectExplorer::Kit *k, Ninja n, bool hasCodeBlocks)
+QList<GeneratorInfo> GeneratorInfo::generatorInfosFor(ProjectExplorer::Kit *k, Ninja n, bool preferNinja, bool hasCodeBlocks)
 {
     QList<GeneratorInfo> results;
     ProjectExplorer::ToolChain *tc = ProjectExplorer::ToolChainKitInformation::toolChain(k);
@@ -194,8 +195,12 @@ QList<GeneratorInfo> GeneratorInfo::generatorInfosFor(ProjectExplorer::Kit *k, N
             results << GeneratorInfo(k);
         }
     }
-    if (n != NoNinja)
-        results << GeneratorInfo(k, true);
+    if (n != NoNinja) {
+        if (preferNinja)
+            results.prepend(GeneratorInfo(k, true));
+        else
+            results.append(GeneratorInfo(k, true));
+    }
     return results;
 }
 
@@ -210,6 +215,9 @@ CMakeOpenProjectWizard::CMakeOpenProjectWizard(CMakeManager *cmakeManager, const
       m_useNinja(false),
       m_kit(0)
 {
+    if (!compatibleKitExist())
+        addPage(new NoKitPage(this));
+
     if (hasInSourceBuild()) {
         m_buildDirectory = m_sourceDirectory;
         addPage(new InSourceBuildPage(this));
@@ -268,19 +276,42 @@ CMakeManager *CMakeOpenProjectWizard::cmakeManager() const
 
 bool CMakeOpenProjectWizard::hasInSourceBuild() const
 {
-    QFileInfo fi(m_sourceDirectory + "/CMakeCache.txt");
+    QFileInfo fi(m_sourceDirectory + QLatin1String("/CMakeCache.txt"));
     if (fi.exists())
         return true;
     return false;
 }
 
+bool CMakeOpenProjectWizard::compatibleKitExist() const
+{
+    bool hasCodeBlocksGenerator = m_cmakeManager->hasCodeBlocksMsvcGenerator();
+    bool hasNinjaGenerator = m_cmakeManager->hasCodeBlocksNinjaGenerator();
+    bool preferNinja = m_cmakeManager->preferNinja();
+
+    QList<ProjectExplorer::Kit *> kitList =
+            ProjectExplorer::KitManager::instance()->kits();
+
+    foreach (ProjectExplorer::Kit *k, kitList) {
+        // OfferNinja and ForceNinja differ in what they return
+        // but not whether the list is empty or not, which is what we
+        // are interested in here
+        QList<GeneratorInfo> infos = GeneratorInfo::generatorInfosFor(k,
+                                                                      hasNinjaGenerator ? GeneratorInfo::OfferNinja : GeneratorInfo::NoNinja,
+                                                                      preferNinja,
+                                                                      hasCodeBlocksGenerator);
+        if (!infos.isEmpty())
+            return true;
+    }
+    return false;
+}
+
 bool CMakeOpenProjectWizard::existsUpToDateXmlFile() const
 {
     QString cbpFile = CMakeManager::findCbpFile(QDir(buildDirectory()));
     if (!cbpFile.isEmpty()) {
         // We already have a cbp file
         QFileInfo cbpFileInfo(cbpFile);
-        QFileInfo cmakeListsFileInfo(sourceDirectory() + "/CMakeLists.txt");
+        QFileInfo cmakeListsFileInfo(sourceDirectory() + QLatin1String("/CMakeLists.txt"));
 
         if (cbpFileInfo.lastModified() > cmakeListsFileInfo.lastModified())
             return true;
@@ -338,6 +369,63 @@ void CMakeOpenProjectWizard::setKit(ProjectExplorer::Kit *kit)
     m_kit = kit;
 }
 
+//////
+// NoKitPage
+/////
+
+NoKitPage::NoKitPage(CMakeOpenProjectWizard *cmakeWizard)
+    : QWizardPage(cmakeWizard), m_cmakeWizard(cmakeWizard)
+{
+    QVBoxLayout *layout = new QVBoxLayout;
+    setLayout(layout);
+
+    m_descriptionLabel = new QLabel(this);
+    m_descriptionLabel->setWordWrap(true);
+    layout->addWidget(m_descriptionLabel);
+
+    m_optionsButton = new QPushButton;
+    m_optionsButton->setText(tr("Show Options"));
+
+    connect(m_optionsButton, SIGNAL(clicked()),
+            this, SLOT(showOptions()));
+
+    QHBoxLayout *hbox = new QHBoxLayout;
+    hbox->addWidget(m_optionsButton);
+    hbox->addStretch();
+
+    layout->addLayout(hbox);
+
+    setTitle(tr("Check Kits"));
+
+    connect(ProjectExplorer::KitManager::instance(), SIGNAL(kitsChanged()),
+            this, SLOT(kitsChanged()));
+
+    kitsChanged();
+}
+
+void NoKitPage::kitsChanged()
+{
+    if (isComplete()) {
+        m_descriptionLabel->setText(tr("There are compatible kits."));
+        m_optionsButton->setVisible(false);
+    } else {
+        m_descriptionLabel->setText(tr("Qt Creator has no kits that are suitable for CMake projects. Please configure a kit."));
+        m_optionsButton->setVisible(true);
+    }
+    emit completeChanged();
+}
+
+bool NoKitPage::isComplete() const
+{
+    return m_cmakeWizard->compatibleKitExist();
+}
+
+void NoKitPage::showOptions()
+{
+    Core::ICore::showOptionsDialog(Core::Id(ProjectExplorer::Constants::PROJECTEXPLORER_SETTINGS_CATEGORY),
+                                   Core::Id(ProjectExplorer::Constants::KITS_SETTINGS_PAGE_ID), this);
+}
+
 InSourceBuildPage::InSourceBuildPage(CMakeOpenProjectWizard *cmakeWizard)
     : QWizardPage(cmakeWizard), m_cmakeWizard(cmakeWizard)
 {
@@ -395,31 +483,31 @@ ChooseCMakePage::ChooseCMakePage(CMakeOpenProjectWizard *cmakeWizard)
     // Show a field for the user to enter
     m_cmakeExecutable = new Utils::PathChooser(this);
     m_cmakeExecutable->setExpectedKind(Utils::PathChooser::ExistingCommand);
-    fl->addRow(tr("cmake Executable:"), m_cmakeExecutable);
+    fl->addRow(tr("CMake Executable:"), m_cmakeExecutable);
 
     connect(m_cmakeExecutable, SIGNAL(editingFinished()),
             this, SLOT(cmakeExecutableChanged()));
     connect(m_cmakeExecutable, SIGNAL(browsingFinished()),
             this, SLOT(cmakeExecutableChanged()));
 
-    setTitle(tr("Choose Cmake Executable"));
+    setTitle(tr("Choose CMake Executable"));
 }
 
 void ChooseCMakePage::updateErrorText()
 {
     QString cmakeExecutable = m_cmakeWizard->cmakeManager()->cmakeExecutable();
     if (m_cmakeWizard->cmakeManager()->isCMakeExecutableValid()) {
-        m_cmakeLabel->setText(tr("The cmake executable is valid."));
+        m_cmakeLabel->setText(tr("The CMake executable is valid."));
     } else {
-        QString text = tr("Please specify the path to the cmake executable. No cmake executable was found in the path.");
+        QString text = tr("Specify the path to the CMake executable. No CMake executable was found in the path.");
         if (!cmakeExecutable.isEmpty()) {
             QFileInfo fi(cmakeExecutable);
             if (!fi.exists())
-                text += tr(" The cmake executable (%1) does not exist.").arg(cmakeExecutable);
+                text += tr(" The CMake executable (%1) does not exist.").arg(cmakeExecutable);
             else if (!fi.isExecutable())
-                text += tr(" The path %1 is not a executable.").arg(cmakeExecutable);
+                text += tr(" The path %1 is not an executable.").arg(cmakeExecutable);
             else
-                text += tr(" The path %1 is not a valid cmake.").arg(cmakeExecutable);
+                text += tr(" The path %1 is not a valid CMake executable.").arg(cmakeExecutable);
         }
         m_cmakeLabel->setText(text);
     }
@@ -440,8 +528,7 @@ bool ChooseCMakePage::isComplete() const
 CMakeRunPage::CMakeRunPage(CMakeOpenProjectWizard *cmakeWizard, Mode mode, const QString &buildDirectory)
     : QWizardPage(cmakeWizard),
       m_cmakeWizard(cmakeWizard),
-      m_complete(false),
-      m_optionalCMake(false),
+      m_haveCbpFile(false),
       m_mode(mode),
       m_buildDirectory(buildDirectory)
 {
@@ -462,6 +549,7 @@ void CMakeRunPage::initWidgets()
     // Run CMake Line (with arguments)
     m_argumentsLineEdit = new Utils::FancyLineEdit(this);
     m_argumentsLineEdit->setHistoryCompleter(QLatin1String("CMakeArgumentsLineEdit"));
+    m_argumentsLineEdit->selectAll();
 
     connect(m_argumentsLineEdit,SIGNAL(returnPressed()), this, SLOT(runCMake()));
     fl->addRow(tr("Arguments:"), m_argumentsLineEdit);
@@ -500,18 +588,18 @@ void CMakeRunPage::initWidgets()
     setMinimumSize(600, 400);
 }
 
-QString CMakeRunPage::cachedGeneratorFromFile(const QString &cache)
+QByteArray CMakeRunPage::cachedGeneratorFromFile(const QString &cache)
 {
     QFile fi(cache);
     if (fi.exists()) {
         // Cache exists, then read it...
         if (fi.open(QIODevice::ReadOnly | QIODevice::Text)) {
             while (!fi.atEnd()) {
-                QString line = fi.readLine();
+                QByteArray line = fi.readLine();
                 if (line.startsWith("CMAKE_GENERATOR:INTERNAL=")) {
                     int splitpos = line.indexOf('=');
                     if (splitpos != -1) {
-                        QString cachedGenerator = line.mid(splitpos + 1).trimmed();
+                        QByteArray cachedGenerator = line.mid(splitpos + 1).trimmed();
                         if (!cachedGenerator.isEmpty())
                             return cachedGenerator;
                     }
@@ -519,7 +607,7 @@ QString CMakeRunPage::cachedGeneratorFromFile(const QString &cache)
             }
         }
     }
-    return QString();
+    return QByteArray();
 }
 
 void CMakeRunPage::initializePage()
@@ -533,8 +621,7 @@ void CMakeRunPage::initializePage()
                     tr("The directory %1 already contains a cbp file, which is recent enough. "
                        "You can pass special arguments and rerun CMake. "
                        "Or simply finish the wizard directly.").arg(m_buildDirectory));
-            m_optionalCMake = true;
-            m_complete = true;
+            m_haveCbpFile = true;
         } else {
             m_descriptionLabel->setText(
                     tr("The directory %1 does not contain a cbp file. Qt Creator needs to create this file by running CMake. "
@@ -567,24 +654,32 @@ void CMakeRunPage::initializePage()
 
     bool hasCodeBlocksGenerator = m_cmakeWizard->cmakeManager()->hasCodeBlocksMsvcGenerator();
     bool hasNinjaGenerator = m_cmakeWizard->cmakeManager()->hasCodeBlocksNinjaGenerator();
+    bool preferNinja = m_cmakeWizard->cmakeManager()->preferNinja();
 
     if (m_mode == Initial) {
         // Try figuring out generator and toolchain from CMakeCache.txt
-        QString cachedGenerator = cachedGeneratorFromFile(m_buildDirectory + "/CMakeCache.txt");
+        QByteArray cachedGenerator = cachedGeneratorFromFile(m_buildDirectory + QLatin1String("/CMakeCache.txt"));
 
         m_generatorComboBox->show();
         QList<ProjectExplorer::Kit *> kitList =
                 ProjectExplorer::KitManager::instance()->kits();
+        int defaultIndex = 0;
 
         foreach (ProjectExplorer::Kit *k, kitList) {
             QList<GeneratorInfo> infos = GeneratorInfo::generatorInfosFor(k,
                                                                           hasNinjaGenerator ? GeneratorInfo::OfferNinja : GeneratorInfo::NoNinja,
+                                                                          preferNinja,
                                                                           hasCodeBlocksGenerator);
 
+            if (k == ProjectExplorer::KitManager::instance()->defaultKit())
+                defaultIndex = m_generatorComboBox->count();
+
             foreach (const GeneratorInfo &info, infos)
                 if (cachedGenerator.isEmpty() || info.generator() == cachedGenerator)
                     m_generatorComboBox->addItem(info.displayName(), qVariantFromValue(info));
         }
+
+        m_generatorComboBox->setCurrentIndex(defaultIndex);
     } else {
         // Note: We don't compare the actually cached generator to what is set in the buildconfiguration
         // We assume that the buildconfiguration is correct
@@ -600,29 +695,27 @@ void CMakeRunPage::initializePage()
 
         QList<GeneratorInfo> infos = GeneratorInfo::generatorInfosFor(m_cmakeWizard->kit(),
                                                                       ninja,
+                                                                      preferNinja,
                                                                       true);
         foreach (const GeneratorInfo &info, infos)
-                m_generatorComboBox->addItem(info.displayName(), qVariantFromValue(info));
+            m_generatorComboBox->addItem(info.displayName(), qVariantFromValue(info));
     }
 }
 
 bool CMakeRunPage::validatePage()
 {
-    if (m_optionalCMake) {
-        int index = m_generatorComboBox->currentIndex();
-        if (index == -1)
-            return false;
-        GeneratorInfo generatorInfo = m_generatorComboBox->itemData(index).value<GeneratorInfo>();
-        m_cmakeWizard->setKit(generatorInfo.kit());
-        m_cmakeWizard->setUseNinja(generatorInfo.isNinja());
-    }
+    int index = m_generatorComboBox->currentIndex();
+    if (index == -1)
+        return false;
+    GeneratorInfo generatorInfo = m_generatorComboBox->itemData(index).value<GeneratorInfo>();
+    m_cmakeWizard->setKit(generatorInfo.kit());
+    m_cmakeWizard->setUseNinja(generatorInfo.isNinja());
     return QWizardPage::validatePage();
 }
 
 void CMakeRunPage::runCMake()
 {
-    m_optionalCMake = false;
-    m_complete = false;
+    m_haveCbpFile = false;
 
     Utils::Environment env = m_cmakeWizard->environment();
     int index = m_generatorComboBox->currentIndex();
@@ -653,12 +746,12 @@ void CMakeRunPage::runCMake()
         connect(m_cmakeProcess, SIGNAL(readyReadStandardError()), this, SLOT(cmakeReadyReadStandardError()));
         connect(m_cmakeProcess, SIGNAL(finished(int)), this, SLOT(cmakeFinished()));
         cmakeManager->createXmlFile(m_cmakeProcess, m_argumentsLineEdit->text(), m_cmakeWizard->sourceDirectory(),
-                                    m_buildDirectory, env, generatorInfo.generatorArgument());
+                                    m_buildDirectory, env, QString::fromLatin1(generatorInfo.generatorArgument()));
     } else {
         m_runCMake->setEnabled(true);
         m_argumentsLineEdit->setEnabled(true);
         m_generatorComboBox->setEnabled(true);
-        m_output->appendPlainText(tr("No valid cmake executable specified."));
+        m_output->appendPlainText(tr("No valid CMake executable specified."));
     }
 }
 
@@ -678,7 +771,7 @@ void CMakeRunPage::cmakeReadyReadStandardOutput()
     tf.setFont(font);
     tf.setForeground(m_output->palette().color(QPalette::Text));
 
-    cursor.insertText(m_cmakeProcess->readAllStandardOutput(), tf);
+    cursor.insertText(QString::fromLocal8Bit(m_cmakeProcess->readAllStandardOutput()), tf);
 }
 
 void CMakeRunPage::cmakeReadyReadStandardError()
@@ -692,7 +785,7 @@ void CMakeRunPage::cmakeReadyReadStandardError()
     tf.setFont(boldFont);
     tf.setForeground(mix_colors(m_output->palette().color(QPalette::Text), QColor(Qt::red)));
 
-    cursor.insertText(m_cmakeProcess->readAllStandardError(), tf);
+    cursor.insertText(QString::fromLocal8Bit(m_cmakeProcess->readAllStandardError()), tf);
 }
 
 void CMakeRunPage::cmakeFinished()
@@ -704,10 +797,11 @@ void CMakeRunPage::cmakeFinished()
     if (m_cmakeProcess->exitCode() != 0) {
         m_exitCodeLabel->setVisible(true);
         m_exitCodeLabel->setText(tr("CMake exited with errors. Please check CMake output."));
-        m_complete = false;
+        static_cast<Utils::HistoryCompleter *>(m_argumentsLineEdit->completer())->removeHistoryItem(0);
+        m_haveCbpFile = false;
     } else {
         m_exitCodeLabel->setVisible(false);
-        m_complete = true;
+        m_haveCbpFile = true;
     }
     m_cmakeProcess->deleteLater();
     m_cmakeProcess = 0;
@@ -718,12 +812,13 @@ void CMakeRunPage::cmakeFinished()
 void CMakeRunPage::cleanupPage()
 {
     m_output->clear();
-    m_complete = false;
+    m_haveCbpFile = false;
     m_exitCodeLabel->setVisible(false);
     emit completeChanged();
 }
 
 bool CMakeRunPage::isComplete() const
 {
-    return m_complete;
+    int index = m_generatorComboBox->currentIndex();
+    return index != -1 && m_haveCbpFile;
 }
