@@ -1,37 +1,35 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "actioncontainer_p.h"
 #include "actionmanager_p.h"
+#include "actionmanager.h"
 
 #include "command_p.h"
 
@@ -293,6 +291,30 @@ void ActionContainerPrivate::addMenu(ActionContainer *before, ActionContainer *m
     scheduleUpdate();
 }
 
+/*!
+ * \fn Command *ActionContainer::addSeparator(const Context &context, const Id &group, QAction **outSeparator)
+ *
+ * Adds a separator to the end of the given \a group to the action container, which is enabled
+ * for a given \a context. The created separator action is returned through \a outSeparator.
+ *
+ * Returns the created Command for the separator.
+ */
+/*! \a context \a group \a outSeparator
+ * \internal
+ */
+Command *ActionContainerPrivate::addSeparator(const Context &context, const Id &group, QAction **outSeparator)
+{
+    static int separatorIdCount = 0;
+    QAction *separator = new QAction(this);
+    separator->setSeparator(true);
+    Id sepId = id().withSuffix(".Separator.").withSuffix(++separatorIdCount);
+    Command *cmd = ActionManager::registerAction(separator, sepId, context);
+    addAction(cmd, group);
+    if (outSeparator)
+        *outSeparator = separator;
+    return cmd;
+}
+
 void ActionContainerPrivate::clear()
 {
     QMutableListIterator<Group> it(m_groups);
@@ -402,11 +424,29 @@ void MenuActionContainer::removeMenu(QMenu *menu)
     m_menu->removeAction(menu->menuAction());
 }
 
+#ifdef Q_OS_MAC
+static bool menuInMenuBar(const QMenu *menu)
+{
+    foreach (const QWidget *widget, menu->menuAction()->associatedWidgets()) {
+        if (qobject_cast<const QMenuBar *>(widget))
+            return true;
+    }
+    return false;
+}
+#endif
+
 bool MenuActionContainer::updateInternal()
 {
     if (onAllDisabledBehavior() == Show)
         return true;
 
+#ifdef Q_OS_MAC
+    // work around QTBUG-25544 which makes menus in the menu bar stay at their enabled state at startup
+    // (so menus that are disabled at startup would stay disabled)
+    if (menuInMenuBar(m_menu))
+        return true;
+#endif
+
     bool hasitems = false;
     QList<QAction *> actions = m_menu->actions();
 
