@@ -1,56 +1,56 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (info@qt.nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at info@qt.nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #ifndef SEARCHSYMBOLS_H
 #define SEARCHSYMBOLS_H
 
+#include "cpptools_global.h"
+
+#include "cppindexingsupport.h"
+
 #include <cplusplus/CppDocument.h>
 #include <cplusplus/Icons.h>
 #include <cplusplus/Overview.h>
 #include <Symbols.h>
 #include <SymbolVisitor.h>
 
-#include <QtGui/QIcon>
-#include <QtCore/QMetaType>
-#include <QtCore/QString>
-#include <QtCore/QSet>
-#include <QtCore/QHash>
+#include <QIcon>
+#include <QMetaType>
+#include <QString>
+#include <QSet>
+#include <QHash>
 
 #include <functional>
 
 namespace CppTools {
-namespace Internal {
 
-struct ModelItemInfo
+struct CPPTOOLS_EXPORT ModelItemInfo
 {
     enum ItemType { Enum, Class, Method, Declaration };
 
@@ -99,18 +99,11 @@ struct ModelItemInfo
     int column;
 };
 
-class SearchSymbols: public std::unary_function<CPlusPlus::Document::Ptr, QList<ModelItemInfo> >,
+class SearchSymbols: public std::binary_function<CPlusPlus::Document::Ptr, int, QList<ModelItemInfo> >,
                      protected CPlusPlus::SymbolVisitor
 {
 public:
-    enum SymbolType {
-        Classes      = 0x1,
-        Functions    = 0x2,
-        Enums        = 0x4,
-        Declarations = 0x8
-    };
-
-    Q_DECLARE_FLAGS(SymbolTypes, SymbolType)
+    typedef SymbolSearcher::SymbolTypes SymbolTypes;
 
     static SymbolTypes AllTypes;
 
@@ -119,10 +112,10 @@ public:
     void setSymbolsToSearchFor(SymbolTypes types);
     void setSeparateScope(bool separateScope);
 
-    QList<ModelItemInfo> operator()(CPlusPlus::Document::Ptr doc)
-    { return operator()(doc, QString()); }
+    QList<ModelItemInfo> operator()(CPlusPlus::Document::Ptr doc, int sizeHint = 500)
+    { return operator()(doc, sizeHint, QString()); }
 
-    QList<ModelItemInfo> operator()(CPlusPlus::Document::Ptr doc, const QString &scope);
+    QList<ModelItemInfo> operator()(CPlusPlus::Document::Ptr doc, int sizeHint, const QString &scope);
 
 protected:
     using SymbolVisitor::visit;
@@ -180,10 +173,9 @@ private:
     bool separateScope;
 };
 
-} // namespace Internal
 } // namespace CppTools
 
-Q_DECLARE_OPERATORS_FOR_FLAGS(CppTools::Internal::SearchSymbols::SymbolTypes)
-Q_DECLARE_METATYPE(CppTools::Internal::ModelItemInfo)
+Q_DECLARE_OPERATORS_FOR_FLAGS(CppTools::SearchSymbols::SymbolTypes)
+Q_DECLARE_METATYPE(CppTools::ModelItemInfo)
 
 #endif // SEARCHSYMBOLS_H
