@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -70,9 +70,8 @@ void FakeVimSettings::insertItem(int code, SavedAction *item,
         m_nameToCode[longName] = code;
         m_codeToName[code] = longName;
     }
-    if (!shortName.isEmpty()) {
+    if (!shortName.isEmpty())
         m_nameToCode[shortName] = code;
-    }
 }
 
 void FakeVimSettings::readSettings(QSettings *settings)
@@ -217,6 +216,13 @@ FakeVimSettings *theFakeVimSettings()
     item->setCheckable(true);
     instance->insertItem(ConfigSmartCase, item, _("smartcase"), _("scs"));
 
+    item = new SavedAction(instance);
+    item->setDefaultValue(true);
+    item->setValue(true);
+    item->setSettingsKey(group, _("WrapScan")); item->setCheckable(true);
+    item->setCheckable(true);
+    instance->insertItem(ConfigWrapScan, item, _("wrapscan"), _("ws"));
+
     item = new SavedAction(instance);
     item->setDefaultValue(_("indent,eol,start"));
     item->setSettingsKey(group, _("Backspace"));
@@ -249,6 +255,13 @@ FakeVimSettings *theFakeVimSettings()
     item->setCheckable(true);
     instance->insertItem(ConfigClipboard, item, _("clipboard"), _("cb"));
 
+    item = new SavedAction(instance);
+    item->setDefaultValue(true);
+    item->setValue(true);
+    item->setSettingsKey(group, _("ShowCmd")); item->setCheckable(true);
+    item->setCheckable(true);
+    instance->insertItem(ConfigShowCmd, item, _("showcmd"), _("sc"));
+
     return instance;
 }
 
