@@ -69,6 +69,7 @@
 
 #include <find/findplugin.h>
 #include <find/textfindconstants.h>
+#include <find/ifindsupport.h>
 
 #include <utils/qtcassert.h>
 #include <utils/savedaction.h>
@@ -77,12 +78,15 @@
 
 #include <cpptools/cpptoolsconstants.h>
 
+#include <extensionsystem/pluginmanager.h>
+
 #include <QAbstractTableModel>
 #include <QDebug>
 #include <QFile>
 #include <QtPlugin>
 #include <QObject>
 #include <QSettings>
+#include <QStackedWidget>
 #include <QTextStream>
 
 #include <QDesktopServices>
@@ -107,50 +111,115 @@ const char SETTINGS_CATEGORY_FAKEVIM_ICON[] = ":/core/images/category_fakevim.pn
 const char SETTINGS_ID[]                    = "A.General";
 const char SETTINGS_EX_CMDS_ID[]            = "B.ExCommands";
 const char SETTINGS_USER_CMDS_ID[]          = "C.UserCommands";
+typedef QLatin1String _;
 
-class MiniBuffer : public QLabel
+class MiniBuffer : public QStackedWidget
 {
     Q_OBJECT
 
 public:
-    void setContents(const QString &contents, int cursorPos)
+    MiniBuffer() : m_label(new QLabel(this)), m_edit(new QLineEdit(this)), m_eventFilter(0)
     {
-        QString msg = contents;
-        if (cursorPos != -1)
-            msg = contents.left(cursorPos) + QChar(10073) + contents.mid(cursorPos);
-        setText("  " + msg);
-    }
-};
+        m_edit->installEventFilter(this);
+        connect(m_edit, SIGNAL(textEdited(QString)), SLOT(changed()));
+        connect(m_edit, SIGNAL(cursorPositionChanged(int,int)), SLOT(changed()));
+        connect(m_edit, SIGNAL(selectionChanged()), SLOT(changed()));
+        m_label->setTextInteractionFlags(Qt::TextSelectableByMouse);
 
-class MiniBuffer1 : public QLineEdit
-{
-    Q_OBJECT
+        addWidget(m_label);
+        addWidget(m_edit);
+    }
 
-public:
-    MiniBuffer1()
+    void setContents(const QString &contents, int cursorPos, int anchorPos,
+                     int messageLevel, QObject *eventFilter)
     {
-        setFrame(false);
+        if (cursorPos != -1) {
+            m_edit->blockSignals(true);
+            m_label->clear();
+            m_edit->setText(contents);
+            if (anchorPos != -1 && anchorPos != cursorPos)
+                m_edit->setSelection(anchorPos, cursorPos - anchorPos);
+            else
+                m_edit->setCursorPosition(cursorPos);
+            m_edit->blockSignals(false);
+            setCurrentWidget(m_edit);
+            m_edit->setFocus();
+        } else if (contents.isEmpty() && messageLevel != MessageShowCmd) {
+            hide();
+        } else {
+            show();
+            m_label->setText(messageLevel == MessageMode ? _("-- ") + contents + _(" --") : contents);
+
+            QString css;
+            if (messageLevel == MessageError) {
+                css = _("border:1px solid rgba(255,255,255,150);"
+                        "background-color:rgba(255,0,0,100);");
+            } else if (messageLevel == MessageWarning) {
+                css = _("border:1px solid rgba(255,255,255,120);"
+                        "background-color:rgba(255,255,0,20);");
+            } else if (messageLevel == MessageShowCmd) {
+                css = _("border:1px solid rgba(255,255,255,120);"
+                        "background-color:rgba(100,255,100,30);");
+            }
+            m_label->setStyleSheet(QString::fromLatin1(
+                "*{border-radius:2px;padding-left:4px;padding-right:4px;%1}").arg(css));
+
+            if (m_edit->hasFocus())
+                emit edited(QString(), -1, -1);
+
+            setCurrentWidget(m_label);
+        }
+
+        if (m_eventFilter != eventFilter) {
+            if (m_eventFilter != 0) {
+                m_edit->removeEventFilter(m_eventFilter);
+                disconnect(SIGNAL(edited(QString,int,int)));
+            }
+            if (eventFilter != 0) {
+                m_edit->installEventFilter(eventFilter);
+                connect(this, SIGNAL(edited(QString,int,int)),
+                        eventFilter, SLOT(miniBufferTextEdited(QString,int,int)));
+            }
+            m_eventFilter = eventFilter;
+        }
     }
-    void showEvent(QShowEvent *ev)
+
+    QSize sizeHint() const
     {
-        QLineEdit::showEvent(ev);
-        QColor color = Qt::black;
-        QPalette pal = parentWidget()->palette();
-        pal.setBrush(QPalette::All, QPalette::WindowText, color);
-        pal.setBrush(QPalette::All, QPalette::ButtonText, color);
-        pal.setBrush(QPalette::All, QPalette::Foreground, color);
-        pal.setBrush(QPalette::All, QPalette::Background, color);
-        //color.setAlpha(100);
-        //pal.setBrush(QPalette::Disabled, QPalette::WindowText, color);
-        //pal.setBrush(QPalette::Disabled, QPalette::ButtonText, color);
-        //pal.setBrush(QPalette::Disabled, QPalette::Foreground, color);
-        setPalette(pal);
+        QSize size = QWidget::sizeHint();
+        // reserve maximal width for line edit widget
+        return currentWidget() == m_edit ? QSize(maximumWidth(), size.height()) : size;
     }
-    void setContents(const QString &contents, int cursorPos)
+
+signals:
+    void edited(const QString &text, int cursorPos, int anchorPos);
+
+private slots:
+    void changed()
     {
-        setText(contents);
-        setCursorPosition(cursorPos);
+        const int cursorPos = m_edit->cursorPosition();
+        int anchorPos = m_edit->selectionStart();
+        if (anchorPos == cursorPos)
+            anchorPos = cursorPos + m_edit->selectedText().length();
+        emit edited(m_edit->text(), cursorPos, anchorPos);
     }
+
+    bool eventFilter(QObject *ob, QEvent *ev)
+    {
+        // cancel editing on escape
+        if (m_eventFilter != 0 && ob == m_edit && ev->type() == QEvent::ShortcutOverride
+            && static_cast<QKeyEvent*>(ev)->key() == Qt::Key_Escape) {
+            emit edited(QString(), -1, -1);
+            ev->accept();
+            return true;
+        }
+        return false;
+    }
+
+private:
+    QLabel *m_label;
+    QLineEdit *m_edit;
+    QObject *m_eventFilter;
 };
 
 ///////////////////////////////////////////////////////////////////////
@@ -161,7 +230,6 @@ public:
 
 typedef QMap<QString, QRegExp> ExCommandMap;
 typedef QMap<int, QString> UserCommandMap;
-typedef QLatin1String _;
 
 class FakeVimOptionPage : public IOptionsPage
 {
@@ -170,9 +238,9 @@ class FakeVimOptionPage : public IOptionsPage
 public:
     FakeVimOptionPage()
     {
-        setId(_(SETTINGS_ID));
+        setId(SETTINGS_ID);
         setDisplayName(tr("General"));
-        setCategory(_(SETTINGS_CATEGORY));
+        setCategory(SETTINGS_CATEGORY);
         setDisplayCategory(tr("FakeVim"));
         setCategoryIcon(_(SETTINGS_CATEGORY_FAKEVIM_ICON));
     }
@@ -231,12 +299,18 @@ QWidget *FakeVimOptionPage::createPage(QWidget *parent)
         m_ui.checkBoxAutoIndent);
     m_group.insert(theFakeVimSetting(ConfigSmartIndent),
         m_ui.checkBoxSmartIndent);
+
     m_group.insert(theFakeVimSetting(ConfigIncSearch),
         m_ui.checkBoxIncSearch);
     m_group.insert(theFakeVimSetting(ConfigUseCoreSearch),
         m_ui.checkBoxUseCoreSearch);
     m_group.insert(theFakeVimSetting(ConfigSmartCase),
         m_ui.checkBoxSmartCase);
+    m_group.insert(theFakeVimSetting(ConfigWrapScan),
+        m_ui.checkBoxWrapScan);
+
+    m_group.insert(theFakeVimSetting(ConfigShowCmd),
+        m_ui.checkBoxShowCmd);
 
     connect(m_ui.pushButtonCopyTextEditorSettings, SIGNAL(clicked()),
         SLOT(copyTextEditorSettings()));
@@ -331,9 +405,9 @@ public:
     FakeVimExCommandsPage(FakeVimPluginPrivate *q)
         : m_q(q)
     {
-        setId(_(SETTINGS_EX_CMDS_ID));
+        setId(SETTINGS_EX_CMDS_ID);
         setDisplayName(tr("Ex Command Mapping"));
-        setCategory(_(SETTINGS_CATEGORY));
+        setCategory(SETTINGS_CATEGORY);
         setDisplayCategory(tr("FakeVim"));
         setCategoryIcon(_(SETTINGS_CATEGORY_FAKEVIM_ICON));
     }
@@ -553,9 +627,9 @@ public:
     FakeVimUserCommandsPage(FakeVimPluginPrivate *q)
         : m_q(q)
     {
-        setId(_(SETTINGS_USER_CMDS_ID));
+        setId(SETTINGS_USER_CMDS_ID);
         setDisplayName(tr("User Command Mapping"));
-        setCategory(_(SETTINGS_CATEGORY));
+        setCategory(SETTINGS_CATEGORY);
         setDisplayCategory(tr("FakeVim"));
         setCategoryIcon(_(SETTINGS_CATEGORY_FAKEVIM_ICON));
     }
@@ -786,17 +860,25 @@ private slots:
     void windowCommand(int key);
     void find(bool reverse);
     void findNext(bool reverse);
+    void foldToggle(int depth);
+    void foldAll(bool fold);
+    void fold(int depth, bool fold);
+    void foldGoTo(int count, bool current);
+    void jumpToGlobalMark(QChar mark, bool backTickMode, const QString &fileName);
     void showSettingsDialog();
     void maybeReadVimRc();
     void setBlockSelection(bool);
     void hasBlockSelection(bool*);
 
-    void showCommandBuffer(const QString &contents, int cursorPos);
+    void resetCommandBuffer();
+    void showCommandBuffer(const QString &contents, int cursorPos, int anchorPos,
+                           int messageLevel, QObject *eventFilter);
     void showExtraInformation(const QString &msg);
     void changeSelection(const QList<QTextEdit::ExtraSelection> &selections);
+    void highlightMatches(const QString &needle);
     void moveToMatchingParenthesis(bool *moved, bool *forward, QTextCursor *cursor);
     void checkForElectricCharacter(bool *result, QChar c);
-    void indentRegion(int beginLine, int endLine, QChar typedChar);
+    void indentRegion(int beginBlock, int endBlock, QChar typedChar);
     void handleExCommand(bool *handled, const ExCommand &cmd);
 
     void writeSettings();
@@ -874,18 +956,18 @@ FakeVimPluginPrivate::FakeVimPluginPrivate(FakeVimPlugin *plugin)
     m_fakeVimOptionsPage = 0;
     m_fakeVimExCommandsPage = 0;
     m_fakeVimUserCommandsPage = 0;
-    defaultExCommandMap()[CppTools::Constants::SWITCH_HEADER_SOURCE] =
-        QRegExp("^A$");
-    defaultExCommandMap()["Coreplugin.OutputPane.previtem"] =
-        QRegExp("^(cN(ext)?|cp(revious)?)!?( (.*))?$");
-    defaultExCommandMap()["Coreplugin.OutputPane.nextitem"] =
-        QRegExp("^cn(ext)?!?( (.*))?$");
-    defaultExCommandMap()[TextEditor::Constants::FOLLOW_SYMBOL_UNDER_CURSOR] =
-        QRegExp("^tag?$");
-    defaultExCommandMap()[Core::Constants::GO_BACK] =
-        QRegExp("^pop?$");
+    defaultExCommandMap()[_(CppTools::Constants::SWITCH_HEADER_SOURCE)] =
+        QRegExp(_("^A$"));
+    defaultExCommandMap()[_("Coreplugin.OutputPane.previtem")] =
+        QRegExp(_("^(cN(ext)?|cp(revious)?)!?( (.*))?$"));
+    defaultExCommandMap()[_("Coreplugin.OutputPane.nextitem")] =
+        QRegExp(_("^cn(ext)?!?( (.*))?$"));
+    defaultExCommandMap()[_(TextEditor::Constants::FOLLOW_SYMBOL_UNDER_CURSOR)] =
+        QRegExp(_("^tag?$"));
+    defaultExCommandMap()[_(Core::Constants::GO_BACK)] =
+        QRegExp(_("^pop?$"));
     defaultExCommandMap()[_("QtCreator.Locate")] =
-        QRegExp("^e$");
+        QRegExp(_("^e$"));
 
     for (int i = 1; i < 10; ++i) {
         QString cmd = QString::fromLatin1(":echo User command %1 executed.<CR>");
@@ -966,7 +1048,7 @@ bool FakeVimPluginPrivate::initialize()
         QAction *act = new QAction(this);
         act->setText(tr("Execute User Action #%1").arg(i));
         act->setData(i);
-        QString id = QString("FakeVim.UserAction%1").arg(i);
+        QString id = QString::fromLatin1("FakeVim.UserAction%1").arg(i);
         cmd = ActionManager::registerAction(act, Id(id), globalcontext);
         cmd->setDefaultKeySequence(QKeySequence((UseMacShortcuts ? tr("Meta+V,%1") : tr("Alt+V,%1")).arg(i)));
         connect(act, SIGNAL(triggered()), SLOT(userActionTriggered()));
@@ -990,7 +1072,10 @@ bool FakeVimPluginPrivate::initialize()
         this, SLOT(handleDelayedQuit(bool,Core::IEditor*)), Qt::QueuedConnection);
     connect(this, SIGNAL(delayedQuitAllRequested(bool)),
         this, SLOT(handleDelayedQuitAll(bool)), Qt::QueuedConnection);
-    maybeReadVimRc();
+
+    // Vimrc can break test so don't source it if running tests.
+    if (!ExtensionSystem::PluginManager::runningTests())
+        maybeReadVimRc();
     //    << "MODE: " << theFakeVimSetting(ConfigUseFakeVim)->value();
 
     return true;
@@ -1102,19 +1187,19 @@ void FakeVimPluginPrivate::maybeReadVimRc()
         return;
     QString fileName =
         QDesktopServices::storageLocation(QDesktopServices::HomeLocation)
-            + "/.vimrc";
+            + _("/.vimrc");
     //qDebug() << "READING VIMRC: " << fileName;
     // Read it into a temporary handler for effects modifying global state.
     QPlainTextEdit editor;
     FakeVimHandler handler(&editor);
-    handler.handleCommand("source " + fileName);
+    handler.handleCommand(_("source ") + fileName);
     //writeSettings();
     //qDebug() << theFakeVimSetting(ConfigShiftWidth)->value();
 }
 
 void FakeVimPluginPrivate::showSettingsDialog()
 {
-    ICore::showOptionsDialog(_(SETTINGS_CATEGORY), _(SETTINGS_ID));
+    ICore::showOptionsDialog(SETTINGS_CATEGORY, SETTINGS_ID);
 }
 
 void FakeVimPluginPrivate::triggerAction(const Id &id)
@@ -1285,6 +1370,164 @@ void FakeVimPluginPrivate::findNext(bool reverse)
         triggerAction(Find::Constants::FIND_NEXT);
 }
 
+void FakeVimPluginPrivate::foldToggle(int depth)
+{
+    IEditor *ieditor = EditorManager::currentEditor();
+    BaseTextEditorWidget *editor = qobject_cast<BaseTextEditorWidget *>(ieditor->widget());
+    QTC_ASSERT(editor != 0, return);
+
+    QTextBlock block = editor->textCursor().block();
+    fold(depth, !BaseTextDocumentLayout::isFolded(block));
+}
+
+void FakeVimPluginPrivate::foldAll(bool fold)
+{
+    IEditor *ieditor = EditorManager::currentEditor();
+    BaseTextEditorWidget *editor = qobject_cast<BaseTextEditorWidget *>(ieditor->widget());
+    QTC_ASSERT(editor != 0, return);
+
+    QTextDocument *doc = editor->document();
+    BaseTextDocumentLayout *documentLayout =
+            qobject_cast<BaseTextDocumentLayout*>(doc->documentLayout());
+    QTC_ASSERT(documentLayout != 0, return);
+
+    QTextBlock block = editor->document()->firstBlock();
+    while (block.isValid()) {
+        BaseTextDocumentLayout::doFoldOrUnfold(block, !fold);
+        block = block.next();
+    }
+
+    documentLayout->requestUpdate();
+    documentLayout->emitDocumentSizeChanged();
+}
+
+void FakeVimPluginPrivate::fold(int depth, bool fold)
+{
+    IEditor *ieditor = EditorManager::currentEditor();
+    BaseTextEditorWidget *editor = qobject_cast<BaseTextEditorWidget *>(ieditor->widget());
+    QTC_ASSERT(editor != 0, return);
+
+    QTextDocument *doc = editor->document();
+    BaseTextDocumentLayout *documentLayout =
+            qobject_cast<BaseTextDocumentLayout*>(doc->documentLayout());
+    QTC_ASSERT(documentLayout != 0, return);
+
+    QTextBlock block = editor->textCursor().block();
+    int indent = BaseTextDocumentLayout::foldingIndent(block);
+    if (fold) {
+        if (BaseTextDocumentLayout::isFolded(block)) {
+            while (block.isValid() && (BaseTextDocumentLayout::foldingIndent(block) >= indent
+                || !block.isVisible())) {
+                block = block.previous();
+            }
+        }
+        if (BaseTextDocumentLayout::canFold(block))
+            ++indent;
+        while (depth != 0 && block.isValid()) {
+            const int indent2 = BaseTextDocumentLayout::foldingIndent(block);
+            if (BaseTextDocumentLayout::canFold(block) && indent2 < indent) {
+                BaseTextDocumentLayout::doFoldOrUnfold(block, false);
+                if (depth > 0)
+                    --depth;
+                indent = indent2;
+            }
+            block = block.previous();
+        }
+    } else {
+        if (BaseTextDocumentLayout::isFolded(block)) {
+            if (depth < 0) {
+                // recursively open fold
+                while (depth < 0 && block.isValid()
+                    && BaseTextDocumentLayout::foldingIndent(block) >= indent) {
+                    if (BaseTextDocumentLayout::canFold(block)) {
+                        BaseTextDocumentLayout::doFoldOrUnfold(block, true);
+                        if (depth > 0)
+                            --depth;
+                    }
+                    block = block.next();
+                }
+            } else {
+                if (BaseTextDocumentLayout::canFold(block)) {
+                    BaseTextDocumentLayout::doFoldOrUnfold(block, true);
+                    if (depth > 0)
+                        --depth;
+                }
+            }
+        }
+    }
+
+    documentLayout->requestUpdate();
+    documentLayout->emitDocumentSizeChanged();
+}
+
+void FakeVimPluginPrivate::foldGoTo(int count, bool current)
+{
+    IEditor *ieditor = EditorManager::currentEditor();
+    BaseTextEditorWidget *editor = qobject_cast<BaseTextEditorWidget *>(ieditor->widget());
+    QTC_ASSERT(editor != 0, return);
+
+    QTextCursor tc = editor->textCursor();
+    QTextBlock block = tc.block();
+
+    int pos = -1;
+    if (count > 0) {
+        int repeat = count;
+        block = block.next();
+        QTextBlock prevBlock = block;
+        int indent = BaseTextDocumentLayout::foldingIndent(block);
+        block = block.next();
+        while (block.isValid()) {
+            int newIndent = BaseTextDocumentLayout::foldingIndent(block);
+            if (current ? indent > newIndent : indent < newIndent) {
+                if (prevBlock.isVisible()) {
+                    pos = prevBlock.position();
+                    if (--repeat <= 0)
+                        break;
+                } else if (current) {
+                    indent = newIndent;
+                }
+            }
+            if (!current)
+                indent = newIndent;
+            prevBlock = block;
+            block = block.next();
+        }
+    } else if (count < 0) {
+        int repeat = -count;
+        int indent = BaseTextDocumentLayout::foldingIndent(block);
+        block = block.previous();
+        while (block.isValid()) {
+            int newIndent = BaseTextDocumentLayout::foldingIndent(block);
+            if (current ? indent > newIndent : indent < newIndent) {
+                while (block.isValid() && !block.isVisible())
+                    block = block.previous();
+                pos = block.position();
+                if (--repeat <= 0)
+                    break;
+            }
+            if (!current)
+                indent = newIndent;
+            block = block.previous();
+        }
+    }
+
+    if (pos != -1) {
+        tc.setPosition(pos, QTextCursor::KeepAnchor);
+        editor->setTextCursor(tc);
+    }
+}
+
+void FakeVimPluginPrivate::jumpToGlobalMark(QChar mark, bool backTickMode,
+    const QString &fileName)
+{
+    Core::IEditor *iedit = Core::EditorManager::openEditor(fileName);
+    if (!iedit)
+        return;
+    FakeVimHandler *handler = m_editorToHandler.value(iedit, 0);
+    if (handler)
+        handler->jumpToLocalMark(mark, backTickMode);
+}
+
 // This class defers deletion of a child FakeVimHandler using deleteLater().
 class DeferredDeleter : public QObject
 {
@@ -1331,10 +1574,12 @@ void FakeVimPluginPrivate::editorOpened(IEditor *editor)
 
     connect(handler, SIGNAL(extraInformationChanged(QString)),
         SLOT(showExtraInformation(QString)));
-    connect(handler, SIGNAL(commandBufferChanged(QString,int)),
-        SLOT(showCommandBuffer(QString,int)));
+    connect(handler, SIGNAL(commandBufferChanged(QString,int,int,int,QObject*)),
+        SLOT(showCommandBuffer(QString,int,int,int,QObject*)));
     connect(handler, SIGNAL(selectionChanged(QList<QTextEdit::ExtraSelection>)),
         SLOT(changeSelection(QList<QTextEdit::ExtraSelection>)));
+    connect(handler, SIGNAL(highlightMatches(QString)),
+        SLOT(highlightMatches(QString)));
     connect(handler, SIGNAL(moveToMatchingParenthesis(bool*,bool*,QTextCursor*)),
         SLOT(moveToMatchingParenthesis(bool*,bool*,QTextCursor*)));
     connect(handler, SIGNAL(indentRegion(int,int,QChar)),
@@ -1355,6 +1600,16 @@ void FakeVimPluginPrivate::editorOpened(IEditor *editor)
         SLOT(find(bool)));
     connect(handler, SIGNAL(findNextRequested(bool)),
         SLOT(findNext(bool)));
+    connect(handler, SIGNAL(foldToggle(int)),
+        SLOT(foldToggle(int)));
+    connect(handler, SIGNAL(foldAll(bool)),
+        SLOT(foldAll(bool)));
+    connect(handler, SIGNAL(fold(int,bool)),
+        SLOT(fold(int,bool)));
+    connect(handler, SIGNAL(foldGoTo(int, bool)),
+        SLOT(foldGoTo(int, bool)));
+    connect(handler, SIGNAL(jumpToGlobalMark(QChar,bool,QString)),
+        SLOT(jumpToGlobalMark(QChar,bool,QString)));
 
     connect(handler, SIGNAL(handleExCommandRequested(bool*,ExCommand)),
         SLOT(handleExCommand(bool*,ExCommand)));
@@ -1367,7 +1622,7 @@ void FakeVimPluginPrivate::editorOpened(IEditor *editor)
 
     // pop up the bar
     if (theFakeVimSetting(ConfigUseFakeVim)->value().toBool()) {
-       showCommandBuffer(QString(), -1);
+       resetCommandBuffer();
        handler->setupWidget();
     }
 }
@@ -1399,7 +1654,7 @@ void FakeVimPluginPrivate::setUseFakeVimInternal(bool on)
         //ICore *core = ICore::instance();
         //core->updateAdditionalContexts(Context(),
         // Context(FAKEVIM_CONTEXT));
-        showCommandBuffer(QString(), -1);
+        resetCommandBuffer();
         foreach (IEditor *editor, m_editorToHandler.keys()) {
             if (BaseTextEditorWidget *textEditor =
                     qobject_cast<BaseTextEditorWidget *>(editor->widget())) {
@@ -1465,7 +1720,7 @@ void FakeVimPluginPrivate::handleExCommand(bool *handled, const ExCommand &cmd)
         return;
 
     *handled = true;
-    if (cmd.matches("w", "write") || cmd.cmd == "wq") {
+    if (cmd.matches(_("w"), _("write")) || cmd.cmd == _("wq")) {
         // :w[rite]
         IEditor *editor = m_editorToHandler.key(handler);
         const QString fileName = handler->currentFileName();
@@ -1476,70 +1731,70 @@ void FakeVimPluginPrivate::handleExCommand(bool *handled, const ExCommand &cmd)
             QFile file3(fileName);
             file3.open(QIODevice::ReadOnly);
             QByteArray ba = file3.readAll();
-            handler->showBlackMessage(FakeVimHandler::tr("\"%1\" %2 %3L, %4C written")
-                .arg(fileName).arg(" ")
+            handler->showMessage(MessageInfo, FakeVimHandler::tr("\"%1\" %2 %3L, %4C written")
+                .arg(fileName).arg(QLatin1Char(' '))
                 .arg(ba.count('\n')).arg(ba.size()));
-            if (cmd.cmd == "wq")
+            if (cmd.cmd == _("wq"))
                 delayedQuitRequested(cmd.hasBang, m_editorToHandler.key(handler));
         } else {
-            handler->showRedMessage(tr("File not saved"));
+            handler->showMessage(MessageError, tr("File not saved"));
         }
-    } else if (cmd.matches("wa", "wall")) {
+    } else if (cmd.matches(_("wa"), _("wall"))) {
         // :w[all]
         QList<IDocument *> toSave = DocumentManager::modifiedDocuments();
         QList<IDocument *> failed = DocumentManager::saveModifiedDocumentsSilently(toSave);
         if (failed.isEmpty())
-            handler->showBlackMessage(tr("Saving succeeded"));
+            handler->showMessage(MessageInfo, tr("Saving succeeded"));
         else
-            handler->showRedMessage(tr("%n files not saved", 0, failed.size()));
-    } else if (cmd.matches("q", "quit")) {
+            handler->showMessage(MessageError, tr("%n files not saved", 0, failed.size()));
+    } else if (cmd.matches(_("q"), _("quit"))) {
         // :q[uit]
         emit delayedQuitRequested(cmd.hasBang, m_editorToHandler.key(handler));
-    } else if (cmd.matches("qa", "qall")) {
+    } else if (cmd.matches(_("qa"), _("qall"))) {
         // :qa[ll]
         emit delayedQuitAllRequested(cmd.hasBang);
-    } else if (cmd.matches("sp", "split")) {
+    } else if (cmd.matches(_("sp"), _("split"))) {
         // :sp[lit]
         triggerAction(Core::Constants::SPLIT);
-    } else if (cmd.matches("vs", "vsplit")) {
+    } else if (cmd.matches(_("vs"), _("vsplit"))) {
         // :vs[plit]
         triggerAction(Core::Constants::SPLIT_SIDE_BY_SIDE);
-    } else if (cmd.matches("mak", "make")) {
+    } else if (cmd.matches(_("mak"), _("make"))) {
         // :mak[e][!] [arguments]
         triggerAction(ProjectExplorer::Constants::BUILD);
-    } else if (cmd.matches("se", "set")) {
+    } else if (cmd.matches(_("se"), _("set"))) {
         if (cmd.args.isEmpty()) {
             // :se[t]
             showSettingsDialog();
-        } else if (cmd.args == "ic" || cmd.args == "ignorecase") {
+        } else if (cmd.args == _("ic") || cmd.args == _("ignorecase")) {
             // :set nc
             setActionChecked(Find::Constants::CASE_SENSITIVE, false);
-        } else if (cmd.args == "noic" || cmd.args == "noignorecase") {
+        } else if (cmd.args == _("noic") || cmd.args == _("noignorecase")) {
             // :set noic
             setActionChecked(Find::Constants::CASE_SENSITIVE, true);
         } else {
             *handled = false; // Let the handler see it as well.
         }
-    } else if (cmd.matches("n", "next")) {
+    } else if (cmd.matches(_("n"), _("next"))) {
         // :n[ext]
         switchToFile(currentFile() + cmd.count);
-    } else if (cmd.matches("prev", "previous") || cmd.matches("N", "Next")) {
+    } else if (cmd.matches(_("prev"), _("previous")) || cmd.matches(_("N"), _("Next"))) {
         // :prev[ious], :N[ext]
         switchToFile(currentFile() - cmd.count);
-    } else if (cmd.matches("bn", "bnext")) {
+    } else if (cmd.matches(_("bn"), _("bnext"))) {
         // :bn[ext]
         switchToFile(currentFile() + cmd.count);
-    } else if (cmd.matches("bp", "bprevious") || cmd.matches("bN", "bNext")) {
+    } else if (cmd.matches(_("bp"), _("bprevious")) || cmd.matches(_("bN"), _("bNext"))) {
         // :bp[revious], :bN[ext]
         switchToFile(currentFile() - cmd.count);
-    } else if (cmd.matches("on", "only")) {
+    } else if (cmd.matches(_("on"), _("only"))) {
         // :on[ly]
         //triggerAction(Core::Constants::REMOVE_ALL_SPLITS);
         triggerAction(Core::Constants::REMOVE_CURRENT_SPLIT);
-    } else if (cmd.cmd == "AS") {
+    } else if (cmd.cmd == _("AS")) {
         triggerAction(Core::Constants::SPLIT);
         triggerAction(CppTools::Constants::SWITCH_HEADER_SOURCE);
-    } else if (cmd.cmd == "AV") {
+    } else if (cmd.cmd == _("AV")) {
         triggerAction(Core::Constants::SPLIT_SIDE_BY_SIDE);
         triggerAction(CppTools::Constants::SWITCH_HEADER_SOURCE);
     } else {
@@ -1614,7 +1869,7 @@ void FakeVimPluginPrivate::moveToMatchingParenthesis(bool *moved, bool *forward,
     }
 }
 
-void FakeVimPluginPrivate::indentRegion(int beginLine, int endLine,
+void FakeVimPluginPrivate::indentRegion(int beginBlock, int endBlock,
       QChar typedChar)
 {
     FakeVimHandler *handler = qobject_cast<FakeVimHandler *>(sender());
@@ -1632,14 +1887,14 @@ void FakeVimPluginPrivate::indentRegion(int beginLine, int endLine,
             ? TabSettings::SpacesOnlyTabPolicy : TabSettings::TabsOnlyTabPolicy;
 
     QTextDocument *doc = bt->document();
-    QTextBlock startBlock = doc->findBlockByNumber(beginLine);
+    QTextBlock startBlock = doc->findBlockByNumber(beginBlock);
 
     // Record line lenghts for mark adjustments
-    QVector<int> lineLengths(endLine - beginLine + 1);
+    QVector<int> lineLengths(endBlock - beginBlock + 1);
     QTextBlock block = startBlock;
 
-    for (int i = beginLine; i <= endLine; ++i) {
-        lineLengths[i - beginLine] = block.text().length();
+    for (int i = beginBlock; i <= endBlock; ++i) {
+        lineLengths[i - beginBlock] = block.text().length();
         if (typedChar == 0 && block.text().simplified().isEmpty()) {
             // clear empty lines
             QTextCursor cursor(block);
@@ -1657,11 +1912,17 @@ void FakeVimPluginPrivate::quitFakeVim()
     theFakeVimSetting(ConfigUseFakeVim)->setValue(false);
 }
 
-void FakeVimPluginPrivate::showCommandBuffer(const QString &contents, int cursorPos)
+void FakeVimPluginPrivate::resetCommandBuffer()
+{
+    showCommandBuffer(QString(), -1, -1, 0, 0);
+}
+
+void FakeVimPluginPrivate::showCommandBuffer(const QString &contents,
+    int cursorPos, int anchorPos, int messageLevel, QObject *eventFilter)
 {
     //qDebug() << "SHOW COMMAND BUFFER" << contents;
     if (MiniBuffer *w = qobject_cast<MiniBuffer *>(m_statusBar->widget()))
-        w->setContents(contents, cursorPos);
+        w->setContents(contents, cursorPos, anchorPos, messageLevel, eventFilter);
 }
 
 void FakeVimPluginPrivate::showExtraInformation(const QString &text)
@@ -1671,14 +1932,22 @@ void FakeVimPluginPrivate::showExtraInformation(const QString &text)
         QMessageBox::information(handler->widget(), tr("FakeVim Information"), text);
 }
 
-void FakeVimPluginPrivate::changeSelection
-    (const QList<QTextEdit::ExtraSelection> &selection)
+void FakeVimPluginPrivate::changeSelection(const QList<QTextEdit::ExtraSelection> &selection)
 {
     if (FakeVimHandler *handler = qobject_cast<FakeVimHandler *>(sender()))
         if (BaseTextEditorWidget *bt = qobject_cast<BaseTextEditorWidget *>(handler->widget()))
             bt->setExtraSelections(BaseTextEditorWidget::FakeVimSelection, selection);
 }
 
+void FakeVimPluginPrivate::highlightMatches(const QString &needle)
+{
+    IEditor *editor = EditorManager::currentEditor();
+    QWidget *w = editor->widget();
+    Find::IFindSupport *find = Aggregation::query<Find::IFindSupport>(w);
+    if (find != 0)
+        find->highlightAll(needle, Find::FindRegularExpression | Find::FindCaseSensitively);
+}
+
 int FakeVimPluginPrivate::currentFile() const
 {
     OpenEditorsModel *model = EditorManager::instance()->openedEditorsModel();
@@ -1754,6 +2023,48 @@ void FakeVimPlugin::extensionsInitialized()
     addAutoReleasedObject(d->m_statusBar);
 }
 
+#ifdef WITH_TESTS
+void FakeVimPlugin::setupTest(QString *title, FakeVimHandler **handler, QWidget **edit)
+{
+    *title = QString::fromLatin1("test.cpp");
+    Core::IEditor *iedit = Core::EditorManager::openEditorWithContents(Core::Id(), title);
+    Core::EditorManager::activateEditor(iedit);
+    *edit = iedit->widget();
+    *handler = d->m_editorToHandler.value(iedit, 0);
+    (*handler)->handleCommand(_("set startofline"));
+
+//    *handler = 0;
+//    m_statusMessage.clear();
+//    m_statusData.clear();
+//    m_infoMessage.clear();
+//    if (m_textedit) {
+//        m_textedit->setPlainText(lines);
+//        QTextCursor tc = m_textedit->textCursor();
+//        tc.movePosition(QTextCursor::Start, QTextCursor::MoveAnchor);
+//        m_textedit->setTextCursor(tc);
+//        m_textedit->setPlainText(lines);
+//        *handler = new FakeVimHandler(m_textedit);
+//    } else {
+//        m_plaintextedit->setPlainText(lines);
+//        QTextCursor tc = m_plaintextedit->textCursor();
+//        tc.movePosition(QTextCursor::Start, QTextCursor::MoveAnchor);
+//        m_plaintextedit->setTextCursor(tc);
+//        m_plaintextedit->setPlainText(lines);
+//        *handler = new FakeVimHandler(m_plaintextedit);
+//    }
+
+//    QObject::connect(*handler, SIGNAL(commandBufferChanged(QString,int)),
+//        this, SLOT(changeStatusMessage(QString,int)));
+//    QObject::connect(*handler, SIGNAL(extraInformationChanged(QString)),
+//        this, SLOT(changeExtraInformation(QString)));
+//    QObject::connect(*handler, SIGNAL(statusDataChanged(QString)),
+//        this, SLOT(changeStatusData(QString)));
+
+//    QCOMPARE(EDITOR(toPlainText()), lines);
+    (*handler)->handleCommand(_("set iskeyword=@,48-57,_,192-255,a-z,A-Z"));
+}
+#endif
+
 } // namespace Internal
 } // namespace FakeVim
 
