@@ -1,48 +1,46 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "searchresulttreemodel.h"
 #include "searchresulttreeitems.h"
 #include "searchresulttreeitemroles.h"
+#include "searchresultcolor.h"
 
-#include <QtGui/QApplication>
-#include <QtGui/QFont>
-#include <QtGui/QFontMetrics>
-#include <QtGui/QColor>
-#include <QtGui/QPalette>
-#include <QtGui/QTextDocument>
-#include <QtGui/QTextCursor>
-#include <QtCore/QDir>
-#include <QtCore/QDebug>
+#include <QApplication>
+#include <QFont>
+#include <QFontMetrics>
+#include <QColor>
+#include <QPalette>
+#include <QTextDocument>
+#include <QTextCursor>
+#include <QDir>
+#include <QDebug>
 
 using namespace Find;
 using namespace Find::Internal;
@@ -51,6 +49,7 @@ SearchResultTreeModel::SearchResultTreeModel(QObject *parent)
     : QAbstractItemModel(parent)
     , m_currentParent(0)
     , m_showReplaceUI(false)
+    , m_editorFontIsUsed(false)
 {
     m_rootItem = new SearchResultTreeItem;
     m_textEditorFont = QFont(QLatin1String("Courier"));
@@ -66,10 +65,11 @@ void SearchResultTreeModel::setShowReplaceUI(bool show)
     m_showReplaceUI = show;
 }
 
-void SearchResultTreeModel::setTextEditorFont(const QFont &font)
+void SearchResultTreeModel::setTextEditorFont(const QFont &font, const SearchResultColor color)
 {
     layoutAboutToBeChanged();
     m_textEditorFont = font;
+    m_color = color;
     layoutChanged();
 }
 
@@ -79,9 +79,8 @@ Qt::ItemFlags SearchResultTreeModel::flags(const QModelIndex &idx) const
 
     if (idx.isValid()) {
         if (const SearchResultTreeItem *item = treeItemAtIndex(idx)) {
-            if (item->isUserCheckable()) {
+            if (item->isUserCheckable())
                 flags |= Qt::ItemIsUserCheckable;
-            }
         }
     }
 
@@ -161,10 +160,12 @@ QVariant SearchResultTreeModel::data(const QModelIndex &idx, int role) const
     QVariant result;
 
     if (role == Qt::SizeHintRole) {
-        // TODO we should not use editor font height if that is not used by any item
-        const int appFontHeight = QApplication::fontMetrics().height();
-        const int editorFontHeight = QFontMetrics(m_textEditorFont).height();
-        result = QSize(0, qMax(appFontHeight, editorFontHeight));
+        int height = QApplication::fontMetrics().height();
+        if (m_editorFontIsUsed) {
+            const int editorFontHeight = QFontMetrics(m_textEditorFont).height();
+            height = qMax(height, editorFontHeight);
+        }
+        result = QSize(0, height);
     } else {
         result = data(treeItemAtIndex(idx), role);
     }
@@ -200,13 +201,12 @@ bool SearchResultTreeModel::setCheckState(const QModelIndex &idx, Qt::CheckState
                     SearchResultTreeItem *child = parent->childAt(i);
                     if (!child->isUserCheckable())
                         continue;
-                    if (child->checkState() == Qt::Checked) {
+                    if (child->checkState() == Qt::Checked)
                         hasChecked = true;
-                    } else if (child->checkState() == Qt::Unchecked) {
+                    else if (child->checkState() == Qt::Unchecked)
                         hasUnchecked = true;
-                    } else if (child->checkState() == Qt::PartiallyChecked) {
+                    else if (child->checkState() == Qt::PartiallyChecked)
                         hasChecked = hasUnchecked = true;
-                    }
                 }
                 if (hasChecked && hasUnchecked)
                     parent->setCheckState(Qt::PartiallyChecked);
@@ -251,6 +251,14 @@ QVariant SearchResultTreeModel::data(const SearchResultTreeItem *row, int role)
         else
             result = QVariant();
         break;
+    case Qt::TextColorRole:
+        if (row->item.useTextEditorFont)
+            result = m_color.textForeground;
+        break;
+    case Qt::BackgroundRole:
+        if (row->item.useTextEditorFont)
+            result = m_color.textBackground;
+        break;
     case ItemDataRoles::ResultLineRole:
     case Qt::DisplayRole:
         result = row->item.text;
@@ -264,6 +272,14 @@ QVariant SearchResultTreeModel::data(const SearchResultTreeItem *row, int role)
     case ItemDataRoles::ResultIconRole:
         result = row->item.icon;
         break;
+    case ItemDataRoles::ResultHighlightBackgroundColor:
+        if (row->item.useTextEditorFont)
+            result = m_color.highlightBackground;
+        break;
+    case ItemDataRoles::ResultHighlightForegroundColor:
+        if (row->item.useTextEditorFont)
+            result = m_color.highlightForeground;
+        break;
     case ItemDataRoles::SearchTermStartRole:
         result = row->item.textMarkPos;
         break;
@@ -273,13 +289,6 @@ QVariant SearchResultTreeModel::data(const SearchResultTreeItem *row, int role)
     case ItemDataRoles::IsGeneratedRole:
         result = row->isGenerated();
         break;
-// TODO this looks stupid in case of symbol tree, is it necessary?
-//    case Qt::BackgroundRole:
-//        if (row->parent() && row->parent()->parent())
-//            result = QVariant();
-//        else
-//            result = QApplication::palette().base().color().darker(105);
-//        break;
     default:
         result = QVariant();
         break;
@@ -393,6 +402,7 @@ QList<QModelIndex> SearchResultTreeModel::addResults(const QList<SearchResultIte
     qStableSort(sortedItems.begin(), sortedItems.end(), lessThanByPath);
     QList<SearchResultItem> itemSet;
     foreach (const SearchResultItem &item, sortedItems) {
+        m_editorFontIsUsed |= item.useTextEditorFont;
         if (!m_currentParent || (m_currentPath != item.path)) {
             // first add all the items from before
             if (!itemSet.isEmpty()) {
@@ -416,9 +426,11 @@ QList<QModelIndex> SearchResultTreeModel::addResults(const QList<SearchResultIte
 
 void SearchResultTreeModel::clear()
 {
+    beginResetModel();
     m_currentParent = NULL;
     m_rootItem->clearChildren();
-    reset();
+    m_editorFontIsUsed = false;
+    endResetModel();
 }
 
 QModelIndex SearchResultTreeModel::nextIndex(const QModelIndex &idx, bool *wrapped) const
@@ -500,61 +512,3 @@ QModelIndex SearchResultTreeModel::prev(const QModelIndex &idx, bool includeGene
     } while (value != idx && !includeGenerated && treeItemAtIndex(value)->isGenerated());
     return value;
 }
-
-QModelIndex SearchResultTreeModel::find(const QRegExp &expr, const QModelIndex &index,
-                                        QTextDocument::FindFlags flags, bool *wrapped)
-{
-    QModelIndex resultIndex;
-    QModelIndex currentIndex = index;
-    bool backward = (flags & QTextDocument::FindBackward);
-    if (wrapped)
-        *wrapped = false;
-    bool anyWrapped = false;
-    bool stepWrapped = false;
-
-    do {
-        anyWrapped |= stepWrapped; // update wrapped state if we actually stepped to next/prev item
-        if (currentIndex.isValid()) {
-            const QString &text = data(currentIndex, ItemDataRoles::ResultLineRole).toString();
-            if (expr.indexIn(text) != -1)
-                resultIndex = currentIndex;
-        }
-        if (backward)
-            currentIndex = prev(currentIndex, true, &stepWrapped);
-        else
-            currentIndex = next(currentIndex, true, &stepWrapped);
-    } while (!resultIndex.isValid() && currentIndex.isValid() && currentIndex != index);
-    if (resultIndex.isValid() && wrapped)
-        *wrapped = anyWrapped;
-    return resultIndex;
-}
-
-QModelIndex SearchResultTreeModel::find(const QString &term, const QModelIndex &index,
-                                        QTextDocument::FindFlags flags, bool *wrapped)
-{
-    QModelIndex resultIndex;
-    QModelIndex currentIndex = index;
-    bool backward = (flags & QTextDocument::FindBackward);
-    flags = (flags & (~QTextDocument::FindBackward)); // backward is handled by us ourselves
-    if (wrapped)
-        *wrapped = false;
-    bool anyWrapped = false;
-    bool stepWrapped = false;
-
-    do {
-        anyWrapped |= stepWrapped; // update wrapped state if we actually stepped to next/prev item
-        if (currentIndex.isValid()) {
-            const QString &text = data(currentIndex, ItemDataRoles::ResultLineRole).toString();
-            QTextDocument doc(text);
-            if (!doc.find(term, 0, flags).isNull())
-                resultIndex = currentIndex;
-        }
-        if (backward)
-            currentIndex = prev(currentIndex, true, &stepWrapped);
-        else
-            currentIndex = next(currentIndex, true, &stepWrapped);
-    } while (!resultIndex.isValid() && currentIndex.isValid() && currentIndex != index);
-    if (resultIndex.isValid() && wrapped)
-        *wrapped = anyWrapped;
-    return resultIndex;
-}
