@@ -1,34 +1,31 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "projectnodes.h"
 
@@ -37,13 +34,16 @@
 
 #include <coreplugin/mimedatabase.h>
 #include <coreplugin/fileiconprovider.h>
+#include <coreplugin/icore.h>
+#include <coreplugin/iversioncontrol.h>
+#include <coreplugin/vcsmanager.h>
 #include <utils/qtcassert.h>
 
-#include <QtCore/QFileInfo>
-#include <QtCore/QDir>
-#include <QtGui/QApplication>
-#include <QtGui/QIcon>
-#include <QtGui/QStyle>
+#include <QFileInfo>
+#include <QDir>
+#include <QApplication>
+#include <QIcon>
+#include <QStyle>
 
 using namespace ProjectExplorer;
 
@@ -73,6 +73,38 @@ Node::Node(NodeType nodeType,
 
 }
 
+void Node::emitNodeSortKeyAboutToChange()
+{
+    if (ProjectNode *project = projectNode()) {
+        foreach (NodesWatcher *watcher, project->watchers())
+            emit watcher->nodeSortKeyAboutToChange(this);
+    }
+}
+
+void Node::emitNodeSortKeyChanged()
+{
+    if (ProjectNode *project = projectNode()) {
+        foreach (NodesWatcher *watcher, project->watchers())
+            emit watcher->nodeSortKeyChanged();
+    }
+}
+
+/*!
+ * \brief The path of the file representing this node.
+ *
+ * This method does not emit any signals, that has to be done by the calling class!
+ */
+void Node::setPath(const QString &path)
+{
+    if (m_path == path)
+        return;
+
+    emitNodeSortKeyAboutToChange();
+    m_path = path;
+    emitNodeSortKeyChanged();
+    emitNodeUpdated();
+}
+
 NodeType Node::nodeType() const
 {
     return m_nodeType;
@@ -102,6 +134,31 @@ QString Node::path() const
     return m_path;
 }
 
+int Node::line() const
+{
+    return -1;
+}
+
+QString Node::displayName() const
+{
+    return QFileInfo(path()).fileName();
+}
+
+QString Node::vcsTopic() const
+{
+    return QString();
+}
+
+QString Node::tooltip() const
+{
+    return QDir::toNativeSeparators(path());
+}
+
+bool Node::isEnabled() const
+{
+    return parentFolderNode()->isEnabled();
+}
+
 void Node::setNodeType(NodeType type)
 {
     m_nodeType = type;
@@ -112,16 +169,18 @@ void Node::setProjectNode(ProjectNode *project)
     m_projectNode = project;
 }
 
+void Node::emitNodeUpdated()
+{
+    if (ProjectNode *node = projectNode())
+        foreach (NodesWatcher *watcher, node->watchers())
+            emit watcher->nodeUpdated(this);
+}
+
 void Node::setParentFolderNode(FolderNode *parentFolder)
 {
     m_folderNode = parentFolder;
 }
 
-void Node::setPath(const QString &path)
-{
-    m_path = path;
-}
-
 /*!
   \class ProjectExplorer::FileNode
 
@@ -159,8 +218,8 @@ bool FileNode::isGenerated() const
 
   \sa ProjectExplorer::FileNode, ProjectExplorer::ProjectNode
 */
-FolderNode::FolderNode(const QString &folderPath)  :
-    Node(FolderNodeType, folderPath),
+FolderNode::FolderNode(const QString &folderPath, NodeType nodeType)  :
+    Node(nodeType, folderPath),
     m_displayName(QDir::toNativeSeparators(folderPath))
 {
 }
@@ -212,7 +271,12 @@ void FolderNode::accept(NodesVisitor *visitor)
 
 void FolderNode::setDisplayName(const QString &name)
 {
+    if (m_displayName == name)
+        return;
+    emitNodeSortKeyAboutToChange();
     m_displayName = name;
+    emitNodeSortKeyChanged();
+    emitNodeUpdated();
 }
 
 void FolderNode::setIcon(const QIcon &icon)
@@ -220,6 +284,49 @@ void FolderNode::setIcon(const QIcon &icon)
     m_icon = icon;
 }
 
+FileNode *FolderNode::findFile(const QString &path)
+{
+    foreach (FileNode *n, fileNodes()) {
+        if (n->path() == path)
+            return n;
+    }
+    return 0;
+}
+
+FolderNode *FolderNode::findSubFolder(const QString &path)
+{
+    foreach (FolderNode *n, subFolderNodes()) {
+        if (n->path() == path)
+            return n;
+    }
+    return 0;
+}
+
+/*!
+  \class ProjectExplorer::VirtualFolderNode
+
+  In-memory presentation of a virtual folder.
+  Note that the node itself + all children (files and folders) are "managed" by the owning project.
+  A virtual folder does not correspond to a actual folder on the file system. See for example the
+  sources, headers and forms folder the qt4projectmanager creates
+  VirtualFolderNodes are always sorted before FolderNodes and are sorted according to their priority.
+
+  \sa ProjectExplorer::FileNode, ProjectExplorer::ProjectNode
+*/
+VirtualFolderNode::VirtualFolderNode(const QString &folderPath, int priority)
+    : FolderNode(folderPath, VirtualFolderNodeType), m_priority(priority)
+{
+}
+
+VirtualFolderNode::~VirtualFolderNode()
+{
+}
+
+int VirtualFolderNode::priority() const
+{
+    return m_priority;
+}
+
 /*!
   \class ProjectExplorer::ProjectNode
 
@@ -242,6 +349,16 @@ ProjectNode::ProjectNode(const QString &projectFilePath)
     setDisplayName(QFileInfo(projectFilePath).fileName());
 }
 
+QString ProjectNode::vcsTopic() const {
+    const QString dir = QFileInfo(path()).absolutePath();
+
+    if (Core::IVersionControl *const vc =
+            Core::ICore::vcsManager()->findVersionControlForDirectory(dir))
+        return vc->vcsTopic(dir);
+
+    return QString();
+}
+
 QList<ProjectNode*> ProjectNode::subProjectNodes() const
 {
     return m_subProjectNodes;
@@ -300,8 +417,8 @@ void ProjectNode::registerWatcher(NodesWatcher *watcher)
 {
     if (!watcher)
         return;
-    connect(watcher, SIGNAL(destroyed(QObject *)),
-            this, SLOT(watcherDestroyed(QObject *)));
+    connect(watcher, SIGNAL(destroyed(QObject*)),
+            this, SLOT(watcherDestroyed(QObject*)));
     m_watchers.append(watcher);
     foreach (ProjectNode *subProject, m_subProjectNodes)
         subProject->registerWatcher(watcher);
@@ -349,10 +466,8 @@ void ProjectNode::addProjectNodes(const QList<ProjectNode*> &subProjects)
             m_subFolderNodes.append(project);
             m_subProjectNodes.append(project);
         }
-        qSort(m_subFolderNodes.begin(), m_subFolderNodes.end(),
-              sortNodesByPath);
-        qSort(m_subProjectNodes.begin(), m_subProjectNodes.end(),
-              sortNodesByPath);
+        qSort(m_subFolderNodes.begin(), m_subFolderNodes.end());
+        qSort(m_subProjectNodes.begin(), m_subProjectNodes.end());
 
         foreach (NodesWatcher *watcher, m_watchers)
             emit watcher->foldersAdded();
@@ -371,7 +486,7 @@ void ProjectNode::removeProjectNodes(const QList<ProjectNode*> &subProjects)
         QList<FolderNode*> toRemove;
         foreach (ProjectNode *projectNode, subProjects)
             toRemove << projectNode;
-        qSort(toRemove.begin(), toRemove.end(), sortNodesByPath);
+        qSort(toRemove.begin(), toRemove.end());
 
         foreach (NodesWatcher *watcher, m_watchers)
             emit watcher->foldersAboutToBeRemoved(this, toRemove);
@@ -380,12 +495,12 @@ void ProjectNode::removeProjectNodes(const QList<ProjectNode*> &subProjects)
         QList<FolderNode*>::iterator folderIter = m_subFolderNodes.begin();
         QList<ProjectNode*>::iterator projectIter = m_subProjectNodes.begin();
         for (; toRemoveIter != toRemove.constEnd(); ++toRemoveIter) {
-            while ((*projectIter)->path() != (*toRemoveIter)->path()) {
+            while (*projectIter != *toRemoveIter) {
                 ++projectIter;
                 QTC_ASSERT(projectIter != m_subProjectNodes.end(),
                     qDebug("Project to remove is not part of specified folder!"));
             }
-            while ((*folderIter)->path() != (*toRemoveIter)->path()) {
+            while (*folderIter != *toRemoveIter) {
                 ++folderIter;
                 QTC_ASSERT(folderIter != m_subFolderNodes.end(),
                     qDebug("Project to remove is not part of specified folder!"));
@@ -421,22 +536,17 @@ void ProjectNode::addFolderNodes(const QList<FolderNode*> &subFolders, FolderNod
             folder->setProjectNode(this);
 
             // Find the correct place to insert
-            if (parentFolder->m_subFolderNodes.count() == 0 || sortNodesByPath(parentFolder->m_subFolderNodes.last(), folder)) {
+            if (parentFolder->m_subFolderNodes.count() == 0
+                    || parentFolder->m_subFolderNodes.last() < folder) {
                 // empty list or greater then last node
                 parentFolder->m_subFolderNodes.append(folder);
             } else {
                 // Binary Search for insertion point
-                int l = 0;
-                int r = parentFolder->m_subFolderNodes.count();
-                while (l != r) {
-                    int i = (l + r) / 2;
-                    if (sortNodesByPath(folder, parentFolder->m_subFolderNodes.at(i))) {
-                        r = i;
-                    } else {
-                        l = i + 1;
-                    }
-                }
-                parentFolder->m_subFolderNodes.insert(l, folder);
+                QList<FolderNode*>::iterator it
+                        = qLowerBound(parentFolder->m_subFolderNodes.begin(),
+                                      parentFolder->m_subFolderNodes.end(),
+                                      folder);
+                parentFolder->m_subFolderNodes.insert(it, folder);
             }
 
             // project nodes have to be added via addProjectNodes
@@ -464,7 +574,7 @@ void ProjectNode::removeFolderNodes(const QList<FolderNode*> &subFolders,
         const bool emitSignals = (parentFolder->projectNode() == this);
 
         QList<FolderNode*> toRemove = subFolders;
-        qSort(toRemove.begin(), toRemove.end(), sortNodesByPath);
+        qSort(toRemove.begin(), toRemove.end());
 
         if (emitSignals)
             foreach (NodesWatcher *watcher, m_watchers)
@@ -475,7 +585,7 @@ void ProjectNode::removeFolderNodes(const QList<FolderNode*> &subFolders,
         for (; toRemoveIter != toRemove.constEnd(); ++toRemoveIter) {
             QTC_ASSERT((*toRemoveIter)->nodeType() != ProjectNodeType,
                 qDebug("project nodes have to be removed via removeProjectNodes"));
-            while ((*folderIter)->path() != (*toRemoveIter)->path()) {
+            while (*folderIter != *toRemoveIter) {
                 ++folderIter;
                 QTC_ASSERT(folderIter != parentFolder->m_subFolderNodes.end(),
                     qDebug("Folder to remove is not part of specified folder!"));
@@ -514,22 +624,16 @@ void ProjectNode::addFileNodes(const QList<FileNode*> &files, FolderNode *folder
             file->setParentFolderNode(folder);
             file->setProjectNode(this);
             // Now find the correct place to insert file
-            if (folder->m_fileNodes.count() == 0 || sortNodesByPath(folder->m_fileNodes.last(), file)) {
+            if (folder->m_fileNodes.count() == 0
+                    || folder->m_fileNodes.last() < file) {
                 // empty list or greater then last node
                 folder->m_fileNodes.append(file);
             } else {
-                // Binary Search for insertion point
-                int l = 0;
-                int r = folder->m_fileNodes.count();
-                while (l != r) {
-                    int i = (l + r) / 2;
-                    if (sortNodesByPath(file, folder->m_fileNodes.at(i))) {
-                        r = i;
-                    } else {
-                        l = i + 1;
-                    }
-                }
-                folder->m_fileNodes.insert(l, file);
+                QList<FileNode *>::iterator it
+                        = qLowerBound(folder->m_fileNodes.begin(),
+                                      folder->m_fileNodes.end(),
+                                      file);
+                folder->m_fileNodes.insert(it, file);
             }
         }
 
@@ -554,7 +658,7 @@ void ProjectNode::removeFileNodes(const QList<FileNode*> &files, FolderNode *fol
         const bool emitSignals = (folder->projectNode() == this);
 
         QList<FileNode*> toRemove = files;
-        qSort(toRemove.begin(), toRemove.end(), sortNodesByPath);
+        qSort(toRemove.begin(), toRemove.end());
 
         if (emitSignals)
             foreach (NodesWatcher *watcher, m_watchers)
@@ -563,7 +667,7 @@ void ProjectNode::removeFileNodes(const QList<FileNode*> &files, FolderNode *fol
         QList<FileNode*>::const_iterator toRemoveIter = toRemove.constBegin();
         QList<FileNode*>::iterator filesIter = folder->m_fileNodes.begin();
         for (; toRemoveIter != toRemove.constEnd(); ++toRemoveIter) {
-            while ((*filesIter)->path() != (*toRemoveIter)->path()) {
+            while (*filesIter != *toRemoveIter) {
                 ++filesIter;
                 QTC_ASSERT(filesIter != folder->m_fileNodes.end(),
                     qDebug("File to remove is not part of specified folder!"));
@@ -584,19 +688,13 @@ void ProjectNode::watcherDestroyed(QObject *watcher)
     unregisterWatcher(static_cast<NodesWatcher*>(watcher));
 }
 
-/*!
-  \brief Sort pointers to FileNodes
-*/
-bool ProjectNode::sortNodesByPath(Node *n1, Node *n2) {
-    return n1->path() < n2->path();
-}
 
 /*!
   \class ProjectExplorer::SessionNode
 */
 
-SessionNode::SessionNode(const QString &sessionPath, QObject *parentObject)
-        : FolderNode(sessionPath)
+SessionNode::SessionNode(QObject *parentObject)
+    : FolderNode(QLatin1String("session"))
 {
     setParent(parentObject);
     setNodeType(SessionNodeType);
@@ -668,6 +766,9 @@ void SessionNode::addProjectNodes(const QList<ProjectNode*> &projectNodes)
             m_projectNodes.append(project);
         }
 
+        qSort(m_subFolderNodes);
+        qSort(m_projectNodes);
+
         foreach (NodesWatcher *watcher, m_watchers)
             emit watcher->foldersAdded();
    }
@@ -680,6 +781,8 @@ void SessionNode::removeProjectNodes(const QList<ProjectNode*> &projectNodes)
         foreach (ProjectNode *projectNode, projectNodes)
             toRemove << projectNode;
 
+        qSort(toRemove);
+
         foreach (NodesWatcher *watcher, m_watchers)
             emit watcher->foldersAboutToBeRemoved(this, toRemove);
 
@@ -687,12 +790,12 @@ void SessionNode::removeProjectNodes(const QList<ProjectNode*> &projectNodes)
         QList<FolderNode*>::iterator folderIter = m_subFolderNodes.begin();
         QList<ProjectNode*>::iterator projectIter = m_projectNodes.begin();
         for (; toRemoveIter != toRemove.constEnd(); ++toRemoveIter) {
-            while ((*projectIter)->path() != (*toRemoveIter)->path()) {
+            while (*projectIter != *toRemoveIter) {
                 ++projectIter;
                 QTC_ASSERT(projectIter != m_projectNodes.end(),
                     qDebug("Project to remove is not part of specified folder!"));
             }
-            while ((*folderIter)->path() != (*toRemoveIter)->path()) {
+            while (*folderIter != *toRemoveIter) {
                 ++folderIter;
                 QTC_ASSERT(folderIter != m_subFolderNodes.end(),
                     qDebug("Project to remove is not part of specified folder!"));
@@ -725,7 +828,7 @@ void SessionNode::watcherDestroyed(QObject *watcher)
   by calling ProjectNode::unregisterWatcher and
   SessionNode::unregisterWatcher().
 
-  The NodesWatcher is similar to the Observer in the  
+  The NodesWatcher is similar to the Observer in the
   well-known Observer pattern (Booch et al).
 
   \sa ProjectExplorer::Node
@@ -754,7 +857,8 @@ FileType typeForFileName(const Core::MimeDatabase *db, const QFileInfo &file)
         return ResourceType;
     if (typeName == QLatin1String(Constants::FORM_MIMETYPE))
         return FormType;
-    if (typeName == QLatin1String(Constants::QML_MIMETYPE))
+    if (mt.subClassesOf().contains(QLatin1String(Constants::QML_MIMETYPE))
+            || typeName == QLatin1String(Constants::QML_MIMETYPE))
         return QMLType;
     return UnknownFileType;
 }
