@@ -29,7 +29,6 @@
 
 #include "nodemetainfo.h"
 #include "model.h"
-#include "widgetqueryview.h"
 #include "invalidargumentexception.h"
 
 #include "metainfo.h"
@@ -37,18 +36,17 @@
 #include <rewriterview.h>
 #include <propertyparser.h>
 
+#include <QDir>
 #include <QSharedData>
 #include <QDebug>
 #include <QIcon>
-#include <QDeclarativeContext>
-#include <QDeclarativeEngine>
-#include <QDeclarativeComponent>
 
 #include <qmljs/qmljsdocument.h>
 #include <qmljs/qmljscontext.h>
 #include <qmljs/qmljsbind.h>
 #include <qmljs/qmljsscopechain.h>
 #include <qmljs/parser/qmljsast_p.h>
+#include <qmljs/qmljsmodelmanagerinterface.h>
 #include <languageutils/fakemetaobject.h>
 
 namespace QmlDesigner {
@@ -87,17 +85,66 @@ using namespace QmlJS;
 
 typedef QPair<QString, QString> PropertyInfo;
 
+QList<PropertyInfo> getObjectTypes(const ObjectValue *ov, const ContextPtr &context, bool local = false);
+
+static QString resolveTypeName(const ASTPropertyReference *ref, const ContextPtr &context,  QList<PropertyInfo> &dotProperties)
+{
+    QString type = QLatin1String("unknown");
+
+    if (!ref->ast()->memberType.isEmpty()) {
+        type = ref->ast()->memberType.toString();
+
+        if (type == QLatin1String("alias")) {
+            const Value *value = context->lookupReference(ref);
+
+            if (!value)
+                return type;
+
+            if (const ASTObjectValue * astObjectValue = value->asAstObjectValue()) {
+                if (astObjectValue->typeName())
+                    type = astObjectValue->typeName()->name.toString();
+            } else if (const ObjectValue * objectValue = value->asObjectValue()) {
+                type = objectValue->className();
+                dotProperties = getObjectTypes(objectValue, context);
+            } else if (value->asColorValue()) {
+                type = QLatin1String("color");
+            } else if (value->asUrlValue()) {
+                type = QLatin1String("url");
+            } else if (value->asStringValue()) {
+                type = QLatin1String("string");
+            } else if (value->asRealValue()) {
+                type = QLatin1String("real");
+            } else if (value->asIntValue()) {
+                type = QLatin1String("int");
+            } else if (value->asBooleanValue()) {
+                type = QLatin1String("boolean");
+            }
+        }
+    }
+
+    return type;
+}
+
 class PropertyMemberProcessor : public MemberProcessor
 {
 public:
+    PropertyMemberProcessor(const ContextPtr &context) : m_context(context)
+    {}
     virtual bool processProperty(const QString &name, const Value *value)
     {
         const ASTPropertyReference *ref = value_cast<ASTPropertyReference>(value);
         if (ref) {
-            QString type = "unknown";
-            if (!ref->ast()->memberType.isEmpty())
-                type = ref->ast()->memberType.toString();
+            QList<PropertyInfo> dotProperties;
+            const QString type = resolveTypeName(ref, m_context, dotProperties);
             m_properties.append(qMakePair(name, type));
+            if (!dotProperties.isEmpty()) {
+                foreach (const PropertyInfo &propertyInfo, dotProperties) {
+                    QString dotName = propertyInfo.first;
+                    QString type = propertyInfo.second;
+                    dotName = name + '.' + dotName;
+                    m_properties.append(qMakePair(dotName, type));
+                }
+            }
         } else {
             if (const CppComponentValue * ov = value_cast<CppComponentValue>(value)) {
                 QString qualifiedTypeName = ov->moduleName().isEmpty() ? ov->className() : ov->moduleName() + '.' + ov->className();
@@ -106,11 +153,10 @@ public:
                 TypeId typeId;
                 QString typeName = typeId(value);
                 if (typeName == QLatin1String("number")) {
-                    if (value->asRealValue()) {
+                    if (value->asRealValue())
                         typeName = "real";
-                    } else {
-                    typeName = "int";
-                    }
+                    else
+                        typeName = "int";
                 }
                 m_properties.append(qMakePair(name, typeName));
             }
@@ -122,6 +168,7 @@ public:
 
 private:
     QList<PropertyInfo> m_properties;
+    const ContextPtr m_context;
 };
 
 static inline bool isValueType(const QString &type)
@@ -160,19 +207,16 @@ QStringList prototypes(const ObjectValue *ov, const ContextPtr &context, bool ve
                 list << qmlValue->moduleName() + QLatin1Char('.') + qmlValue->className();
             }
         } else {
-            if (versions) {
+            if (versions)
                 list << ov->className() + " -1.-1";
-            } else {
+            else
                 list << ov->className();
-            }
         }
         ov = ov->prototype(context);
     }
     return list;
 }
 
-QList<PropertyInfo> getObjectTypes(const ObjectValue *ov, const ContextPtr &context, bool local = false);
-
 QList<PropertyInfo> getQmlTypes(const CppComponentValue *ov, const ContextPtr &context, bool local = false)
 {
     QList<PropertyInfo> list;
@@ -181,7 +225,7 @@ QList<PropertyInfo> getQmlTypes(const CppComponentValue *ov, const ContextPtr &c
     if (ov->className().isEmpty())
         return list;
 
-    PropertyMemberProcessor processor;
+    PropertyMemberProcessor processor(context);
     ov->processMembers(&processor);
 
     QList<PropertyInfo> newList = processor.properties();
@@ -224,11 +268,10 @@ QList<PropertyInfo> getQmlTypes(const CppComponentValue *ov, const ContextPtr &c
 
         const CppComponentValue * qmlObjectValue = value_cast<CppComponentValue>(prototype);
 
-        if (qmlObjectValue) {
+        if (qmlObjectValue)
             list << getQmlTypes(qmlObjectValue, context);
-        } else {
+        else
             list << getObjectTypes(prototype, context);
-        }
     }
 
     return list;
@@ -240,11 +283,10 @@ QList<PropertyInfo> getTypes(const ObjectValue *ov, const ContextPtr &context, b
 
     const CppComponentValue * qmlObjectValue = value_cast<CppComponentValue>(ov);
 
-    if (qmlObjectValue) {
+    if (qmlObjectValue)
         list << getQmlTypes(qmlObjectValue, context, local);
-    } else {
+    else
         list << getObjectTypes(ov, context, local);
-    }
 
     return list;
 }
@@ -257,7 +299,7 @@ QList<PropertyInfo> getObjectTypes(const ObjectValue *ov, const ContextPtr &cont
     if (ov->className().isEmpty())
         return list;
 
-    PropertyMemberProcessor processor;
+    PropertyMemberProcessor processor(context);
     ov->processMembers(&processor);
 
     list << processor.properties();
@@ -270,11 +312,10 @@ QList<PropertyInfo> getObjectTypes(const ObjectValue *ov, const ContextPtr &cont
 
         const CppComponentValue * qmlObjectValue = value_cast<CppComponentValue>(prototype);
 
-        if (qmlObjectValue) {
+        if (qmlObjectValue)
             list << getQmlTypes(qmlObjectValue, context);
-        } else {
+        else
             list << getObjectTypes(prototype, context);
-        }
     }
 
     return list;
@@ -288,29 +329,10 @@ public:
     ~NodeMetaInfoPrivate() {}
 
     bool isValid() const;
-
-    bool isComponent() const
-    {
-        return m_isComponent;
-    }
-
-    QStringList properties() const
-    {
-        return m_properties;
-    }
-
-    QStringList localProperties() const
-    {
-        return m_localProperties;
-    }
-
-    QString defaultPropertyName() const
-    {
-        if (!m_defaultPropertyName.isEmpty())
-            return m_defaultPropertyName;
-        return QLatin1String("data");
-    }
-
+    bool isFileComponent() const;
+    QStringList properties() const;
+    QStringList localProperties() const;
+    QString defaultPropertyName() const;
     QString propertyType(const QString &propertyName) const;
 
     void setupPrototypes();
@@ -325,39 +347,24 @@ public:
     bool cleverCheckType(const QString &otherType) const;
     QVariant::Type variantTypeId(const QString &properyName) const;
 
-    int majorVersion() const
-    { return m_majorVersion; }
+    int majorVersion() const;
+    int minorVersion() const;
+    QString qualfiedTypeName() const;
+    Model *model() const;
 
-    int minorVersion() const
-    { return m_minorVersion; }
-
-    QString qualfiedTypeName() const
-    { return m_qualfiedTypeName; }
-
-    Model *model() const
-    { return m_model; }
-
-    QString packageName() const;
+    QString cppPackageName() const;
 
     QString componentSource() const;
     QString componentFileName() const;
+    QString importDirectoryPath() const;
 
     static Pointer create(Model *model, const QString &type, int maj = -1, int min = -1);
 
-     QSet<QString> &prototypeCachePositives()
-     {
-         return m_prototypeCachePositives;
-     }
+    QSet<QString> &prototypeCachePositives();
+    QSet<QString> &prototypeCacheNegatives();
 
-     QSet<QString> &prototypeCacheNegatives()
-     {
-         return m_prototypeCacheNegatives;
-     }
+    static void clearCache();
 
-     static void clearCache()
-     {
-         m_nodeMetaInfoCache.clear();
-     }
 
 private:
     NodeMetaInfoPrivate(Model *model, QString type, int maj = -1, int min = -1);
@@ -375,7 +382,7 @@ private:
     int m_majorVersion;
     int m_minorVersion;
     bool m_isValid;
-    bool m_isComponent;
+    bool m_isFileComponent;
     QStringList m_properties;
     QStringList m_propertyTypes;
     QStringList m_localProperties;
@@ -394,6 +401,42 @@ private:
 
 QHash<QString, NodeMetaInfoPrivate::Pointer> NodeMetaInfoPrivate::m_nodeMetaInfoCache;
 
+bool NodeMetaInfoPrivate::isFileComponent() const
+{
+    return m_isFileComponent;
+}
+
+QStringList NodeMetaInfoPrivate::properties() const
+{
+    return m_properties;
+}
+
+QStringList NodeMetaInfoPrivate::localProperties() const
+{
+    return m_localProperties;
+}
+
+QSet<QString> &NodeMetaInfoPrivate::prototypeCachePositives()
+{
+    return m_prototypeCachePositives;
+}
+
+QSet<QString> &NodeMetaInfoPrivate::prototypeCacheNegatives()
+{
+    return m_prototypeCacheNegatives;
+}
+
+void NodeMetaInfoPrivate::clearCache()
+{
+    m_nodeMetaInfoCache.clear();
+}
+
+QString NodeMetaInfoPrivate::defaultPropertyName() const
+{
+    if (!m_defaultPropertyName.isEmpty())
+        return m_defaultPropertyName;
+    return QLatin1String("data");
+}
 
 static inline QString stringIdentifier( const QString &type, int maj, int min)
 {
@@ -404,11 +447,10 @@ NodeMetaInfoPrivate::Pointer NodeMetaInfoPrivate::create(Model *model, const QSt
 {
     if (m_nodeMetaInfoCache.contains(stringIdentifier(type, maj, min))) {
         const Pointer &info = m_nodeMetaInfoCache.value(stringIdentifier(type, maj, min));
-        if (info->model() == model) {
+        if (info->model() == model)
             return info;
-        } else {
+        else
             m_nodeMetaInfoCache.clear();
-        }
     }
 
     Pointer newData(new NodeMetaInfoPrivate(model, type, maj, min));
@@ -424,33 +466,49 @@ NodeMetaInfoPrivate::NodeMetaInfoPrivate() : m_isValid(false)
 
 NodeMetaInfoPrivate::NodeMetaInfoPrivate(Model *model, QString type, int maj, int min) :
                                         m_qualfiedTypeName(type), m_majorVersion(maj),
-                                        m_minorVersion(min), m_isValid(false), m_isComponent(false),
+                                        m_minorVersion(min), m_isValid(false), m_isFileComponent(false),
                                         m_model(model)
 {
     if (context()) {
         const CppComponentValue *objectValue = getCppComponentValue();
         if (objectValue) {
+            if (m_majorVersion == -1 && m_minorVersion == -1) {
+                m_majorVersion = objectValue->componentVersion().majorVersion();
+                m_minorVersion = objectValue->componentVersion().minorVersion();
+            }
             setupPropertyInfo(getTypes(objectValue, context()));
             setupLocalPropertyInfo(getTypes(objectValue, context(), true));
             m_defaultPropertyName = objectValue->defaultPropertyName();
-            setupPrototypes();
             m_isValid = true;
+            setupPrototypes();
         } else {
             const ObjectValue *objectValue = getObjectValue();
             if (objectValue) {
                 const CppComponentValue *qmlValue = value_cast<CppComponentValue>(objectValue);
                 if (qmlValue) {
-                    m_majorVersion = qmlValue->componentVersion().majorVersion();
-                    m_minorVersion = qmlValue->componentVersion().minorVersion();
-                    m_qualfiedTypeName = qmlValue->moduleName() + '.' + qmlValue->className();
+                    if (m_majorVersion == -1 && m_minorVersion == -1) {
+                        m_majorVersion = qmlValue->componentVersion().majorVersion();
+                        m_minorVersion = qmlValue->componentVersion().minorVersion();
+                        m_qualfiedTypeName = qmlValue->moduleName() + '.' + qmlValue->className();
+                    } else if (m_majorVersion == qmlValue->componentVersion().majorVersion() && m_minorVersion == qmlValue->componentVersion().minorVersion()) {
+                        m_qualfiedTypeName = qmlValue->moduleName() + '.' + qmlValue->className();
+                    } else {
+                        return;
+                    }
                 } else {
-                    m_isComponent = true;
+                    m_isFileComponent = true;
+                    const Imports *imports = context()->imports(document());
+                    ImportInfo importInfo = imports->info(lookupNameComponent().last(), context().data());
+                    if (importInfo.isValid() && importInfo.type() == ImportInfo::LibraryImport) {
+                        m_majorVersion = importInfo.version().majorVersion();
+                        m_minorVersion = importInfo.version().minorVersion();
+                    }
                 }
                 setupPropertyInfo(getTypes(objectValue, context()));
                 setupLocalPropertyInfo(getTypes(objectValue, context(), true));
                 m_defaultPropertyName = context()->defaultPropertyName(objectValue);
-                setupPrototypes();
                 m_isValid = true;
+                setupPrototypes();
             }
         }
     }
@@ -480,8 +538,12 @@ const QmlJS::CppComponentValue *NodeMetaInfoPrivate::getCppComponentValue() cons
         if (import.info.path() != module)
             continue;
         const Value *lookupResult = import.object->lookupMember(type, context());
-        if ((value = value_cast<CppComponentValue>(lookupResult)))
-            return value;
+        const CppComponentValue *cppValue = value_cast<CppComponentValue>(lookupResult);
+        if (cppValue
+                && (m_majorVersion == -1 || m_majorVersion == cppValue->componentVersion().majorVersion())
+                && (m_minorVersion == -1 || m_minorVersion == cppValue->componentVersion().minorVersion())
+                )
+            return cppValue;
     }
 
     return 0;
@@ -494,17 +556,15 @@ const QmlJS::ObjectValue *NodeMetaInfoPrivate::getObjectValue() const
 
 QmlJS::ContextPtr NodeMetaInfoPrivate::context() const
 {
-    if (m_model && m_model->rewriterView() && m_model->rewriterView()->scopeChain()) {
+    if (m_model && m_model->rewriterView() && m_model->rewriterView()->scopeChain())
         return m_model->rewriterView()->scopeChain()->context();
-    }
     return QmlJS::ContextPtr(0);
 }
 
 const QmlJS::Document *NodeMetaInfoPrivate::document() const
 {
-    if (m_model && m_model->rewriterView()) {
+    if (m_model && m_model->rewriterView())
         return m_model->rewriterView()->document();
-    }
     return 0;
 }
 
@@ -534,9 +594,8 @@ bool NodeMetaInfoPrivate::isPropertyWritable(const QString &propertyName) const
         const QString rawPropertyName = parts.last();
         const QString objectType = propertyType(objectName);
 
-        if (isValueType(objectType)) {
+        if (isValueType(objectType))
             return true;
-        }
 
         QSharedPointer<NodeMetaInfoPrivate> objectInfo(create(m_model, objectType));
         if (objectInfo->isValid())
@@ -566,9 +625,8 @@ bool NodeMetaInfoPrivate::isPropertyList(const QString &propertyName) const
         const QString rawPropertyName = parts.last();
         const QString objectType = propertyType(objectName);
 
-        if (isValueType(objectType)) {
+        if (isValueType(objectType))
             return false;
-        }
 
         QSharedPointer<NodeMetaInfoPrivate> objectInfo(create(m_model, objectType));
         if (objectInfo->isValid())
@@ -594,9 +652,8 @@ bool NodeMetaInfoPrivate::isPropertyPointer(const QString &propertyName) const
         const QString rawPropertyName = parts.last();
         const QString objectType = propertyType(objectName);
 
-        if (isValueType(objectType)) {
+        if (isValueType(objectType))
             return false;
-        }
 
         QSharedPointer<NodeMetaInfoPrivate> objectInfo(create(m_model, objectType));
         if (objectInfo->isValid())
@@ -622,9 +679,8 @@ bool NodeMetaInfoPrivate::isPropertyEnum(const QString &propertyName) const
         const QString rawPropertyName = parts.last();
         const QString objectType = propertyType(objectName);
 
-        if (isValueType(objectType)) {
+        if (isValueType(objectType))
             return false;
-        }
 
         QSharedPointer<NodeMetaInfoPrivate> objectInfo(create(m_model, objectType));
         if (objectInfo->isValid())
@@ -650,9 +706,8 @@ QString NodeMetaInfoPrivate::propertyEnumScope(const QString &propertyName) cons
         const QString rawPropertyName = parts.last();
         const QString objectType = propertyType(objectName);
 
-        if (isValueType(objectType)) {
+        if (isValueType(objectType))
             return QString();
-        }
 
         QSharedPointer<NodeMetaInfoPrivate> objectInfo(create(m_model, objectType));
         if (objectInfo->isValid())
@@ -672,12 +727,20 @@ QString NodeMetaInfoPrivate::propertyEnumScope(const QString &propertyName) cons
     return QString();
 }
 
+static QString getUnqualifiedName(const QString &name)
+{
+    const QStringList nameComponents = name.split('.');
+    if (nameComponents.size() < 2)
+        return QString();
+    return nameComponents.last();
+}
+
 bool NodeMetaInfoPrivate::cleverCheckType(const QString &otherType) const
 {
     if (otherType == qualfiedTypeName())
             return true;
 
-    if (isComponent())
+    if (isFileComponent())
         return false;
 
     QStringList split = otherType.split('.');
@@ -687,8 +750,8 @@ bool NodeMetaInfoPrivate::cleverCheckType(const QString &otherType) const
         package = split.first();
         typeName = split.at(1);
     }
-    if (packageName() == package)
-        return QString(package + '.' + typeName) == qualfiedTypeName();
+    if (cppPackageName() == package)
+        return QString(package + '.' + typeName) == cppPackageName() + '.' + getUnqualifiedName(qualfiedTypeName());
 
     const CppComponentValue *qmlObjectValue = getCppComponentValue();
     if (!qmlObjectValue)
@@ -739,6 +802,26 @@ QVariant::Type NodeMetaInfoPrivate::variantTypeId(const QString &properyName) co
     return QVariant::nameToType(typeName.toLatin1().data());
 }
 
+int NodeMetaInfoPrivate::majorVersion() const
+{
+    return m_majorVersion;
+}
+
+int NodeMetaInfoPrivate::minorVersion() const
+{
+    return m_minorVersion;
+}
+
+QString NodeMetaInfoPrivate::qualfiedTypeName() const
+{
+    return m_qualfiedTypeName;
+}
+
+Model *NodeMetaInfoPrivate::model() const
+{
+    return m_model;
+}
+
 
 QStringList NodeMetaInfoPrivate::keysForEnum(const QString &enumName) const
 {
@@ -751,9 +834,9 @@ QStringList NodeMetaInfoPrivate::keysForEnum(const QString &enumName) const
     return qmlObjectValue->getEnum(enumName).keys();
 }
 
-QString NodeMetaInfoPrivate::packageName() const
+QString NodeMetaInfoPrivate::cppPackageName() const
 {
-    if (!isComponent()) {
+    if (!isFileComponent()) {
         if (const CppComponentValue *qmlObject = getCppComponentValue())
             return qmlObject->moduleName();
     }
@@ -762,7 +845,7 @@ QString NodeMetaInfoPrivate::packageName() const
 
 QString NodeMetaInfoPrivate::componentSource() const
 {
-    if (isComponent()) {
+    if (isFileComponent()) {
         const ASTObjectValue * astObjectValue = value_cast<ASTObjectValue>(getObjectValue());
         if (astObjectValue)
             return astObjectValue->document()->source().mid(astObjectValue->typeName()->identifierToken.begin(),
@@ -773,7 +856,7 @@ QString NodeMetaInfoPrivate::componentSource() const
 
 QString NodeMetaInfoPrivate::componentFileName() const
 {
-    if (isComponent()) {
+    if (isFileComponent()) {
         const ASTObjectValue * astObjectValue = value_cast<ASTObjectValue>(getObjectValue());
         if (astObjectValue) {
             QString fileName;
@@ -786,6 +869,29 @@ QString NodeMetaInfoPrivate::componentFileName() const
     return QString();
 }
 
+QString NodeMetaInfoPrivate::importDirectoryPath() const
+{
+    ModelManagerInterface *modelManager = ModelManagerInterface::instance();
+
+    if (isValid()) {
+        const Imports *imports = context()->imports(document());
+        ImportInfo importInfo = imports->info(lookupNameComponent().last(), context().data());
+
+        if (importInfo.type() == ImportInfo::DirectoryImport) {
+            return importInfo.path();
+        } else if (importInfo.type() == ImportInfo::LibraryImport) {
+            if (modelManager) {
+                foreach (const QString &importPath, modelManager->importPaths()) {
+                    const QString targetPath = QDir(importPath).filePath(importInfo.path());
+                    if (QDir(targetPath).exists()) {
+                        return targetPath;
+                    }
+                }
+            }
+        }
+    }
+    return QString();
+}
 
 QString NodeMetaInfoPrivate::lookupName() const
 {
@@ -819,17 +925,29 @@ bool NodeMetaInfoPrivate::isValid() const
 QString NodeMetaInfoPrivate::propertyType(const QString &propertyName) const
 {
     if (!m_properties.contains(propertyName))
-        return QString();
+        return QLatin1String("Property does not exist...");
     return m_propertyTypes.at(m_properties.indexOf(propertyName));
 }
 
 void NodeMetaInfoPrivate::setupPrototypes()
 {
     QList<const ObjectValue *> objects;
-    if (m_isComponent)
-        objects = PrototypeIterator(getObjectValue(), context()).all();
+
+    const ObjectValue *ov;
+
+    if (m_isFileComponent)
+        ov = getObjectValue();
     else
-        objects = PrototypeIterator(getCppComponentValue(), context()).all();
+        ov = getCppComponentValue();
+
+    PrototypeIterator prototypeIterator(ov, context());
+
+    objects = prototypeIterator.all();
+
+    if (prototypeIterator.error() != PrototypeIterator::NoError) {
+        m_isValid = false;
+        return;
+    }
 
     foreach (const ObjectValue *ov, objects) {
         TypeDescription description;
@@ -841,13 +959,12 @@ void NodeMetaInfoPrivate::setupPrototypes()
             description.majorVersion = qmlValue->componentVersion().majorVersion();
             LanguageUtils::FakeMetaObject::Export qtquickExport = qmlValue->metaObject()->exportInPackage("QtQuick");
             LanguageUtils::FakeMetaObject::Export cppExport = qmlValue->metaObject()->exportInPackage("<cpp>");
-            if (qtquickExport.isValid()) {
+            if (qtquickExport.isValid())
                 description.className = qtquickExport.package + '.' + qtquickExport.type;
-            } else if (qmlValue->moduleName().isEmpty() && cppExport.isValid()) {
+            else if (qmlValue->moduleName().isEmpty() && cppExport.isValid())
                 description.className = cppExport.package + '.' + cppExport.type;
-            } else if (!qmlValue->moduleName().isEmpty()) {
+            else if (!qmlValue->moduleName().isEmpty())
                 description.className = qmlValue->moduleName() + '.' + description.className;
-            }
             m_prototypes.append(description);
         } else {
             if (context()->lookupType(document(), QStringList() << ov->className()))
@@ -864,7 +981,7 @@ QList<TypeDescription> NodeMetaInfoPrivate::prototypes() const
 
 const QmlJS::CppComponentValue *NodeMetaInfoPrivate::getNearestCppComponentValue() const
 {
-    if (m_isComponent)
+    if (m_isFileComponent)
         return findQmlPrototype(getObjectValue(), context());
     return getCppComponentValue();
 }
@@ -910,9 +1027,9 @@ bool NodeMetaInfo::isValid() const
     return m_privateData->isValid();
 }
 
-bool NodeMetaInfo::isComponent() const
+bool NodeMetaInfo::isFileComponent() const
 {
-    return m_privateData->isComponent();
+    return m_privateData->isFileComponent();
 }
 
 bool NodeMetaInfo::hasProperty(const QString &propertyName) const
@@ -974,9 +1091,8 @@ QVariant NodeMetaInfo::propertyCastedValue(const QString &propertyName, const QV
 {
 
     QVariant variant = value;
-    if (propertyIsEnumType(propertyName)) {
+    if (propertyIsEnumType(propertyName))
         return variant;
-    }
 
     const QString typeName = propertyTypeName(propertyName);
 
@@ -1043,7 +1159,12 @@ QString NodeMetaInfo::componentSource() const
 
 QString NodeMetaInfo::componentFileName() const
 {
-    return NodeMetaInfo::m_privateData->componentFileName();
+    return m_privateData->componentFileName();
+}
+
+QString NodeMetaInfo::importDirectoryPath() const
+{
+    return m_privateData->importDirectoryPath();
 }
 
 bool NodeMetaInfo::hasCustomParser() const
@@ -1096,4 +1217,11 @@ void NodeMetaInfo::clearCache()
     Internal::NodeMetaInfoPrivate::clearCache();
 }
 
+bool NodeMetaInfo::isPositioner() const
+{
+    if (majorVersion() < 2)
+        return isSubclassOf("<cpp>.QDeclarativeBasePositioner", -1, -1);
+    return isSubclassOf("QtQuick.Positioner", -1, -1);
+}
+
 } // namespace QmlDesigner
