@@ -45,11 +45,13 @@
 #include <qtsupport/qtversionmanager.h>
 #include <qtsupport/qtkitinformation.h>
 #include <qmljs/qmljsmodelmanagerinterface.h>
+#include <qmljstools/qmljsmodelmanager.h>
 #include <utils/fileutils.h>
 #include <projectexplorer/kitinformation.h>
 #include <projectexplorer/kitmanager.h>
 #include <projectexplorer/target.h>
 #include <utils/filesystemwatcher.h>
+#include <qtsupport/qtsupportconstants.h>
 
 #include <QTextStream>
 #include <QDeclarativeComponent>
@@ -57,13 +59,65 @@
 
 namespace QmlProjectManager {
 
+namespace Internal {
+
+class QmlProjectKitMatcher : public ProjectExplorer::KitMatcher
+{
+public:
+    QmlProjectKitMatcher(const QmlProject::QmlImport &import)
+        : import(import)
+    {
+    }
+
+    bool matches(const ProjectExplorer::Kit *k) const
+    {
+        if (!k->isValid())
+            return false;
+
+        ProjectExplorer::IDevice::ConstPtr dev = ProjectExplorer::DeviceKitInformation::device(k);
+        if (dev.isNull() || dev->type() != ProjectExplorer::Constants::DESKTOP_DEVICE_TYPE)
+            return false;
+        QtSupport::BaseQtVersion *version = QtSupport::QtKitInformation::qtVersion(k);
+        if (!version || version->type() != QLatin1String(QtSupport::Constants::DESKTOPQT))
+            return false;
+
+        bool hasViewer;
+        QtSupport::QtVersionNumber minVersion;
+        switch (import) {
+        case QmlProject::UnknownImport:
+            minVersion = QtSupport::QtVersionNumber(4, 7, 0);
+            hasViewer = !version->qmlviewerCommand().isEmpty() || !version->qmlsceneCommand().isEmpty();
+            break;
+        case QmlProject::QtQuick1Import:
+            minVersion = QtSupport::QtVersionNumber(4, 7, 1);
+            hasViewer = !version->qmlviewerCommand().isEmpty();
+            break;
+        case QmlProject::QtQuick2Import:
+            minVersion = QtSupport::QtVersionNumber(5, 0, 0);
+            hasViewer = !version->qmlsceneCommand().isEmpty();
+            break;
+        }
+
+        if (version->qtVersion() >= minVersion
+                && hasViewer)
+            return true;
+
+        return false;
+    }
+private:
+    QmlProject::QmlImport import;
+};
+
+} // namespace Internal
+
 QmlProject::QmlProject(Internal::Manager *manager, const QString &fileName)
     : m_manager(manager),
       m_fileName(fileName),
+      m_defaultImport(UnknownImport),
       m_modelManager(QmlJS::ModelManagerInterface::instance())
 {
     setProjectContext(Core::Context(QmlProjectManager::Constants::PROJECTCONTEXT));
-    setProjectLanguage(Core::Context(QmlProjectManager::Constants::LANG_QML));
+    setProjectLanguages(Core::Context(QmlProjectManager::Constants::LANG_QML));
 
     QFileInfo fileInfo(m_fileName);
     m_projectName = fileInfo.completeBaseName();
@@ -111,6 +165,18 @@ QDir QmlProject::projectDir() const
 QString QmlProject::filesFileName() const
 { return m_fileName; }
 
+static QmlProject::QmlImport detectImport(const QString &qml) {
+    static QRegExp qtQuick1RegExp(QLatin1String("import\\s+QtQuick\\s+1"));
+    static QRegExp qtQuick2RegExp(QLatin1String("import\\s+QtQuick\\s+2"));
+
+    if (qml.contains(qtQuick1RegExp))
+        return QmlProject::QtQuick1Import;
+    else if (qml.contains(qtQuick2RegExp))
+        return QmlProject::QtQuick2Import;
+    else
+        return QmlProject::UnknownImport;
+}
+
 void QmlProject::parseProject(RefreshOptions options)
 {
     Core::MessageManager *messageManager = Core::ICore::messageManager();
@@ -142,11 +208,14 @@ void QmlProject::parseProject(RefreshOptions options)
             QString mainFilePath = m_projectItem.data()->mainFile();
             if (!mainFilePath.isEmpty()) {
                 mainFilePath = projectDir().absoluteFilePath(mainFilePath);
-                if (!QFileInfo(mainFilePath).isReadable()) {
+                Utils::FileReader reader;
+                QString errorMessage;
+                if (!reader.fetch(mainFilePath, &errorMessage)) {
                     messageManager->printToOutputPane(
                                 tr("Warning while loading project file %1.").arg(m_fileName));
-                    messageManager->printToOutputPane(
-                                tr("File '%1' does not exist or is not readable.").arg(mainFilePath), true);
+                    messageManager->printToOutputPane(errorMessage, true);
+                } else {
+                    m_defaultImport = detectImport(QString::fromUtf8(reader.data()));
                 }
             }
         }
@@ -168,22 +237,11 @@ void QmlProject::refresh(RefreshOptions options)
     if (options & Files)
         m_rootNode->refresh();
 
-    QmlJS::ModelManagerInterface::ProjectInfo pinfo(this);
-    pinfo.sourceFiles = files();
-    pinfo.importPaths = customImportPaths();
-    QtSupport::BaseQtVersion *version = 0;
-    if (activeTarget()) {
-        ProjectExplorer::ToolChain *tc = ProjectExplorer::ToolChainKitInformation::toolChain(activeTarget()->kit());
-        version = QtSupport::QtKitInformation::qtVersion(activeTarget()->kit());
-        QtSupport::QmlDumpTool::pathAndEnvironment(this, version, tc, false, &pinfo.qmlDumpPath, &pinfo.qmlDumpEnvironment);
-    }
-    if (version) {
-        pinfo.tryQmlDump = true;
-        pinfo.qtImportsPath = version->qmakeProperty("QT_INSTALL_IMPORTS");
-        pinfo.qtQmlPath = version->qmakeProperty("QT_INSTALL_QML");
-        pinfo.qtVersionString = version->qtVersionString();
-    }
-    m_modelManager->updateProjectInfo(pinfo);
+    QmlJS::ModelManagerInterface::ProjectInfo projectInfo =
+            QmlJSTools::defaultProjectInfoForProject(this);
+    projectInfo.importPaths = customImportPaths();
+
+    m_modelManager->updateProjectInfo(projectInfo);
 }
 
 QStringList QmlProject::convertToAbsoluteFiles(const QStringList &paths) const
@@ -201,11 +259,10 @@ QStringList QmlProject::convertToAbsoluteFiles(const QStringList &paths) const
 QStringList QmlProject::files() const
 {
     QStringList files;
-    if (m_projectItem) {
+    if (m_projectItem)
         files = m_projectItem.data()->files();
-    } else {
+    else
         files = m_files;
-    }
     return files;
 }
 
@@ -245,6 +302,11 @@ void QmlProject::refreshProjectFile()
     refresh(QmlProject::ProjectFile | Files);
 }
 
+QmlProject::QmlImport QmlProject::defaultImport() const
+{
+    return m_defaultImport;
+}
+
 void QmlProject::refreshFiles(const QSet<QString> &/*added*/, const QSet<QString> &removed)
 {
     refresh(Files);
@@ -281,16 +343,26 @@ bool QmlProject::supportsKit(ProjectExplorer::Kit *k, QString *errorMessage) con
         return false;
     }
 
-    // TODO: Limit supported versions?
     QtSupport::BaseQtVersion *version = QtSupport::QtKitInformation::qtVersion(k);
-    if (!version && errorMessage)
-        *errorMessage = tr("No Qt version set in kit.");
-    return version;
-}
+    if (!version) {
+        if (errorMessage)
+            *errorMessage = tr("No Qt version set in kit.");
+        return false;
+    }
 
-QList<ProjectExplorer::BuildConfigWidget*> QmlProject::subConfigWidgets()
-{
-    return QList<ProjectExplorer::BuildConfigWidget*>();
+    if (version->qtVersion() < QtSupport::QtVersionNumber(4, 7, 0)) {
+        if (errorMessage)
+            *errorMessage = tr("Qt version is too old.");
+        return false;
+    }
+
+    if (version->qtVersion() < QtSupport::QtVersionNumber(5, 0, 0)
+            && defaultImport() == QtQuick2Import) {
+        if (errorMessage)
+            *errorMessage = tr("Qt version is too old.");
+        return false;
+    }
+    return true;
 }
 
 ProjectExplorer::ProjectNode *QmlProject::rootProjectNode() const
@@ -308,12 +380,26 @@ bool QmlProject::fromMap(const QVariantMap &map)
     if (!Project::fromMap(map))
         return false;
 
-    ProjectExplorer::Kit *defaultKit = ProjectExplorer::KitManager::instance()->defaultKit();
-    if (!activeTarget() && defaultKit)
-        addTarget(createTarget(defaultKit));
-
+    // refresh first - project information is used e.g. to decide the default RC's
     refresh(Everything);
 
+    if (!activeTarget()) {
+        // find a kit that matches prerequisites (prefer default one)
+        Internal::QmlProjectKitMatcher matcher(defaultImport());
+        ProjectExplorer::KitManager *kitManager = ProjectExplorer::KitManager::instance();
+        QList<ProjectExplorer::Kit*> kits = kitManager->kits(&matcher);
+
+        if (!kits.isEmpty()) {
+            ProjectExplorer::Kit *kit = 0;
+            if (kits.contains(kitManager->defaultKit())) {
+                kit = kitManager->defaultKit();
+            } else {
+                kit = kits.first();
+            }
+            addTarget(createTarget(kit));
+        }
+    }
+
     // addedTarget calls updateEnabled on the runconfigurations
     // which needs to happen after refresh
     foreach (ProjectExplorer::Target *t, targets())
