@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -46,18 +46,21 @@
 #include <coreplugin/messagemanager.h>
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/progressmanager/progressmanager.h>
+#include <coreplugin/documentmanager.h>
+#include <coreplugin/variablemanager.h>
 #include <extensionsystem/pluginmanager.h>
 #include <cpptools/ModelManagerInterface.h>
 #include <qmljs/qmljsmodelmanagerinterface.h>
+#include <qmljstools/qmljsmodelmanager.h>
 #include <projectexplorer/buildtargetinfo.h>
 #include <projectexplorer/deploymentdata.h>
 #include <projectexplorer/toolchain.h>
 #include <projectexplorer/headerpath.h>
 #include <projectexplorer/target.h>
-#include <projectexplorer/buildenvironmentwidget.h>
 #include <projectexplorer/kitinformation.h>
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/projectexplorerconstants.h>
+#include <projectexplorer/projectmacroexpander.h>
 #include <utils/qtcassert.h>
 #include <qtsupport/customexecutablerunconfiguration.h>
 #include <qtsupport/qmldumptool.h>
@@ -67,6 +70,7 @@
 #include <qtsupport/qtsupportconstants.h>
 #include <qtsupport/qtversionmanager.h>
 #include <utils/QtConcurrentTools>
+#include <utils/stringutils.h>
 
 #include <QDebug>
 #include <QDir>
@@ -97,24 +101,6 @@ Qt4BuildConfiguration *enableActiveQt4BuildConfiguration(ProjectExplorer::Target
     return bc;
 }
 
-QString sanitize(const QString &input)
-{
-    QString result;
-    result.reserve(input.size());
-    foreach (const QChar &qc, input) {
-        const char c = qc.toLatin1();
-        if ((c >= 'a' && c <='z')
-                || (c >= 'A' && c <= 'Z')
-                || (c >= '0' && c <= '9')
-                || c == '-'
-                || c == '_')
-            result.append(qc);
-        else
-            result.append(QLatin1Char('_'));
-    }
-    return result;
-}
-
 } // namespace
 
 namespace Qt4ProjectManager {
@@ -359,12 +345,11 @@ Qt4Project::Qt4Project(Qt4Manager *manager, const QString& fileName) :
     m_pendingEvaluateFuturesCount(0),
     m_asyncUpdateState(NoState),
     m_cancelEvaluate(false),
-    m_codeModelCanceled(false),
     m_centralizedFolderWatcher(0),
     m_activeTarget(0)
 {
     setProjectContext(Core::Context(Qt4ProjectManager::Constants::PROJECT_ID));
-    setProjectLanguage(Core::Context(ProjectExplorer::Constants::LANG_CXX));
+    setProjectLanguages(Core::Context(ProjectExplorer::Constants::LANG_CXX));
 
     m_asyncUpdateTimer.setSingleShot(true);
     m_asyncUpdateTimer.setInterval(3000);
@@ -397,6 +382,32 @@ void Qt4Project::updateFileList()
     }
 }
 
+bool Qt4Project::setupTarget(ProjectExplorer::Target *t)
+{
+    QList<BuildConfigurationInfo> infoList
+            = Qt4BuildConfigurationFactory::availableBuildConfigurations(t->kit(), m_fileInfo->fileName());
+    setupTarget(t, infoList);
+    return true;
+}
+
+void Qt4Project::setupTarget(ProjectExplorer::Target *t, const QList<BuildConfigurationInfo> &infoList)
+{
+    // Build Configurations:
+    foreach (const BuildConfigurationInfo &info, infoList) {
+        QString name = info.buildConfig & QtSupport::BaseQtVersion::DebugBuild
+                ? tr("Debug") : tr("Release");
+        Qt4BuildConfiguration *bc
+                = Qt4BuildConfiguration::setup(t, name, name,
+                                               info.buildConfig, info.additionalArguments,
+                                               info.directory, info.importing);
+        t->addBuildConfiguration(bc);
+    }
+
+    // Deploy Configurations:
+    t->updateDefaultDeployConfigurations();
+    // Do not create Run Configurations: Those will be generated later anyway.
+}
+
 bool Qt4Project::fromMap(const QVariantMap &map)
 {
     if (!Project::fromMap(map))
@@ -423,8 +434,8 @@ bool Qt4Project::fromMap(const QVariantMap &map)
     updateCodeModels();
 
     // We have the profile nodes now, so we know the runconfigs!
-    connect(m_nodesWatcher, SIGNAL(kitUpdated(Qt4ProjectManager::Qt4ProFileNode*,bool,bool)),
-            this, SIGNAL(kitUpdated(Qt4ProjectManager::Qt4ProFileNode*,bool,bool)));
+    connect(m_nodesWatcher, SIGNAL(proFileUpdated(Qt4ProjectManager::Qt4ProFileNode*,bool,bool)),
+            this, SIGNAL(proFileUpdated(Qt4ProjectManager::Qt4ProFileNode*,bool,bool)));
 
     // Now we emit update once :)
     m_rootProjectNode->emitProFileUpdatedRecursive();
@@ -454,9 +465,9 @@ bool Qt4Project::equalFileList(const QStringList &a, const QStringList &b)
     QStringList::const_iterator bend = b.constEnd();
 
     while (ait != aend && bit != bend) {
-        if (*ait == QLatin1String("<configuration>"))
+        if (*ait == CPlusPlus::CppModelManagerInterface::configurationFileName())
             ++ait;
-        else if (*bit == QLatin1String("<configuration>"))
+        else if (*bit == CPlusPlus::CppModelManagerInterface::configurationFileName())
             ++bit;
         else if (*ait == *bit)
             ++ait, ++bit;
@@ -514,11 +525,17 @@ void Qt4Project::updateCppCodeModel()
     QStringList allFiles;
     foreach (Qt4ProFileNode *pro, proFiles) {
         ProjectPart::Ptr part(new ProjectPart);
-        part->qtVersion = qtVersionForPart;
+
+        if (pro->variableValue(ConfigVar).contains(QLatin1String("qt")))
+            part->qtVersion = qtVersionForPart;
+        else
+            part->qtVersion = ProjectPart::NoQt;
+
+        const QStringList cxxflags = pro->variableValue(CppFlagsVar);
 
         // part->defines
         if (tc)
-            part->defines = tc->predefinedMacros(pro->variableValue(CppFlagsVar));
+            part->defines = tc->predefinedMacros(cxxflags);
         part->defines += pro->cxxDefines();
 
         // part->includePaths
@@ -526,10 +543,9 @@ void Qt4Project::updateCppCodeModel()
 
         QList<HeaderPath> headers;
         if (tc)
-            headers = tc->systemHeaderPaths(SysRootKitInformation::sysRoot(k)); // todo pass cxxflags?
-        if (qtVersion) {
+            headers = tc->systemHeaderPaths(cxxflags, SysRootKitInformation::sysRoot(k));
+        if (qtVersion)
             headers.append(qtVersion->systemHeaderPathes(k));
-        }
 
         foreach (const HeaderPath &headerPath, headers) {
             if (headerPath.kind() == HeaderPath::FrameworkHeaderPath)
@@ -550,26 +566,21 @@ void Qt4Project::updateCppCodeModel()
         part->precompiledHeaders.append(pro->variableValue(PrecompiledHeaderVar));
 
         // part->language
-        part->language = CPlusPlus::CppModelManagerInterface::CXX;
-        // part->flags
         if (tc)
-            part->cxx11Enabled = tc->compilerFlags(pro->variableValue(CppFlagsVar)) == ToolChain::STD_CXX11;
+            part->language = tc->compilerFlags(cxxflags) == ToolChain::STD_CXX11 ? ProjectPart::CXX11 : ProjectPart::CXX;
+        else
+            part->language = CPlusPlus::CppModelManagerInterface::ProjectPart::CXX11;
 
         part->sourceFiles = pro->variableValue(CppSourceVar);
-        part->sourceFiles += pro->variableValue(CppHeaderVar);
-        part->sourceFiles.prepend(QLatin1String("<configuration>"));
+        part->headerFiles += pro->variableValue(CppHeaderVar);
+        part->headerFiles += pro->uiFiles();
+        part->sourceFiles.prepend(CPlusPlus::CppModelManagerInterface::configurationFileName());
+        part->objcSourceFiles = pro->variableValue(ObjCSourceVar);
         pinfo.appendProjectPart(part);
 
+        allFiles += part->headerFiles;
         allFiles += part->sourceFiles;
-
-        part = ProjectPart::Ptr(new ProjectPart);
-        //  todo objc code?
-        part->language = CPlusPlus::CppModelManagerInterface::OBJC;
-        part->sourceFiles = pro->variableValue(ObjCSourceVar);
-        if (!part->sourceFiles.isEmpty())
-            pinfo.appendProjectPart(part);
-
-        allFiles += part->sourceFiles;
+        allFiles += part->objcSourceFiles;
     }
 
     modelmanager->updateProjectInfo(pinfo);
@@ -582,54 +593,37 @@ void Qt4Project::updateQmlJSCodeModel()
     if (!modelManager)
         return;
 
-    QmlJS::ModelManagerInterface::ProjectInfo projectInfo = modelManager->projectInfo(this);
-    projectInfo.sourceFiles = m_projectFiles->files[QMLType];
+    QmlJS::ModelManagerInterface::ProjectInfo projectInfo =
+            QmlJSTools::defaultProjectInfoForProject(this);
 
     FindQt4ProFiles findQt4ProFiles;
     QList<Qt4ProFileNode *> proFiles = findQt4ProFiles(rootProjectNode());
 
     projectInfo.importPaths.clear();
+
+    bool hasQmlLib = false;
     foreach (Qt4ProFileNode *node, proFiles) {
         projectInfo.importPaths.append(node->variableValue(QmlImportPathVar));
+        if (!hasQmlLib) {
+            QStringList qtLibs = node->variableValue(QtVar);
+            hasQmlLib = qtLibs.contains(QLatin1String("declarative")) ||
+                    qtLibs.contains(QLatin1String("qml")) ||
+                    qtLibs.contains(QLatin1String("quick"));
+            break;
+        }
     }
 
-    bool preferDebugDump = false;
-    projectInfo.tryQmlDump = false;
+    // If the project directory has a pro/pri file that includes a qml or quick or declarative
+    // library then chances of the project being a QML project is quite high.
+    // This assumption fails when there are no QDeclarativeEngine/QDeclarativeView (QtQuick 1)
+    // or QQmlEngine/QQuickView (QtQuick 2) instances.
+    Core::Context pl(ProjectExplorer::Constants::LANG_CXX);
+    if (hasQmlLib)
+        pl.add(ProjectExplorer::Constants::LANG_QMLJS);
+    setProjectLanguages(pl);
 
-    ProjectExplorer::Target *t = activeTarget();
-    ProjectExplorer::Kit *k = t ? t->kit() : ProjectExplorer::KitManager::instance()->defaultKit();
-    QtSupport::BaseQtVersion *qtVersion = QtSupport::QtKitInformation::qtVersion(k);
-
-    if (t) {
-        if (Qt4BuildConfiguration *bc = qobject_cast<Qt4BuildConfiguration *>(t->activeBuildConfiguration()))
-            preferDebugDump = bc->qmakeBuildConfiguration() & QtSupport::BaseQtVersion::DebugBuild;
-    } else {
-        if (qtVersion)
-            preferDebugDump = qtVersion->defaultBuildConfig() & QtSupport::BaseQtVersion::DebugBuild;
-    }
-    if (qtVersion && qtVersion->isValid()) {
-        projectInfo.tryQmlDump = qtVersion->type() == QLatin1String(QtSupport::Constants::DESKTOPQT)
-                || qtVersion->type() == QLatin1String(QtSupport::Constants::SIMULATORQT);
-        projectInfo.qtQmlPath = qtVersion->qmakeProperty("QT_INSTALL_QML");
-        if (!projectInfo.qtQmlPath.isEmpty())
-            projectInfo.importPaths += projectInfo.qtQmlPath;
-        projectInfo.qtImportsPath = qtVersion->qmakeProperty("QT_INSTALL_IMPORTS");
-        if (!projectInfo.qtImportsPath.isEmpty())
-            projectInfo.importPaths += projectInfo.qtImportsPath;
-        projectInfo.qtVersionString = qtVersion->qtVersionString();
-    }
     projectInfo.importPaths.removeDuplicates();
 
-    if (projectInfo.tryQmlDump) {
-        QtSupport::QmlDumpTool::pathAndEnvironment(this, qtVersion,
-                                                   ToolChainKitInformation::toolChain(k),
-                                                   preferDebugDump, &projectInfo.qmlDumpPath,
-                                                   &projectInfo.qmlDumpEnvironment);
-    } else {
-        projectInfo.qmlDumpPath.clear();
-        projectInfo.qmlDumpEnvironment.clear();
-    }
-
     modelManager->updateProjectInfo(projectInfo);
 }
 
@@ -715,7 +709,6 @@ void Qt4Project::scheduleAsyncUpdate(Qt4ProFileNode *node)
 
         // Cancel running code model update
         m_codeModelFuture.cancel();
-        m_codeModelCanceled = true;
     } else if (m_asyncUpdateState == AsyncUpdateInProgress) {
         // A update is in progress
         // And this slot only gets called if a file changed on disc
@@ -764,7 +757,6 @@ void Qt4Project::scheduleAsyncUpdate()
 
     // Cancel running code model update
     m_codeModelFuture.cancel();
-    m_codeModelCanceled = true;
 }
 
 
@@ -792,6 +784,7 @@ void Qt4Project::decrementPendingEvaluateFutures()
         // We are done!
         if (debug)
             qDebug()<<"  reporting finished";
+
         m_asyncUpdateFutureInterface->reportFinished();
         delete m_asyncUpdateFutureInterface;
         m_asyncUpdateFutureInterface = 0;
@@ -846,7 +839,7 @@ void Qt4Project::asyncUpdate()
     } else {
         if (debug)
             qDebug()<<"  partial update,"<<m_partialEvaluate.size()<<"nodes to update";
-        foreach(Qt4ProFileNode *node, m_partialEvaluate)
+        foreach (Qt4ProFileNode *node, m_partialEvaluate)
             node->asyncUpdate();
     }
 
@@ -903,10 +896,10 @@ QStringList Qt4Project::files(FilesMode fileMode) const
 // Find the folder that contains a file a certain type (recurse down)
 static FolderNode *folderOf(FolderNode *in, FileType fileType, const QString &fileName)
 {
-    foreach(FileNode *fn, in->fileNodes())
+    foreach (FileNode *fn, in->fileNodes())
         if (fn->fileType() == fileType && fn->path() == fileName)
             return in;
-    foreach(FolderNode *folder, in->subFolderNodes())
+    foreach (FolderNode *folder, in->subFolderNodes())
         if (FolderNode *pn = folderOf(folder, fileType, fileName))
             return pn;
     return 0;
@@ -937,10 +930,10 @@ void Qt4Project::proFileParseError(const QString &errorMessage)
     Core::ICore::messageManager()->printToOutputPanePopup(errorMessage);
 }
 
-QtSupport::ProFileReader *Qt4Project::createProFileReader(Qt4ProFileNode *qt4ProFileNode, Qt4BuildConfiguration *bc)
+QtSupport::ProFileReader *Qt4Project::createProFileReader(const Qt4ProFileNode *qt4ProFileNode, Qt4BuildConfiguration *bc)
 {
     if (!m_qmakeGlobals) {
-        m_qmakeGlobals = new QMakeGlobals;
+        m_qmakeGlobals = new ProFileGlobals;
         m_qmakeGlobalsRefCnt = 0;
 
         Kit *k;
@@ -952,12 +945,10 @@ QtSupport::ProFileReader *Qt4Project::createProFileReader(Qt4ProFileNode *qt4Pro
         if (bc) {
             k = bc->target()->kit();
             env = bc->environment();
-            if (bc->qmakeStep()) {
+            if (bc->qmakeStep())
                 qmakeArgs = bc->qmakeStep()->parserArguments();
-                m_qmakeGlobals->qmakespec = m_qmakeGlobals->xqmakespec = bc->qmakeStep()->mkspec().toString();
-            } else {
+            else
                 qmakeArgs = bc->configCommandLineArguments();
-            }
         } else {
             k = KitManager::instance()->defaultKit();
         }
@@ -977,7 +968,7 @@ QtSupport::ProFileReader *Qt4Project::createProFileReader(Qt4ProFileNode *qt4Pro
         for (; eit != eend; ++eit)
             m_qmakeGlobals->environment.insert(env.key(eit), env.value(eit));
 
-        m_qmakeGlobals->setCommandLineArguments(qmakeArgs);
+        m_qmakeGlobals->setCommandLineArguments(m_rootProjectNode->buildDir(), qmakeArgs);
 
         QtSupport::ProFileCacheManager::instance()->incRefCount();
     }
@@ -990,7 +981,7 @@ QtSupport::ProFileReader *Qt4Project::createProFileReader(Qt4ProFileNode *qt4Pro
     return reader;
 }
 
-QMakeGlobals *Qt4Project::qmakeGlobals()
+ProFileGlobals *Qt4Project::qmakeGlobals()
 {
     return m_qmakeGlobals;
 }
@@ -1036,13 +1027,6 @@ bool Qt4Project::parseInProgress(const QString &proFilePath) const
     return node && node->parseInProgress();
 }
 
-QList<BuildConfigWidget*> Qt4Project::subConfigWidgets()
-{
-    QList<BuildConfigWidget*> subWidgets;
-    subWidgets << new BuildEnvironmentWidget;
-    return subWidgets;
-}
-
 void Qt4Project::collectAllfProFiles(QList<Qt4ProFileNode *> &list, Qt4ProFileNode *node)
 {
     list.append(node);
@@ -1152,7 +1136,7 @@ void Qt4Project::notifyChanged(const QString &name)
     if (files(Qt4Project::ExcludeGeneratedFiles).contains(name)) {
         QList<Qt4ProFileNode *> list;
         findProFile(name, rootQt4ProjectNode(), list);
-        foreach(Qt4ProFileNode *node, list) {
+        foreach (Qt4ProFileNode *node, list) {
             QtSupport::ProFileCacheManager::instance()->discardFile(name);
             node->update();
         }
@@ -1248,9 +1232,8 @@ void CentralizedFolderWatcher::unwatchFolders(const QList<QString> &folders, Qt4
         if (!folder.endsWith(slash))
             folder.append(slash);
         m_map.remove(folder, node);
-        if (!m_map.contains(folder)) {
+        if (!m_map.contains(folder))
             m_watcher.removePath(folder);
-        }
 
         // Figure out which recursive directories we can remove
         // this might not scale. I'm pretty sure it doesn't
@@ -1295,7 +1278,7 @@ void CentralizedFolderWatcher::folderChanged(const QString &folder)
 
 void CentralizedFolderWatcher::onTimer()
 {
-    foreach(const QString &folder, m_changedFolders)
+    foreach (const QString &folder, m_changedFolders)
         delayedFolderChanged(folder);
     m_changedFolders.clear();
 }
@@ -1396,19 +1379,21 @@ QString Qt4Project::disabledReasonForRunConfiguration(const QString &proFilePath
             .arg(QFileInfo(proFilePath).fileName());
 }
 
-QString Qt4Project::shadowBuildDirectory(const QString &profilePath, const Kit *k, const QString &suffix)
+QString Qt4Project::shadowBuildDirectory(const QString &proFilePath, const Kit *k, const QString &suffix)
 {
-    if (profilePath.isEmpty())
+    if (proFilePath.isEmpty())
         return QString();
-    QFileInfo info(profilePath);
+    QFileInfo info(proFilePath);
 
     QtSupport::BaseQtVersion *version = QtSupport::QtKitInformation::qtVersion(k);
     if (version && !version->supportsShadowBuilds())
         return info.absolutePath();
 
-    QString base = QDir::cleanPath(projectDirectory(profilePath) + QLatin1String("/../")
-                                   + info.baseName() + QLatin1String("-build-"));
-    return base + buildNameFor(k) + QLatin1String("-") + sanitize(suffix);
+    const QString projectName = QFileInfo(proFilePath).completeBaseName();
+    ProjectExplorer::ProjectExpander expander(proFilePath, projectName, k, suffix);
+    QDir projectDir = QDir(projectDirectory(proFilePath));
+    QString buildPath = Utils::expandMacros(Core::DocumentManager::buildDirectory(), &expander);
+    return QDir::cleanPath(projectDir.absoluteFilePath(buildPath));
 }
 
 QString Qt4Project::buildNameFor(const Kit *k)
@@ -1425,22 +1410,7 @@ Target *Qt4Project::createTarget(Kit *k, const QList<BuildConfigurationInfo> &in
         return 0;
 
     Target *t = new Target(this, k);
-
-    // Build Configurations:
-    foreach (const BuildConfigurationInfo &info, infoList) {
-        QString name = info.buildConfig & QtSupport::BaseQtVersion::DebugBuild
-                ? tr("Debug") : tr("Release");
-        Qt4BuildConfiguration *bc
-                = Qt4BuildConfiguration::setup(t, name, name,
-                                               info.buildConfig, info.additionalArguments,
-                                               info.directory, info.importing);
-        t->addBuildConfiguration(bc);
-    }
-
-    // Deploy Configurations:
-    t->updateDefaultDeployConfigurations();
-    // Do not create Run Configurations: Those will be generated later anyway.
-
+    setupTarget(t, infoList);
     return t;
 }
 
@@ -1530,7 +1500,7 @@ void Qt4Project::collectLibraryData(const Qt4ProFileNode *node, DeploymentData &
         break;
     }
     case ProjectExplorer::Abi::MacOS:
-        if (config.contains("lib_bundle")) {
+        if (config.contains(QLatin1String("lib_bundle"))) {
             ti.workingDir.append(QLatin1Char('/')).append(ti.target)
                     .append(QLatin1String(".framework"));
         } else {
