@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 ** Author: Milian Wolff, KDAB (milian.wolff@kdab.com)
 **
@@ -55,7 +55,7 @@ public:
     virtual QVariantMap defaults() const;
     virtual void fromMap(const QVariantMap &map);
 
-    virtual QString id() const;
+    virtual Core::Id id() const;
     virtual QString displayName() const;
 
 signals:
@@ -173,6 +173,7 @@ public:
     QVariantMap toMap() const;
     QVariantMap defaults() const;
     void fromMap(const QVariantMap &map);
+    virtual AbstractAnalyzerSubConfig *clone();
 
     /*
      * Global memcheck settings
@@ -230,6 +231,7 @@ public:
     QVariantMap toMap() const;
     QVariantMap defaults() const;
     void fromMap(const QVariantMap &map);
+    virtual AbstractAnalyzerSubConfig *clone();
 
     /**
      * Per-project memcheck settings, saves a diff to the global suppression files list
