@@ -1,38 +1,35 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (info@qt.nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at info@qt.nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "cgi.h"
 
-#include <QtCore/QByteArray>
+#include <QByteArray>
 
 const char cgi_chars[] = "0123456789abcdef"; // RFC 1738 suggests lower-case to be optimal
 
@@ -44,13 +41,13 @@ QString CGI::encodeURL(const QString &rawText)
 
     QByteArray::const_iterator it = utf.constBegin();
     while (it != utf.constEnd()) {
-        char ch = *it;
+        const char ch = *it;
         if (('A' <= ch && ch <= 'Z')
             || ('a' <= ch && ch <= 'z')
             || ('0' <= ch && ch <= '9'))
-            enc.append(*it);
+            enc.append(QLatin1Char(ch));
         else if (ch == ' ')
-            enc.append('+');
+            enc.append(QLatin1Char('+'));
         else {
             switch (ch) {
             case '-': case '_':
@@ -58,14 +55,14 @@ QString CGI::encodeURL(const QString &rawText)
             case '.': case '!':
             case '~': case '*':
             case '\'':
-                enc.append(ch);
+                enc.append(QLatin1Char(ch));
                 break;
             default:
                 ushort c1 = (*it & 0xF0) >> 4;
                 ushort c2 = (*it & 0x0F);
-                enc.append('%');
-                enc.append(QChar(*(cgi_chars + c1)));
-                enc.append(QChar(*(cgi_chars + c2)));
+                enc.append(QLatin1Char('%'));
+                enc.append(QLatin1Char(*(cgi_chars + c1)));
+                enc.append(QLatin1Char(*(cgi_chars + c2)));
                 break;
             }
         }
@@ -164,13 +161,13 @@ inline const char *unicodeToHTML(ushort unicode_char)
     case 0x00BD: return "frac12";  // (189 ) vulgar fraction one half = fraction one half
     case 0x00BE: return "frac34";  // (190 ) vulgar fraction three quarters = fraction three quarters
     case 0x00BF: return "iquest";  // (191 ) inverted question mark = turned question mark
-    case 0x00C0: return "Agrave";  // (192 ) capital letter A with grave = capital letter 
+    case 0x00C0: return "Agrave";  // (192 ) capital letter A with grave = capital letter
     case 0x00C1: return "Aacute";  // (193 ) capital letter A with acute
     case 0x00C2: return "Acirc";   // (194 ) capital letter A with circumflex
     case 0x00C3: return "Atilde";  // (195 ) capital letter A with tilde
     case 0x00C4: return "Auml";    // (196 ) capital letter A with diaeresis
-    case 0x00C5: return "Aring";   // (197 ) capital letter A with ring above = capital letter 
-    case 0x00C6: return "AElig";   // (198 ) capital letter AE =  capital ligature 
+    case 0x00C5: return "Aring";   // (197 ) capital letter A with ring above = capital letter
+    case 0x00C6: return "AElig";   // (198 ) capital letter AE =  capital ligature
     case 0x00C7: return "Ccedil";  // (199 ) capital letter C with cedilla
     case 0x00C8: return "Egrave";  // (200 ) capital letter E with grave
     case 0x00C9: return "Eacute";  // (201 ) capital letter E with acute
@@ -188,7 +185,7 @@ inline const char *unicodeToHTML(ushort unicode_char)
     case 0x00D5: return "Otilde";  // (213 ) capital letter O with tilde
     case 0x00D6: return "Ouml";    // (214 ) capital letter O with diaeresis
     case 0x00D7: return "times";   // (215 ) multiplication sign
-    case 0x00D8: return "Oslash";  // (216 ) capital letter O with stroke = capital letter 
+    case 0x00D8: return "Oslash";  // (216 ) capital letter O with stroke = capital letter
     case 0x00D9: return "Ugrave";  // (217 ) capital letter U with grave
     case 0x00DA: return "Uacute";  // (218 ) capital letter U with acute
     case 0x00DB: return "Ucirc";   // (219 ) capital letter U with circumflex
@@ -196,13 +193,13 @@ inline const char *unicodeToHTML(ushort unicode_char)
     case 0x00DD: return "Yacute";  // (221 ) capital letter Y with acute
     case 0x00DE: return "THORN";   // (222 ) capital letter THORN
     case 0x00DF: return "szlig";   // (223 ) small letter sharp s = ess-zed
-    case 0x00E0: return "agrave";  // (224 ) small letter a with grave = small letter 
+    case 0x00E0: return "agrave";  // (224 ) small letter a with grave = small letter
     case 0x00E1: return "aacute";  // (225 ) small letter a with acute
     case 0x00E2: return "acirc";   // (226 ) small letter a with circumflex
     case 0x00E3: return "atilde";  // (227 ) small letter a with tilde
     case 0x00E4: return "auml";    // (228 ) small letter a with diaeresis
-    case 0x00E5: return "aring";   // (229 ) small letter a with ring above = small letter 
-    case 0x00E6: return "aelig";   // (230 ) small letter ae = small letter 
+    case 0x00E5: return "aring";   // (229 ) small letter a with ring above = small letter
+    case 0x00E6: return "aelig";   // (230 ) small letter ae = small letter
     case 0x00E7: return "ccedil";  // (231 ) small letter c with cedilla
     case 0x00E8: return "egrave";  // (232 ) small letter e with grave
     case 0x00E9: return "eacute";  // (233 ) small letter e with acute
@@ -220,7 +217,7 @@ inline const char *unicodeToHTML(ushort unicode_char)
     case 0x00F5: return "otilde";  // (245 ) small letter o with tilde
     case 0x00F6: return "ouml";    // (246 ) small letter o with diaeresis
     case 0x00F7: return "divide";  // (247 ) division sign
-    case 0x00F8: return "oslash";  // (248 ) small letter o with stroke = small letter 
+    case 0x00F8: return "oslash";  // (248 ) small letter o with stroke = small letter
     case 0x00F9: return "ugrave";  // (249 ) small letter u with grave
     case 0x00FA: return "uacute";  // (250 ) small letter u with acute
     case 0x00FB: return "ucirc";   // (251 ) small letter u with circumflex
@@ -403,22 +400,22 @@ QString CGI::encodeHTML(const QString &rawText, int conversionFlags)
     while (it != rawText.constEnd()) {
         const char *html = unicodeToHTML((*it).unicode());
         if (html) {
-            enc.append('&');
-            enc.append(html);
-            enc.append(';');
+            enc.append(QLatin1Char('&'));
+            enc.append(QLatin1String(html));
+            enc.append(QLatin1Char(';'));
         } else if ((conversionFlags & CGI::LineBreaks)
                    && ((*it).toLatin1() == '\n')) {
-                enc.append("<BR>\n");
+                enc.append(QLatin1String("<BR>\n"));
         } else if ((conversionFlags & CGI::Spaces)
                    && ((*it).toLatin1() == ' ')) {
-                enc.append("&nbsp;");
+                enc.append(QLatin1String("&nbsp;"));
         } else if ((conversionFlags & CGI::Tabs)
                    && ((*it).toLatin1() == '\t')) {
-                enc.append("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
+                enc.append(QLatin1String("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"));
         } else if ((*it).unicode() > 0x00FF) {
-            enc.append("&#");
+            enc.append(QLatin1String("&#"));
             enc.append(QString::number((*it).unicode()));
-            enc.append(';');
+            enc.append(QLatin1Char(';'));
         } else {
             enc.append(*it);
         }
